
; ********************************************************************************************
; * FJRX_2FP: Receiver control based on:
; *
; * - DF1FO's Atmel-Programm V3.3  (www.mydarc.de/df1fo)
; * - DC7GB's Atmel-Programm V3.3+ (www.mydarc.de/dc7gb)
; *
; * Adapted for recreational (Flying Pig) mode of Foxhunting by W.J.(Pim) Niessen, PA2PIM
; * (www.planet.nl/~niess153)
; *
; * Follows the Flying Pig QRP club principle: 
; * "No Dues, No Rules, Just Fun - and if we don't like it we fix it!" 
; * But let me be very clear: The work of DF1FO and also DF7GB is much appreciated!
; * It's just that IARU/ARDF foxhunting is not my kettle of fish.
; *
; * So I changed the code with the following "project guide lines":
; * -	Leave orignal DF1FO principle and touch and feel of Man Machine Interface in place as
; *		much as possible. I.e.it should still be a very nice ARDF Foxhunting receiver.
; * -	Add VFO mode, AGC on etc
; * -	Easy switching between DF1FO/ARDF and FP (Flying Pig) mode.
; * -	All changes preferrably in software. 
; *		Hardware changes if needed must be very very limited.
; * -	Modified software must run in standard DF1FO receiver and provide original
; * 	functionality
; *
; ********************************************************************************************
;
;	12-08-07	0.1: Renamed files to FP1724-2, Attempt to reduce birdies by updating LCD
;					 only if data has changed (OUTLCD Routine)
;	21-08-07	0.6: Changed memory symbol from 5 to 4 rows high. Defined gain symbols. Moved
;					 * and " " for automatic gain reduction to memgainsymbol. LCD character
;					 programming in sequence, uses less code. Started removing stsi macro to 
;					 free up program memory.
;	22-08-07	0.7: Connected FlyingPigMode switch to PB5 via 1K2. PB5 Low during power up sets 
;					 FPMode and selects StrModeHuntFpGain as standard display. Using 8184 of 8192
;					 bytes
;	04-09-07	3.0: In Free Mode Mem symbol appears if VFO = one of the 6 memories. 
;					 Removed a lot of the STSI macro's to gain program space. Restructured 
;					 EEPNEW routine to reclaim even more program space
;				3.1: Added NextMemIndex, PrevMemIndex routines and MemIndex variable. 
;					 Still need some more work on when to update MemIndex
;				3.2: Changed menu's into Englisch
;	05-09-07	3.3: Various clean ups to make sure EEPROM gets re-init as a result of 
;					 investigating (jumps) errors in Attenuator table..
;				3.4: Fixed errors in EEPNEW (lds and ldi are different instructions!)
;				3.5: CopyFreqToFreqVfo routine added (Needed for FP mode MEM<>VFO copy)
;	06-09-07	3.6: Added StartFPMode2 routine to update parameters when entering mode 2 in
;					 FP mode
;				3.7: Added StrModeFreqInpFp for frequency display/change in FP mode 2
;				3.8: Reshuffeld variables definitions to ensure that displayed variables
;					 have an address <= $FF
;	09-09-07	3.9: Split CopyFreqtoFreqVfo routine and let it call universal byte
;					 copy routine, CopyBytes
;				4.0: Prepared for changing VFO freq in FP Freq entry mode. Removed some more
;					 stsi macro's to get program space
;				4.1: Split of 2 routine from GETFRQ to allow proper symbol in FP mode 2.
;					 Removed a few more stsi macro's (In interrupt routine)
;				4.2: Changed mem symbol for VFO mode from blank to <>
;				4.3: Changed string StrModeFreqInpFp
;				4.4: Moved GainMemSymbol to init of software
;	10-09-07	4.5: Undid changes of 4.4, Changed StrModeFreqInpFp to contain MemSymbol, 
;					 CreateGainMemSymbol writes now character 6 and 7, Found some more program
;					 space by using SetRegister (1) and ClearRegister (0) for setting switches
;					 instead of LDI R16, XX, STS VAR,R16, Saves 2 bytes for every replacement
;				4.6: Corrected bugs in creating LCD char 6 and 7
;	17-09-07	4.7: MEM channel symbol displayed on power up, was vfo symbol
;	22-09-07	4.8: Split of CreateGainMemSym from GetFrq
;				4.9: Modified StartFPMode2
;				4.D: Modified GETFRQ to cope with FRQ0
;				4.F: Modified PUTPF and GETPF to cope with Pfox0
;				5.0: modified PrevFreqIndex, NextFreqIndex
;				5.1: Moved various copy routines as code into StartFP2Mode
;				5.2: Moved FrqBinVfo usage in StartFpMode2 to FRQ0 Was the same Data
;				5.3: Modified PrevFreqIndex to cope with FRQX=0 (VFO)
;	23-09-07	5.4: Undid changes to PrevFreqIndex in 5.3
;				5.5: Modified NextFreqIndex, PrevFreqIndex to update only FRQX
;				5.6: Took VfoIsMem out of StartFPMode2
;				5.7: Took FP code out around M30:, replaced CreateMemSym by VfoIsMem
;				5.8: Toggle FP and ARDF mode by pressing FPswitch during power on
;	27-09-07	5.8a:Used ClearRegister to clear flags. Saves program space
;				5.p: Debugged PrevFreqindex and PrevMemIndex
;				5.q: Changed Mem 1...6 to 144.100 ...144.600
;	01-10-07	5.s: Split of UpdateLCD from OUTLCD. Call UpdateLCD to force LCD update
;	02-10-07	5.t: Gainmode
;				5.u: Inclim changed breq > brge, Longpress = 75
;	04-10-07	5.v: MainA..MainB updated to handle Manual Gain, AutoGainReduction and 
;					 Full AGC, removed last of the stsi macro's
;	05-10-07	5.w: VFO to memory routines part 1
;				5.x: VFO to memory routines part 2
;				5.z: Added M to second line of StrModeFreqInpFp to indicate Memory is displayed on this line
;	06-10-07	6.0: Debugged VFO to Memory routines
;				6.1: More debugging
;				6.2: And...more nextd FRQX to MemIndex
;				6.3: Put AutoOn symbol back in StrModeHuntArdfTimer and StrModeHuntArdfStopw
;				6.4: Send P in CW when memory is programmed
;				6.5: Reset dynamic stepsize if direction of encoder changes (MAIN94..Main97)
;				6.6: Direct to mode 2 when FP switch pressed and "shalter" active.
;	08-10-07	6.7: Made 144-146 or 144-148 a conditional assembly option (.set FBER68)
;					 Used available space to select DF1FO/ARDF or PA2PIM/FP mode as default
;					 Current selected mode toggles by pressing FP switch during power up.
;				6.8: Debugging
;				7.2: Some more debugging
;				7.3: Moved FPMode between EEP2 and EEPEND, ChngArdfFPMode is only routine
;					 changing FPMode byte, Program uses solely copy of it in FPModeReg.
;					 Changed Power on string to "FJRX 2FP" to indicate PB5 (FP) switch.
;	09-10-07	7.4: Debugged udabs for FPmode, mode 3 push and turn
;	22-10-07	7.6: Removed FP switch, 8060 bytes, needs functionality check
;				7.7: Bit 7 in SWIFLG is encoder button long press processed by DebEncPush.
;					 DEB3..DEB9 removed, needs functionality check. 8094 bytes
;				7.8: Defined modes with .equ statements. Updated Main2..Main29 with them.(8106 bytes)
;				7.9:
;				7.A: Restructured mode processing, moved once called routines in main stream (8088 bytes)
;	23-10-07	7.B: Continued with 7.A (8094 Bytes)
;	24-10-07	7.C: Continued with 7.A (8076 Bytes)
;	25-10-07	7.D: Continued with 7.A (8040 Bytes)
;				7.E: Selection of mode processing using lookup table (ModeOpsTab) and icall instruction (8004 bytes)
;	26-10-07	7.F: Wrote some ModeOPs code
;	28-10-07	8.0: And some more
;	02-11-07	8.1: Debugged debounce changes from 7.7
;	04-11-07	8.2: Cleaned up modetab, showtab and modeopstab
;				8.3: Restart code if end of modetab is ever reached
;	07-11-07	8.5: Debugged modeops selection code
;				8.6:pUT BACK OLD DEBNCE
;	12-11-07	8.9: Started to build state machine
;	19-11-07	8.B: Continued with building state machine
;	20-11-07	8.C: Continued with building state machine
;	20-11-07	8.D: Continued with building state machine	
;	27-22-07	9.0: Debugged state machine. encflg 0 is no action.
;	01-12-07:	9.3: Debugged VfoIsMem,and 7th programmable character programming
;	02-12-07	9.4: Parameters to display allowed outside page 0
;	03-12-07	9.A: Encoder Longpress in IRQ routine (EncFLg= 6, SWIFLG bit 7)
;					 3 ARDF modes to menu, various debugging
;	09-12-07	9.F: ACSM via lookup table
;				A.0: Added a few Calibration routines
;	11-12-07	A.A: Debugged F up/down to respond to first step of encoder
;	12-12-07	A.C: Moved Enc history processing to low registers, changed weighting
;	13-12-07	B.0: Removed Eprom write flags.
;	13-12-07	B.1: Rf attn only display partially added
;	13-12-07	B.2: Debugged state machine for CalFreqDown
;	13-12-07	B.3: Converted COnvPf to table lookup code, saved 114 Bytes
;	15-12-07	B.4: Started porting to Mega168
;	16-12-07	B.5: Finished porting to Mega168
;	16-12-07	B.6: Debugged porting
;	18-12-07	B.8: Debugged ConvPf (Not done yet)
;	19-12-07	B.9: And some more + ACSM (Z is a byte register)
;	21-12-07	B.A: Corrected QRG Powerstring table
;	22-12-07	B.B: Attenuator cal in 5 dB steps
;				B.C: Debugged mode table for RF calibration down
;	23-12-07	B.D: Debugged SOUGEN routine for high pitched tones
;				B.E: Prevent long press actions of encoder when up/down + pressed events 
;					 ocurred without releasing the switch first.
;	24-12-07	B.F: Fixed ConvAB20 routine
;				C.0: StoreMem adapted for > page 1
;	26-12-07	C.1: Changed STR100
;	27-12-07	C.2: Removed
;	29-12-07	C.3: Added ModeARDF12_3	
;				C.4: Added ModeArdf2_5 
;	30-12-07	C.5: Debugged GetCurrentFrq1x2 for fox <>fox+1
;				C.6: In timer reset mode Fox can be selected with press and turn
;				C.7: Added Hunting mode MHuntFpGainMem, MHuntFpFreqMwm
;	02-01-08	C.9: Put ARDF and FP hunting modes under one menu item
;	03-01-08	C.A: Defined Hunting modes properties bits
;	06-01-08	C.C: Double click on encoder code added
;	07-01-08	C.E: Toggling RxA/B with two short clicks on the pushbutton switch (Needs debugging)
;	12-01-08	D.0: Debugged EEProm save, RXA/B switching implemented,
;	14-01-08	D.1: Moved Eprom init and ACSM lookup to EEProm, Need to program EEprom from now on.
;	16-01-08	D.2: Inclim works only for Xh <$80, REduced RxA/B maxtime to 120s
;	20-01-08	D.5: Inclim works now for any Xh value, Implemented auto switching RxA/B
;	21-01-08	D.6: Removed EEPROMINIT, Auto RxAB switching off when changing hunting modes
;	22-01-08	D.7: Warning tone in all hunting modes if # of foxes >1
;	23-01-08	D.8: Force RxA when changing hunting modes
;	24-01-08	D.A: Debugged DebInc code
;	25-01-08	D.B: Restore gain mode when coming back to hunting mode (Freq entry mode forces AGC)
;	26-01-08	D.D: Only 1 mem entry mode, Mem # in lower line of disp during mem entry mode, 
;					 Attenuator table back to 10 dB entries
;	01-02-08	D.E: Save RF gain mode during init and when entering menu's, not only when changing memories
;					 Calculate DistCfVal during startup
;	03-02-08	D.F: No functional changes, put subroutines in alphabetical order
;	10-02-08	E.0: No Auto RXA/B switching when in MFreqEntry mode
;					 Made displayed tekst consistent (N <> #)
;					 Load RxXTimeReg during init so auto toggle Rx A/B works properly from the start
;					 EEPCLR Variable definition removed
;					 FPModeFreq Variable and it's usage removed (Function is part of state machine)
;					 EEWRAK variable removed. Not needed with auto EEPROM save functionality
;					 Only prep RxB data during startup
;	13-02-08	E.1: RDBEEand WRBEE routines removed, they were no longer called.
;	18-02-08	E.3: Redesigned VfoIsMem routine for indicating vfo<>mem difference in a graphical manner 
;	19-02-08	E.4: Default to Freq. adjustment when AGC is turned on in ALL FP hunting modes
;					 Indicate memory channel number for exact freq +/- VfoMemDelta
;	21-02-08	E.5: Changed mem channel difference indicators. fixe default freq entry fp mode if AGC is turned on	
;	26-02-08	E.6: Cleaned up a lot of labels in the main, m100ms and m1s routines, No functional changes
;	27-02-08	E.7: Made UpdateLCD more code efficient
;	02-03-08	0.1: Adapted from FJRX 2FP.Version E.7.
;					 Changed startup name to FJRX-2FP. 
;					 Attenuator table every 5 dB, (No longer compatible with existing
;					 DF1FO calibration data, but you can interpolate)
;					 LCD LED Brightness via PB5
;	09-03-08	0.2: Debugged CONVAB for correct distcf processing Add R16,R17 not Add R17,R16
;     				 Changed I2Wait from 5 to 6.4 Micro Second
;	09-03-08	0.3: Wrote universal I2Out routine
;	10-03-08	0.4: No sounds for gain change when programming memories (PE5EDW request)
;					 Dynamic freq stepsize via lookup table
;	11-03-08	0.5: LCDStr has the potential to be more code efficient. Push pop not needed
;					 LPM has posibilities to store data directly in a register and post increment Z
;	13-03-08	0.6: Bad day, revert to 0.5
;	15-03-08	0.7: Fixed ARDF12>3 mode. Old mem (1 or 2) was not saved. (Reported by PE5EDW)
;					 LCDStr routine more code efficient, push/pop of registers 
;					 not needed and LPM with post incr for Z. 
;					 No wraparound for # of timed foxes in NFoxesUp/Down. 
;					 Store Compilation Date/time Time at the end of the memory
;				0.9: Debugged I2C prep part near M100ms200
;	18-03-08	1.0: Shortened Attenuator sounds to single beeps
;	19-03-08	1.1: Attn sounds on/of in freq mode under cal menu, 
;					 lcd brightness removed, Bat cal to own value of 126
;	20-03-08	1.2: F1, Att. values and freq correction from PA2PIM's receiver in EEPROM
;	21-03-08	1.3: No wraparound of PWM value when calibratring attenuator.
;					 New PWM values for >20 year old TCA440 Removed blank
;					 gain symbol definition.
;	29-04-08	1.4  Jutberg 2008 experience:  (www.radiokampweek.nl/index2008.html)
;					 ========================
;					 Changed Gainmode indication tones.
;					 Morse A =  AGC, Morse M = MGC, Morse D = DF1FO Auto
;					 Gain reduction. Reduced max # channel steps from 80 to 40
;					 (50 kHz/click) ACSM auto on to max 7/8 (was 3/8)
;	01-05-08	1.5	 Freq stepsize from exponential to linear curve.
;					 Shortened ACSM "Nachlaufzeit" from 3s to 1.2 s
;	02-05-08	1.6	 Tones on/off in AGC mode selectable, (Not only in freq. entry mode)
;				1.7	 In Freq entry mode only change FreqIndex when changing memory number.
;					 Do not change the Rx freqency

; >> FUSES << :	EESAVE = 0
; ===========	CKSEL  = 1111, SUT = 11, BODEN  = 0, BODLVL = 0   ; Brown-Out Erkennung @ 4V	
;		Ergibt Hex-Pattern D13F

; DEFinitions
;******************************************************************************
.def	C0 				= R1	; Contains $00, for replacing cp, sts instr.
.def	C1 				= R2	; Contains $01, for replacing cp, sts instr.
.def	EncFlgReg 		= R3	; Encoder Flag Register
.def	SwitchFlgReg	= R4	; Switch Flag Register
.def	CurrentUDReg	= R5	; 1 if current encoder pulse is up, 2 for down
.def	PreviousUDReg	= R6	; 1 if previous encoder pulse was up, 2 for down
.def	EncoEvent0_100	= R7	; # Encoder events in previous 0-100 ms
.def	EncoEvent100_200 = R8	; Weighted # of encoder events in prev. 100-200ms
.def	EncoEvent200_300 = R9	; Weighted # of encoder events in prev. 200-300ms
.def	EncoEvent300_400 = R10	; Weighted # of encoder events in prev. 300-400ms
.def	EncoEvent400_500 = R11	; Weighted # of encoder events in prev. 400-500ms
.def 	EncoEventHistory = R12	; Weighted # of encoder events in prev. 0-400 ms
.def	HuntMPropReg	= R13	; Properties of selected hunting mode
.def	RxXReg			= R14	; Rx selected, 0 = RxA, 1 = RxB
.def	RxXTimeReg		= R15	; RxX remaining on time in sec


; EQUal statements
;******************************************************************************
;
; Various constants
;
.equ	ACSMDwellTime	= 12		; Acoustic S-meter continuous n *100 ms after level dropps below ACSMLV
.equ	AGC = 3 					; Auto Gain Control
.equ	AutoRxABBit = 0				; Bit to test in AutoRxAB variable to see if Auto switching is on
.equ	BatTestInterval = 2			; Test Battery Voltage every 2 min
.equ	DGC = 1						; DF1FO gain Control. Auto Gain reduction
.equ	EncLongpress = 6 			; EncFlag value for a long press on the encoder switch
.equ	EncShortPress1 = 5			; EncFlg value for single short press of encoder switch (Position in EncoderStateTable)
.equ	EncShortPress2 = 7			; EncFlg value for 2 or more short presses of encoder switch (Position in EncoderStateTable)
.equ	GainDecrDelayTime = 1 		; Wait a min of GDDelayTime *100ms before decreasing gain in DGC and AGC mode 
.equ	GainIncrDelayTime = 2 		; Wait a min of GIDelayTime *100ms before increasing gain in AGC mode
.equ	HMPAGC	= 0b00100000		; AGC allowed in this hunting mode
.equ	HMPAGCBit = 5
.equ	HMPFP	= 0b00000100		; Huntingmode is a FP mode
.equ	HMPFPBit = 2
.equ	HMPMenuDisp = 0b01000000 	; Display Hunting mode in Hunting mode selection menu
.equ	HMPMenuDispBit = 6
.equ	HMPTimedMem = 0b00001000	; Huntingmode is a Timed Memory Hunting mode
.equ	HMPTimedMemBit = 3
.equ	HMPWatchPrio = 0b00010000 	; Huntingmode is a Watch Priority (capable)Hunting mode
.equ	HMPWatchPrioBit = 4
.equ	I2CTSA6057 = $C4			; I2C Address of TSA6057
.equ	LCD_size_char= 16			; Total number of characters on the display
.equ	LongPress = 75	 			; 75*8ms is 600msec press. Long press bit/value is set in SWIFLG, ENCFLG
.equ	MGC = 2						; Manual Gain Control
.equ	PFoxUnknown = (PFoxStringsEnd-PFoxStringsStart)/2 - 1	; Last entry in powerstringtable is unknown power > dB display, no distance
.equ	ShortPress = 10				; 10*8ms is 80 msec press. Short press bit/value is set in SwiFLG and EncFlg
.equ	SwitchLongPress = 4			; Value in SwiFlg when pushbutton is long pressed
.equ	SwitchLongPressBit	= 2
.equ	SwitchPushFiFoLength = 4	; FiFo length is 4 * 100 ms
.equ	SwitchShortPress1 = 1		; Value in SwiFlg when pushbutton is short pressed
.equ	SwitchShortPress1Bit = 0
.equ	SwitchShortPress2 = 2		; Value in SwiFlg when pushbutton is multiple times shortly pressed
.equ	SwitchShortpress2Bit = 1
.equ	UBatteryMax = 90			; Maximum value of Battery Warning level in 100 mV units
.equ	UBatteryMin = 58			; Minimum value of Battery Warning level in 100 mV units
.equ	UmessAMin = 10				; Min value of RSSI before gain increase in AGC mode
.equ	UmessAmax = 130				; In auto gain mode reduce gain if this Signal strength value is exceeded
.equ	VERSHI = '1'				; Main Version of Code
.equ	VERSLO = '7'				; Sub Version of code
.equ	VfoMemDelta = 3				; Indicate Memory # when vfo freq differs from memory by < VfoMemDelta

; *****************************************************************************
; Equal Statements for operating modes

.equ 	MHuntStart	= 0	
.equ	MHuntArdfStart = MHuntStart

; Receiver OFF
.equ	MRxOff	= MHuntStart

; ARDF Hunting modes
.equ	MHuntArdf	= MRxOff +1
.equ	MHuntArdf12_3 = MHuntArdf+1
.equ	MhuntArdf2_5 = MHuntArdf12_3 +1
.equ	MHuntArdfStop = MHuntArdf2_5

; FP Hunting modes
.equ	MHuntFpStart =  MHuntArdfStop+1
.equ	MHuntFpGain	= MHuntFpStart
.equ	MHuntFpGainMem = MHuntFpGain+1
.equ	MHuntFpFreq = MHuntFPGainMem+1
.equ	MHuntFpFreqMem = MHuntFpFreq+1
.equ	MHuntFpStop = MHuntFpFreqMem	
.equ	MHuntStop	= MHuntFPstop	
		
;Freq Entry Modes	$10	
.equ	MFreqEntryStart	= MHuntStart + $10
.equ	MFreqEntry		= MFreqEntryStart
.equ	MFreqEntryStop	= MFreqEntry				
		
;Aux Modes		$20
.equ	MAuxStart	= MFreqEntryStart + $10
.equ	MAuxFreqCh	= MAuxStart	
.equ	MAutoRxABSw = MAuxFreqCh+1
.equ	MAuxStopw 	= MAutoRxABSw+1
.equ	MAuxTimer	= MAuxStopw+1
.equ	MPQrg		= MAuxTimer+1
;.equ	MLEDBright	= MPQrg+1
;.equ	MAuxSettings = MLEDBright+1
.equ	MAuxSettings = MPQrg + 1
.equ	MAuxStop	= MAuxSettings
		
;Setting Modes	$30	
.equ	MSetStart	= MAuxStart + $10
.equ	MNFoxes		= MSetStart
.equ	MTRxA		= MNFoxes+1
.equ	MTRxB		= MTRxA+1
.equ	MTFoxS		= MTRxB+1
.equ	MTFoxMs		= MTFoxS+1
.equ    MAuxHunt	= MTFoxMs+1
.equ	MNQrg		= MAuxHunt+1
.equ	MEOTAlarm	= MNQrg+1
.equ	MAcSmLv		= MEOTAlarm+1
.equ	MSetStop	= MAcsmLv
		
;Calibration Modes	$40	
.equ	MCalStart	= MSetStart + $10
.equ	MCalStarting = MCalStart
.equ	MCalBat		= MCalStarting+1
.equ	MCalDist	= MCalBat+1
.equ	MCalFreq	= MCalDist+1	
.equ	MCalAttn	= MCalFreq+1
.equ	MBatAl 		= MCalAttn+1
.equ	MAGCAudio = MBatAl+1
.equ	MCalStop 	= MAGCAudio

;Attenuator Calibration mode $50
.equ 	MCalAttnStart = MCalStart + $10
.equ	MCalAttnActive = MCalAttnStart
.equ	MCalAttnStop = MCalAttnActive


; SET Statements

;*********************************************************************************************
.set	FBER68 = 0			; Frequenzbereich 0:144..146, 1:144..148
.set	Processor = 8		; Uncomment this line for compiling for Mega8 processor
;.set Processor = 168		; Uncomment this line for compiling for Mega168 processor
							; Note: Mega 168 code not complete/tested
; INCLUDE statements
;*********************************************************************************************
.IF Processor == 8
	.include 	"m8def.inc"	; System fr Atmels ATmega8
.EndIf
.If Processor == 168
	.include	"m168def.inc"
.Endif

.include	"FJRX_2FP_macros.asm"	; Macros for this project

; Other directives
;*********************************************************************************************
.listmac				; Expansion of the macro's is to be shown in the listfile 

; ********************************************************************************************
; Variablen:

.dseg				; Data segement from here onwards
.org	SRAM_START	; Start of RAM address range

; ++ Eingestellte Parameter, werden im EEPROM gerettet
;
EEPBeg:
TBD1:	.Byte	1	; Acc. to Atmel first byte can be clobbered

EEP0:				; Beginn des Frequenz-Bereichs
					; 0..3360 = 143,9..148,1, LSB,MSB
FRQ1:	.Byte	2	; Frequenz-Codewert Frequenz Speicher 1
FRQ2:	.Byte	2	; Frequenz-Codewort Frequenz Speicher 2 
FRQ3:	.Byte	2	; Frequenz-Codewort Frequenz Speicher 3
FRQ4:	.Byte	2	; Frequenz-Codewort Frequenz Speicher 4
FRQ5:	.byte	2	; Frequenz-Codewort Frequenz Speicher 5
FRQ6:	.byte	2	; Frequenz-Codewort Frequenz Speicher 6
FRQX:	.Byte	1	; Aktive Frequenz Speicher (1..6)

; Grenze zwischen Frequenz- und HW-SetUp-Bereich

EEP1:
NFOX:	.Byte	2	; Zahl der Fchse Z,E  1..10	
TFOXS:	.Byte 	2	; Fuchs-Laufzeit Sekunden Z,E  1..99
TFOXMS:	.Byte	2	; Fuchs-Laufzeit Milli-Sekunden H,Z 0..98 in 2er Schritten
PFOX1:	.byte	1	; Sendeleistung Fuchs 1
PFOX2:	.byte	1	; Sendeleistung Fuchs 2
PFOX3:	.byte	1	; Sendeleistung Fuchs 3
PFOX4:	.byte	1	; Sendeleistung Fuchs 4
PFOX5:	.byte	1	; Sendeleistung Fuchs 5
PFOX6:	.byte	1	; Sendeleistung Fuchs 6
PFOXBN:	.Byte	1	; Sendeleistung der aktiven Frequenz in FRQX:
					;    Coded 0=1W, 1=3W, 2=10W ..15=30W 
NFREQ:	.Byte	1	; Zahl der Frequ. 1..6
TALARM:	.Byte	2	; EOT Alarm Vorlaufzeit 0..30 sec, 0 = Aus Z,E
ACSMLV: .Byte	1	; Akustisches S-Meter Schwelle 0..7 Achtel
FRQ1X2:	.Byte	10	; Modus 1x2<>3 Frequenz 1 oder 2 fr Fuchs 10, 1..9
MHuntX:	.Byte	1	;Current Hunting Mode
RxATimeBin:	.Byte	1	; RxA On Time
RxBTimeBin:	.Byte	1	; RxB On Time
; Grenze zwischen HW-SetUp und Abgleich-Bereich

EEP2:

ABSTAB:	.Byte	25	; Tabelle Abschwcher-Codes 0,5,10,15,20..120 dB
FFINE:	.Byte	1	; Frequenz Fein 64 -/+ 25 = -25..+25 * 1,25 kHz
UBFINE:	.Byte   1	; Skalierungswert fr Ub-Messung 0..255
UBWBIN: .Byte   1	; Schwelle Batteriewarnung in 1/10 V (60..80)
DISTCF:	.Byte	1	; Korrekturfaktor Entfernung -5..+5 = -/+25dB
;LEDBright:	.Byte	1	; 0 = Low LCD LED current (Night hunt), 1 = High current 
AGCAudio:	.Byte	1 	; 0 = No short Gain change tones during freq entry
							; 1 = short audio tones
EEPEND:				; Ende der geretteten Einstellungen

AbsVal:	.Byte	3	; Abschwcher Analogwert 0..255 H,Z,E 	
AbschW:	.Byte	3	; Abschwcher Anzeige 0..120 dB in 5 dB Stufen H,Z,E
AutoOn:	.Byte	1	; Abschwcher-Automatik ein = ' ', aus = '*'
DCFVAL:	.Byte	2	; Distanz Korrekturfaktor Anzeigewert VZ,E
Dist:	.Byte	4	; Distanz-Anzeige H,Z,E
FFiVal: .Byte   3	; FFINE Anzeigewert -/+0..25  VZ,Z,E
Fox:	.Byte	2	; Current Fox Z,E  1..10
Freq:	.Byte	4	; Frequenz-Dezimalstellen 1M,100k,10k,1k
GainMemSymbol: 	.Byte 1 ; LCD charcode for combined Gain/mem symbol
PFox: 	.Byte	4	; Sendeleistung Watt Anzeigewert Z,E,h,z
RxATime: .Byte	3	; RxA On time, BCD, H,Z,E
RxBTime: .Byte	3	; RxB On Time, BCD, H.Z,E
RxXSymbol:		.Byte	1  ; A or B for Receiver A or B
Stopw: 	.Byte	5	; Stopuhr Eh,Zm,Em,Zs,Es
Timer:	.Byte	4	; Count Down Timer Zs,Es,Hms,Zms zhlt in 20 ms Schritten
UBFVal: .Byte   3	; UBat Korrektur UBFINE Anzeigewert 0..255 H,Z,E
UBWarn: .Byte   2	; Batteriewarnungsschwelle V/mV E,h 
UBatt:	.Byte	3	; Batterie-Spannung in V/mV    Z,E,h 	
Umess:  .Byte	8	; Messspannung Bargraph 8 Stellen


;End of list of Variables which must have an address <=$FF - LCD_size_char

;++ Display-Ansteuerung

LCDDAT:	.Byte   LCD_size_char	; New Data to be displayed on the LCD
LCDDAT_Current: .Byte LCD_size_char ; Current data on the LCD

; ++ Zustands-Variablen

AAWAIT:			.Byte	1	; Wartezeit nach 5dB-Auto-Abschwchung (n * 100 ms)
ABSBIN:			.Byte	1	; Abschwcher Binaer-Wert 0..24 fr 0, 5, 10..120 dB
ACSMON:			.Byte	1	; Akustisches S-Meter ein = $40, aus = $00
ACSTIM:			.Byte	1	; Akust.S.-Meter > Schwelle, Nachlaufzeit (*100ms)
AGIWait: 		.Byte 	1	; Waiting time after 5 dB Auto Gain Increase (n * 100 ms)
AutoRxABSw:		.Byte	1	; Auto RxA/B switching enabled. 0 = no, 1 = yes
Deltasign:		.Byte 	1	; Sign for the smallest difference between FrqBin and
;							  any of the frequencies stored in frq1...6
DeltaValue:		.Byte 	2	; Absolute value of above difference
FLATIM:			.Byte	1	; Verbleibende Flashtime fr Text-Flash (n * 100 ms)
FRQBIN:			.Byte	2	; Aktuelle Frequenz 0..3360 = 143,9..148,1, LSB,MSB
FrqBinDelta:	.Byte	12	; Array with delta between FrqBin and FRQ1...6
FRQCHG: 		.Byte   1	; 1 = Frequenz gendert => PLL neu laden
FUDARM:			.Byte	1	; Scharf zeit for Frq+/- with <<*>> Mode Peilen (* 100ms)
Frq512:			.Byte	1	; Previous freq in mode MhuntArdf12_3, returning from Freq-nr 3
GainMode: 		.Byte 	1	; 1 = ARDF/DF1FO mode, 2 = MGC, 3 = AGC
GainModeSaved:	.Byte	1	; Gain mode saved when entering freq entry mode which uses AGC
I2CBytes:		.Byte	1	; How many bytes to send to I2C chip, excluding Address
I2CData:
I2CAddress:		.Byte	1	; Address of I2C chip to communicate with
				.Byte	5	; Data to send to I2C chip, First byte is address
MODE:			.Byte	1	; Mode (See .def statements above for description )

; RXA saved parameters

RxAParam:
RxAFrqX:		.Byte	1	; Memory
RxAAbsBin:		.Byte	1	; Attenuator setting
RxAFrqBin:		.Byte	2	; Frequency
RxAACSMon:		.Byte	1	; AcSM status
RxAGainMode:	.Byte	1	; Gain mode is DGC
RxAMHuntx:		.Byte	1	; Hunting mode
RxAHuntMPropReg:	.Byte	1 ; and the properties thereof

;RXB saved parameters

RxBParam:
RxBFrqX:		.Byte	1
RxBAbsBin:		.Byte	1
RxBFrqBin:		.Byte	2
RxBACSMon:		.Byte	1
RxBGainMode:	.Byte	1
RxBMHuntx:		.Byte	1
RxBHuntMPropReg:	.Byte	1

SOUPNT:			.Byte	2	; Pointer auf Soundstring LSB,MSB; Fertig bei MSB = 0
STOPWR:			.Byte	1	; 1 = Stopuhr luft, 0 = steht
SwitchPushFiFo:	.Byte	SwitchPushFiFoLength
TimedMemAct:	.Byte	1	; 1= Fox  # uses mem freq out of FRQ1x2 array





; ++ Analog-Werte

UMESSA:	.Byte	1	; Messspannung fr akustisches S-Meter
UMESSP:	.Byte	4	; Spitzenwertspeicher Umess, ltester Wert in *+3 

; ++ Kommunikation mit Int.-Handler

EncFlg:	.Byte	1	; Encoder state (Value)
					; 0 = Nothing active
					; 1 = Up
					; 2 = Down
					; 3 = Up+Pressed, 
					; 4 = Down+Pressed, 
					; 5 = Drehdrcker Puls (Short)
					; 6 = Drehdrcker Puls (Long)

SWIFLG: .Byte   1	; Switch Flags, (Bit)
					; 0=Short press once, 
					; 1=Short press more then once
					; 2=Long press
					; 3=not used 
					; 4=not used
					; 5=not used
					; 6=not used
					; 7=not used

TICK:	.Byte	1	; 100 ms Timing Flag 
TACK:   .Byte	1	; 1 s Timing Flag 
TOCK:   .Byte	1	; 10 s Timing Flag 

ALATIM:	.Byte	1	; EOT Alarm Flag, 1 = Alarm auslsen 
BATTIM:	.Byte	1	; Batterie-Prf-Zeit, 1 = prfen
BatTest:	.Byte	1	; Counts from BatTestInterval to 0 every minute
FRQTIM:	.Byte	1	; Frequenzwechselzeit Mode MhuntArdf2_5 and FPxxxMem modes
 

; ++ Interrupt-Handler

DEBP1:	.Byte   1	; Debounce Counter Drehgeber 
EncEvent:	.Byte   1 ; 1 = Encoder event with switch pressed happend and switch is not released yet
DEBDD:	.Byte	1	; Debounce Drcker Drehgeber 
DEBST:	.Byte   1	; Debounce Schnell-Taster
TIK3U1:	.Byte   1 	; Zaehlt alle 320 us bis 3 (1 msec)
TIK3U2:	.Byte   1 	; Zaehlt alle 320 us bis 25 (8 msec)
TIK8M:	.Byte	1	; Zaehlt jede 8 msec bis 5
TIK20M: .Byte	1	; Zaehlt jede 20 msec bis 50 (1 sec)	
TIK100: .Byte   1	; Zaehlt jede 20msec bis 5 (100 ms) 
TIK1S:	.Byte	1	; Zaehlt jede 1 s bis 10 (10 s)
Tik10s:	.Byte	1	; Zaehlt jede 10 s bis 6 (1 min)

; *****************************************************************************
; Audio S Meter lookup table and character definition table.
;
; Data gets copied here from EEProm during start of code
;
ACSMLookupRam:		.Byte	302
Ram_Char_def_start: .Byte 48	; Space for the first 6 LCD programmable characters
Ram_Char_def_1:

Ram_MemNumDef:					; Space for the Roman Numerals indicators for the memory
Ram_MemNum1_Def:	.Byte 4
Ram_MemNum2_Def: 	.Byte 4
Ram_MemNum3_Def: 	.Byte 4
Ram_MemNum4_Def: 	.Byte 4
Ram_MemNum5_Def: 	.Byte 4
Ram_MemNum6_Def: 	.Byte 4

; Space for the signs to indicate delta between mem and vfo in 10 kHz steps
; (20 khz for 144-148 receivers)

Ram_MemDelta:
Ram_MemDelta1:		.Byte 4
Ram_MemDelta2:		.Byte 4
Ram_MemDelta3:		.Byte 4
Ram_MemDelta4:		.Byte 4
Ram_MemDelta5:		.Byte 4
Ram_MemDelta6:		.Byte 4
Ram_MemDelta7:		.Byte 4
Ram_MemDelta8:		.Byte 4

; Space for the characters to indicate gain mode
Ram_GainMode_Def:
;Ram_GainMode0_Def:	.Byte 4
Ram_GainMode1_Def:	.Byte 4
Ram_GainMode2_Def:	.Byte 4
Ram_GainMode3_Def:	.Byte 4
Ram__Char_def_end:

; Space for the encoder clicks lookup table
Ram_SynthChannelStepsStart:	
					.Byte 16
Ram_SynthChannelStepsEnd:

; *****************************************************************************
; S T A R T  O F  C O D E
; *****************************************************************************
;
.cseg
.org	0

; ++++ Interrupt Vectors
;
.IF	Processor == 8
		RJMP	START		; 0 Reset   
		RJMP	START		; 1 Ext Int 0 
		RJMP	START		; 2 Ext Int 1 
		RJMP	START		; 3 Timer Compare 2    
		RJMP	START		; 4 Timer Overflow 2 
		RJMP	START		; 5 Input Capture 1 
		RJMP	START		; 6 Timer Compare 1A
        RJMP	START		; 7 Timer Compare 1B
        RJMP    START		; 8 Timer Overflow 1
		RJMP    TIMINT		; 9 Timer Overflow 0
        RJMP    START		; $A SPI     
        RJMP	START		; $B USART Receive Complete
        RJMP	START		; $C USART Data Reg. Empty
        RJMP    START		; $D USART Transmit Complete
        RJMP    START		; $E A/D-Converter
        RJMP    START		; $F EEPROM
        RJMP    START		; $10 Analog Comparator
        RJMP    START		; $11 2-wire I/F
        RJMP    START		; $12 SPM complete
.Endif
.IF Processor == 168
		JMP		START		; 0 Reset   
		JMP		START		; 1 Ext Int 0 
		JMP		START		; 2 Ext Int 1 
		JMP		START		; 3 Timer Compare 2    
		JMP		START		; 4 Timer Overflow 2 
		JMP		START		; 5 Input Capture 1 
		JMP		START		; 6 Timer Compare 1A
        JMP		START		; 7 Timer Compare 1B
        JMP    	START		; 8 Timer Overflow 1
		JMP    	TIMINT		; 9 Timer Overflow 0
        JMP    	START		; $A SPI     
        JMP		START		; $B USART Receive Complete
        JMP		START		; $C USART Data Reg. Empty
        JMP    	START		; $D USART Transmit Complete
        JMP    	START		; $E A/D-Converter
        JMP    	START		; $F EEPROM
        JMP    	START		; $10 Analog Comparator
        JMP    	START		; $11 2-wire I/F
        JMP    	START		; $12 SPM complete
.EndIf

; ++++	Entry-Point:

START:  ldi     R16,LOW(RAMEND)		; Stackpointer
        out     SPL,R16			    ; ...an das
        ldi     R16,HIGH(RAMEND)	; ...RAM-Ende
        out     SPH,R16			    ; ...setzen

; ++++ Initialize RAM area

		clr		XH
		ldi		XL,LOW(SRAM_START)
		clr		R16
STA01:
		st		X+,R16		
		cpi		XL,LOW(RAMEND+1)
		brne	STA01
		cpi		XH,HIGH(RAMEND+1)
		brne	STA01

; ++++ Initialize Port B Bit 
;				 0 as Input P1 with Pull-Up
;			     1 as Sound Output (=OC1A)
;			     2 as Sound Degate Output        
;			     3..5 used by ISP	  
;			     3 also Uregel Output (OC2)	
;			     4 also Input Drcker with Pull-Up
;				 5 also tristate for LCD Background LED brightness

        ldi     R16,0b00001110
        out     DDRB,R16
		ldi		R16,0b00010001
        out		PORTB,R16

; ++++ Initialize Port C Bit
;				 0 as Vorstufe Ein/Aus Output
;			     1,2 as Analog In Ub,Umess
;			     3 as Input P0 with Pull-Up 
;			     4 as I2C SCL Line, Output, OC, Hi      
;		     	 5 as I2C SDA Line, BiDi, OC, Hi

        ldi     R16,$01
		mov		C1,R16			; Move a constant to the C1 register
        out     DDRC,R16
        ldi		R16,$08	
        out		PORTC,R16

; ++++ Initialize Port D Bit 
;				0..5 as Output for Display
;	            6,7 as Input with Pull-Up for Mode-Schalter

        ldi     R16,$3F 
        out     DDRD,R16
        ldi		R16,$C0
        out		PORTD,R16

; ++++ Initialize A/D-Converter

.If Processor == 8         
        OUT     ADMUX,C1	; Select Analog Input 1
		LDI     R16,$86     ; Enable ADC, Clock Divider 64 = 78 kHz
        OUT     ADCSR,R16	;     = 180 usec Conversion Time
.EndIf

.If Processor == 168
		STS		ADMUX,C1	; Select Analog Input 1
        LDI     R16,$86     ; Enable ADC, Clock Divider 64 = 78 kHz
        STS     ADCSRA,R16	; = 180 usec Conversion Time
.Endif

; ++++ Intialize Timer/Ctr 0 as 320 usec Timing Interrupt (5Mhz/64/25)

		ldi		R16,$03		; Set Clock Divider to 64
.If Processor == 8
		out		TCCR0,R16
.Endif
.If Processor == 168
		out		TCCR0B,R16
.EndIf
		ldi		R16,-25		; Set Counter to Start-Value
		out		TCNT0,R16
.If Processor == 8
		out		TIMSK,C1	; Enable Counter 0 Overflow Interrupt
.EndIf
.If Processor == 168
		sts		TIMSK0,C1
.Endif
		ldi		R25,50		; Wait a little
		rcall	DELXMS

; ++++ Intialize Timer/Counter 1 as Variable Osc. for Acoustic S-Meter
; -- CTC-Mode on OC1A Pin, f is 5,0 MHz /2 /OCR1AH_L 

		ldi 	R16,$00		; $00 = Sound Off, $40 = On	
		mov		C0,R16
		
.If Processor == 8
		out		TCCR1A,C0
		ldi 	R16,$09
		out		TCCR1B,R16
		ldi 	R16,$0A		; Komparator fuer Counter Reset & Output Toggle
		out		OCR1AH,R16 
		out		OCR1AL,C0 	; $0A00 = 1 kHz Startwert
.EndIf
.If Processor == 168
		sts 	TCCR1A,C0
		ldi		R16,$09
		sts		TCCR1B,R16
		ldi		R16,$0A
		sts		OCR1AH,R16
		sts		OCR1AL,C0
.Endif
; ++++ Intialize Timer/Counter 2 as fast PWM (19 kHz) for Uabschw
.If Processor == 8
		ldi		R16,$69		; Clock Divider 1, fast PWM, OC2 active
		out		TCCR2,R16
.Endif
.If Processor == 168
		sts		TCCR2B,C1	; Clock Divider 1, fast PWM, OC2a active
		ldi		R16,$83
		sts		TCCR2A,R16
.Endif

; ++++ Hole Parameter aus EEPROM
		
		ldi		Yh,high(EePromInitValues)	; Pointer to EEprom
		LDI		YL,low(EePromInitValues)	; 
		ldi		Xh,high(EepBeg)				; Pointer to Ram
		ldi		Xl,low(EepBeg)
		ldi		Zh,high(EePromInitValuesEnd-EePromInitValues) ; Counter
		ldi		Zl,low (EePromInitValuesEnd-EePromInitValues)
		rcall EEPromRead


; ++++ Initialize some Variables	


; Copy the ACSM lookup table and the character definition from EEProm to Ram
		ldi		Yh,high(ACSMLookup)	; Pointer to EEprom
		LDI		YL,low(ACSMLookup)
		ldi		Xh,high(ACSMLookupRam)	; Pointer to Ram
		ldi		Xl,low(ACSMLookupRam)
		ldi		Zh,high(SynthChannelStepsEnd - ACSMlookup) ;Counter
		ldi		Zl,low (SynthChannelStepsEnd - ACSMlookup)
		rcall 	EEPromRead
	
		sts 	STOPWR,C1	; Stopuhr luft nach Power-On an
		sts 	FRQCHG,C1	; Laden der PLL anfordern
		sts 	Gainmode,C1	; Gainmode is auto gain reduction
		sts		GainmodeSaved,C1
		ldi 	R16,2			
		sts 	ABSBIN,R16	; Abschwcher auf 2 (10 dB)
		ldi 	R16,' '
		sts 	AUTOON,R16  ; ACSM Off
		ldi 	R16,5
		sts 	AAWAIT,R16	; Abschwchermechanismus verzgert einschalten
		sts		FRQTIM,C1
		sts		BATTIM,C1	; Check battery
		sbic	PINB,4 		; Wenn Drcker gedrckt
		rjmp	STA030	 
		ldi 	R16,MCalStart	;Calibration mode
		sts 	MODE,R16
		sts		FRQ512,C1
		mov		RxXReg,C0	; RxA active


STA030:
; ++++ Start Interrupt Handler
		sts		Fox+1,C1
		rcall	CLRTIM		; Fuchstimer auf Startwert	
		sei					; Global Interrupt enable

; ++++ LCD initialisieren und Start-Text anzeigen


; *******************************************************************************************
; LCD-Initialisierung ber die 4-Bit Schnittstelle

INILCD: 
.If Processor == 168		
		ldi	R25,20		; Wait 20 msec
		rcall	DELXMS
		ldi	R25,$30		; Initialize LCD 
		call	LC1CMD
		ldi	R25,5		; Wait 5 msec
		rcall	DELXMS
		ldi	R25,$30     ; Initialize LCD 
		call	LC1CMD
		ldi	R25,5		; Wait 5 msec
		rcall	DELXMS
		ldi	R25,$30		; Initialize LCD
		call	LC1CMD
		ldi	R25,16		; Wait Another 16 msec
		rcall	DELXMS
		ldi	R25,$20		; Auf 4 bit Betrieb schalten
		call	LC1CMD
		ldi	R25,$28		; System Set: 2zeilig, 5*7 Matrix,4 bit Betrieb
		call	LC2CMD
		ldi	R25,$06		; Cursor Mode: laeuft nach rechts
		call	LC2CMD
		ldi	R25,$0C		; Display on, Cursor Off
		call	LC2CMD
		ldi	R25,$80		; DDRAM-Adress auf 0
		call	LC2CMD
		ldi	R25,$01		; Clear Display, Cursor Home
		call	LC2CMD
		ldi	R25,2		; Wait 2 msec
		rcall	DELXMS
.Endif

.If Processor == 8		
		ldi	R25,20		; Wait 20 msec
		rcall	DELXMS
		ldi	R25,$30		; Initialize LCD 
		rcall	LC1CMD
		ldi	R25,5		; Wait 5 msec
		rcall	DELXMS
		ldi	R25,$30  	; Initialize LCD 
		rcall	LC1CMD
		ldi	R25,5		; Wait 5 msec
		rcall	DELXMS
		ldi	R25,$30		; Initialize LCD
		rcall	LC1CMD
		ldi	R25,16		; Wait Another 16 msec
		rcall	DELXMS
		ldi	R25,$20		; Auf 4 bit Betrieb schalten
		rcall	LC1CMD
		ldi	R25,$28		; System Set: 2zeilig, 5*7 Matrix,4 bit Betrieb
		rcall	LC2CMD
		ldi	R25,$06		; Cursor Mode: laeuft nach rechts
		rcall	LC2CMD
		ldi	R25,$0C		; Display on, Cursor Off
		rcall	LC2CMD
		ldi	R25,$80		; DDRAM-Adress auf 0
		rcall	LC2CMD
		ldi	R25,$01		; Clear Display, Cursor Home
		rcall	LC2CMD
		ldi	R25,2		; Wait 2 msec
		rcall	DELXMS
.Endif

	
;Prepare for bulkloading of characters
		ldi 	ZL,LOW (Ram_Char_def_start)
		ldi 	ZH,HIGH (Ram_Char_def_start)
		mov 	R25,C0		;Start at char 0
		mov		R26,C0		; and at the top Row
		ldi 	R17,(Ram_Char_def_1 - Ram_Char_def_start)		;Do this # of Bytes characters
		rcall 	LCD_Char_Prog 
		ldi		R16,$87		; 8th programmable character
		sts		GainMemSymbol,R16

; Start-Message fr 20x 100ms anzeigen		
		ldi     ZL,low(2*StrPowerOn)	; Ausgabe-Kommand-String
        ldi     ZH,high(2*StrPowerOn)
		rcall	LCDSTR		; .. ausfhren
.If Processor == 8
		rcall	OUTLCD		; .. und Ergebnis anzeigen
.EndIf
.If Processor == 168
		call	OutLCD
.EndIf
		ldi		ZL,20		; Flash Timer aufsetzen
		sts		FLATIM,ZL	;
		ldi		R25,100		; Wait 100 ms
		rcall	DELXMS

; ++++	Frequenz, Leistung und Batteriespannung laden:

		rcall	GETFRQ		; Aktuelle Frequenz laden
		rcall	CreateGainMemSym ; Create the GainMem Symbol in LCD CGRam

.IF Processor == 8
		rcall 	VfoIsMem	
.EndIf
.If Processor == 168
		call 	VfoIsMem
.Endif	
		rcall	CreateRxXSym	; Create RXA/B symbol
		rcall	MHuntPrep		; Prepare Hunting modes
		rcall	MHuntNextPrev
		rcall	GETPF		; Aktuelle Leistung laden
		rcall	CONVF		; Frequenz Dezimal ausrechnen
		rcall	CONVPF		; Leistung Dezimal ausr$echnen
		rcall	CONVBW		; Batterie-Warnung Dezimal ausrechnen
		lds		R16,FFine
		rcall	CalFreqUpDownDisplay ; Calculate FFIVAL in case we go to cal mode
		lds		R16,UBFine	; Calculate UBVAL in case we go to cal mode
.IF Processor == 8
		rcall 	UBFineUpDown5	
.EndIf
.If Processor == 168
		call 	UBFineUpDown5
.Endif
		rcall	CalAttUpdate5 ;Calculate ABS...in case we go to cal mode
		lds		R16, DistCf	; Calculate current distance correction factor
		rcall	DistCorUpDown15 ; to display in case we go to cal mode

; +++++ Init some registers
		mov		CurrentUDReg,C1		; Set to a default value
		mov		PreviousUDReg, CurrentUDReg ; Keep previous and current in sync
		mov		EncFlgReg, C0		; Make sure no stray Encoderflag value is set in the register

; +++++ And memory locations		
		sts 	SWIFLG,C0			; Drehgeber Flags initialisieren
		sts		EncFlg,C0			; Encoder flags init

; +++++ Prep RxB parameters  
; Prepping RxA is not needed, sfwr starts  with RxA active
		sts		RxBFrqX,C1		; Memory 1
		sts		RxBGainMode,C1	; Gainmode is DGC
		ldi		r16,2
		sts		RxBAbsBin,R16	; 10 dB attenuator setting
		ldi		R16,$A0			; 144.100 MHz
		sts		RxBFrqBin,R16	; Low Byte
		sts		RxBFrqBin+1,C0  ; High byte
		ldi		r16, MHuntFpFreq ; Hunting mode
		sts		RxBMHuntx,R16
		ldi		R16,HMPAGC+HMPFP+AGC+ HMPWatchPrio	; And the properties thereof
		sts		RXBHuntMPropreg,R16

; +++++ Prep RxA/B on time
		lds		R16, RxATimeBin
.IF Processor == 8
		rcall 	TrXaUp5	
.EndIf
.If Processor == 168
		call 	TrXaUp5
.Endif
		lds		R16, RxBtimeBin
.IF Processor == 8
		rcall 	TrXbUp5
		rcall	ReloadRxX	; Load the RxXTimeRegister
.EndIf
.If Processor == 168
		call 	TrXbUp5
		call	ReloadRxX	; Load the RxXTimeRegister
.Endif

; +++++ Set the LCD Background LED brightness
.IF Processor == 8
;		rcall	PortLedDim	; Default to Dim
.EndIf
.If Processor == 168
;		call	PortLEdDim	; Default to Dim
.EndIf
;		lds		R16, LEDBright	; But turn to bright
;		sbrc	R16,0			; if status byte
.IF Processor == 8
;		rcall	PortLedBright	; says so.
.EndIf
.If Processor == 168
;		call	PortLEdBright	; says so
.EndIf


; ++++	Ende der Initialiserung, well...nearly:
		sound	SOUPO			; Startsound abspielen
		rcall	M100ms70		; Start Main Loop with A/D of UBat so display is correct when
;								  starting with the switch up
		

; =============================================================================
; *******************************************************************************************
; Hauptscheife
MAIN:
	
; ++++ Mode Updaten

	lds		R16,MODE	; Mode holen
	sbis	PIND,6		; Wenn Schalter nicht auf Men (Freq)
	rjmp	Main15

; Schalter in Stellung: Peilen...

	sbic	PinC,4		; When RX power is off
	rjmp 	Main5
	ldi		R16,MRxOff 	; Load the code for Rx hardware OFF mode
	rjmp 	Main20		; Done with mode change processing
Main5:					; Rx is on here
	cpi		R16, MHuntStop+1 ; When mode > any hunting mode
	brlo	Main10
	sts		Flatim,C0	; Stop any flash message
	lds		R16,GainmodeSaved ; get saved gain mode back
	sts		GainMode,R16
	rcall	CreateGainSym	; And update the display
Main10:
	lds		R16,MHuntx
	rjmp 	Main20			; Done with mode change processing
	
; Schalter in Stellung: Men

Main15:					; Was it in this position already?
	cpi		R16,MHuntStop+1 ; Mode > any hunting mode
	brsh	Main20			; Yes, then done with mode change processing
							; Switch was just put in Menu mode
	lds		R16,GainMode	; Save currently selected gain mode
	sts 	GainModeSaved,R16
	ldi		R16,MAuxStart	; Get the new mode
	flash	StrModeMenuStart,30	; Start Menu Message

Main20:
	sts		MODE,R16		; Mode speichern

; Je nach Mode: Drehdrcker + Taster behandeln:

	lds		R16,MODE		; Mode holen
	cli						; No interrupts during EncFlg and Swiflg processing
	lds		SwitchFlgReg,SwiFlg	; Switch Flags holen
	sts		SWIFLG,C0	;	und bis zum nchsten Interrupt lschen
	lds		EncFlgReg, EncFlg ; Copy Encoder state to the register
	sts		EncFlg, C0		; And clear it in Ram
	sei					
	tst		EncFlgReg		; Drehgeber bettigt?
	breq	Main25			; nein...
	sts		FLATIM,C0		; ... ja, dann laufende Flashes abbrechen

; Pushbutton switch pressed

Main25:	
	sbrs	SwitchFlgReg,SwitchLongPressBit		; Lang drcken ?
	rjmp	Main45		; nein...

; Long press, change gain mode

	lds		R16,GainMode 	; Get current gain mode
	mov		Xl,C1 			; Min gain mode is 1
	ldi		XH, 2			; Max gain mode is 2
	sbrc	HuntMPropReg,HMPFPBit
	inc		XH				; Max gain mode is 3 in a FP Hunting mode
	rcall 	inclim 			; Cycle through gain modes
	sts		Gainmode,R16 	; Save new gain mode
	cpi		R16,DGC			; Create symbol depending on gain mode
	brne	Main30			; No DGC selected
	ldi		R18, ' '
	sound	SouD			; Send a D

Main30:
	cpi 	R16,MGC			; Create symbol depending on gain mode
	brne 	Main35
	ldi		R18,'*'
	sound 	SOUM			; Send a M

Main35:
	cpi 	R16,AGC			; Create symbol depending on gain mode
	brne 	Main40
	lds		R16,MHuntX		; Set a frequency entry FP hunting mode 
	ori		R16, 0b00000010	; when AGC is turned on
	sts		MHuntX,R16
	ldi		R18,' '
	sound 	SouA			; Send an A

Main40:
	sts		AutoOn,R18		; Store the Gain Symbol (DF1FO/ARDF mode usage only)
	rcall 	CreateGainSym
	rjmp	Main65

; Single Short press of pushbutton?

Main45:	
	sbrs	SwitchFlgReg,SwitchShortPress1Bit	; Kurz drcken
	rjmp	Main65			; nein...

; kurz gedrckt:

	lds		R16,ABSBIN		; = Abschwcher hochschalten
	cpi		R16,9			; Wenn > 40 dB		 
	brcs	Main50
	ldi 	R16,8			; .. dann auf 40 dB
	sts 	ABSBIN,R16
	rjmp	Main55

Main50:	
	ldi 	R16,2
	sts 	ABSBIN,R16

Main55:
	lds		R18,PFOXBN		; Wenn allerdings PFOX <= 7 (3 mW)
	andi	R18,$08
	brne	Main60
	sts 	ABSBIN,C0 		; dann Abschwcher immer gleich auf 0dB

Main60:	
	push	R17
	rcall	CONVAB			; .. neuen Abschwcherwert einstellen
	pop		R17

; Multiple Short presses of pushbutton?

Main65:	
	sbrs	SwitchFlgReg,SwitchShortPress2Bit	; Two short presses in a row??
	rjmp	Main70					; nein...
	sbrs	HuntMPropReg,HMPFPBit	; And in a FP Hunting mode ?
	rjmp	Main70					; No

;  Then toggle RxA/RXB

.IF Processor == 8
		rcall 	ToggleRxABAll	; execute all the little routines for Rx switching	
.EndIf
.If Processor == 168
		call 	ToggleRxABAll	; execute all the little routines for Rx switching
.Endif


; Process any encoder activity

Main70:
	tst		EncFlgReg			; Any Encoder Activity?
	breq	Main85

; Start of state machine

	ldi 	Zl,low(EncoderStateTable*2-2)	; Load the Z register with the Encoder 
	ldi		ZH,high(EncoderStateTable*2-2) ; state table adress
	lsl		EncFlgReg			; Multiply Encoder state by 2
	add		Zl,EncFlgReg		; Add it to the base of the EncoderStateTable
	adc		Zh,C0
	lpm  	Xl,Z+				; Get the address of the wanted Encoder processing table
	lpm		Xh,Z				; to X
	lsr		EncFlgReg			; Restore register
	ldi 	Zl,Low(ModeTab*2)	; Load Z with the ModeOps table
	ldi		Zh,High(ModeTab*2)
	lds		R16,Mode
								; R16 = mode
								; Z = ModeOps table
								; X = Encoder processing table, depends on up/down etc.
	sbiw	Xl,1				; Compensate for adiw later

Main75:
	adiw 	Xl,1				; Increment MenuOpsTab pointer
	lpm		R18,Z+				; Get next mode 
	cpi 	R18,$FF				; End of table reached?
	brne	Main80		
	rjmp	Start				; Yes, Get out, unknown state!

Main80:
	cp		R18,R16				; Mode entry found ?
	brne	Main75				; No keep on looking

	movw	Zh:Zl,Xh:Xl			; Move pointer for MenuOpsTab entry to Z
	lsl		Zl					; And multiply it
	rol		Zh					; with 2

	lpm		Xl,z+				; Get the adress of the routine to X
	lpm		Xh,z+
	movw	Zh:Zl,Xh:Xl			; Transfer the address from X to Z
	icall						; Execute the Mode Operation subroutine pointed to by Z
	
;End of state machine

Main85:	 
	rcall	ADUM				; Messspannung A/D-Wandeln und behandeln
	lds		R16,TICK			; 100ms Flag
	tst		R16					; ...testen
	breq	Main90		
	rcall 	M100ms				; -> 100ms abgelaufen

Main90:
	lds		R16, Tack			; 1 s Flag
	tst		R16					; ...testen
	breq	Main95
	rcall	M1s					; -> 1s abgelaufen

Main95:
	lds		R16, Tock			; 10 s Flag
	tst		R16					; ...testen
	breq	Main100
	rcall	M10s				; -> 10s abgelaufen

Main100:
	rjmp	MAIN


;*********************************************************************************************
; This routine is executed every 100ms

M100ms:							; 100ms-Flag
	sts	TICK,C0					; ...lschen

; Change RF Gain depending on Gainmode parameter, with sound effects

	lds		R16,Gainmode 		;Get current gain mode
	cpi		R16,MGC
	brne	M100ms5
	rjmp	M100ms35			; No changes to gain if in Manual gain mode

; First process gain reduction if needed

M100ms5:
	lds		R17, UmessA			; Get current signal strength
	cpi		R17, UmessAmax		; Exceeding max?
	brcc	M100ms15			; Br if yes
	cpi		R16,AGC				; Is AGC active?
;	brne	M100ms35			; Br if not
	breq	M100ms7
	rjmp  	M100ms35
; AGC is active
M100ms7:
	cpi		R17,UmessAMin 		; Do we have a signal level above min?	
;	brcc	M100ms35			; Yes no need to increase gain
	brcs	M100ms9
	rjmp	M100ms35

; Signal level is too low
M100ms9:
	lds		R16,AGIWait			; Decay time expired?
	tst		R16
	breq	M100ms10			; Yes, increase gain
	dec		R16					; Not yet, decrement timer
	sts		AGIWait,R16			; and save it
	rjmp	M100ms35			; Done for now, get out

M100ms10:						; Increase gain
	lds		R16,ABSBIN			; Get current Attenuator setting
	cpi		R16, 0				; Is gain at Max.?
	brne	M100ms12			
	rjmp	M100ms35			; Yes, do nothing further
M100ms12:
	dec		R16					; Next gain step
	ldi		R17, GainIncrDelayTime ; Get decay time
	sts		AGIWait,R17 		; And reset decay timer
	mov		R17,C1				; Indicate gain went up
	rjmp	M100ms25			; Go and do final Gain change processing

M100ms15:						; Process Decrease Gain
	lds		R16, AAWait			; Attack time expired?
	tst		R16
	breq	M100ms20			; Yes, decr. gain
	dec		R16					; Not yet, just decrement delay counter
	sts		AAWait,R16
	rjmp	M100ms35			; Done with gain processing

M100ms20:						; Decrease gain
	lds		R16,ABSBIN 			; Get current gain setting
	cpi		R16,24				; Is gain at minimum
	breq	M100ms35			; Then get out
	inc		R16					; Next gain value
	ldi		R17,GainDecrDelayTime
	sts		AAWait,R17			; Reset Attack time delay counter

M100ms25:						; Finisch new gain value processing with sound effects
	sts		ABSBIN, R16			; Store new Gain value
	rcall	CONVAB				; .. neuen Abschwcherwert einstellen
;	lds		R16,FLATIM			; .. Wenn Textanzeige nicht aktive
;	tst		R16	
;	brne	M100ms35
	lds		R16,GainMode		; .. and not in the frequency entry mode
	cpi		R16, AGC
	brne	M100ms27
	lds		R16,AGCAudio		; Can we make noise in AGC mode?
	tst		R16
	breq	M100ms26			; No
	sound   Souabss				; Short beeps only when tuning freq.

M100ms26:
	rjmp    M100ms35

M100ms27:
	lds		R16,SOUPNT+1		; .... und wenn Soundgenerator nicht busy
	tst		R16
	brne	M100ms30
	sound	SOUABS				; ...... Zweiton erzeugen
	rjmp	M100ms35

M100ms30:	
	sound	SOUAB2				; ...... Sonst Folge-Zweiton erzeugen
	


; ++++ Alarm-Tonfolge erzeugen, wenn Flag gesetzt
;      und mehr als 1 Fuchs

M100ms35:	
	lds		R17,ALATIM			; Flag holen
	sts		ALATIM,C0			; und lschen
	tst		R17					; Flag > 0?
	breq	M100ms40	
	lds		R18,NFOX+1			; und > 1 Fuchs.. 
	cp		R18,C1
	breq	M100ms40
	sound	SOUALA				; .. dann Tonsignal 
	flash	STRALA,8			; .. und Meldung zeigen

; ++++ ACSM > Schwelle Nachlaufzeit dekrementieren (falls > 0) 

M100ms40:	
	lds		R17,ACSTIM			; ACSM-Timer > 0
	tst		R17
	breq	M100ms45
	dec		R17					; .. dann dekrementieren
	sts		ACSTIM,R17

; ++++ Batterie checken wenn Flag BATTIM gesetzt

M100ms45:	
	lds		R17,BATTIM			; Flag holen
	sts		BATTIM,C0			; und lschen
	tst		R17					; Flag > 0?
	breq	M100ms60	
	;
	lds		R16,UBATT			; Batteriespannung 10V-Stelle holen
	tst		R16					; Wenn > 0: Bat ok
	brne	M100ms60		
	;
	lds		R16,UBATT+1			; Sonst 1V-Stelle holen 	
	lds		R17,UBWARN
	cp		R16,R17		
	breq	M100ms50			; Wenn = Alarmschwelle: nchste Stelle
	brcs	M100ms55			; Wenn < Alarmschwelle: Alarm
	rjmp	M100ms60			; Wenn > Alarmschwelle: Bat ok 
	
M100ms50:	
	lds		R16,UBATT+2			; Wenn 1V-Stelle gleich
	lds		R17,UBWARN+1		; .. 100mV-Stelle holen
	cp		R16,R17				; Wenn > Alarmschwelle: Bat ok
	brcc	M100ms60

M100ms55:	
	flash	STRBAT,50			; Sonst Alarm-Text und -Sound
	sound	SOUBAT

M100ms60:
	lds		R16,FudArm			; When FudArm Flag >0
	dec		R16					; Count down every 100 ms
	brmi	M100ms65
	sts		FudArm,R16

; In Timed Memory capable hunting mode test if FrqTim flag is set

M100ms65:
	lds		R17,FrqTim			; Get the flag
	sts		FrqTim,C0			; and clear it
	tst		R17					; If FrqTim flag is set
	breq	M100ms70
	sbrs	HuntMPropReg, HMPTimedMemBit ; and this a Timed Mem Hunting mode
	rjmp	M100ms70
	ldi		R18,6				; Max mem in FP mode
	sbrs	R17, HMPFpBit 		; When not in FP mode
	ldi		R18,6				; Set max mem for ARDF mode
	lds		R17,FrqX			; Get current mem
	cp		R17,R18				; Equal to max?
	brcc	M100ms70			; yep get out
	rcall	GetCurrentFRQ1x2 	; Then get current required frequency in R16
	cp		R16,R17				; When not equal
	breq	M100ms70
	sts		Frqx,R16			; Set  new frequency
	rcall	NextPrevMem			; and update some more parameters

; ++++ Batteriespannung A/D-konvertieren und skalieren

M100ms70:
	mov		R16,C1				; Analog-Channel 1 = UBATT
	lds		R19,UBFINE			; Scaling Factor, ist $2xxd
	swap	R19	        		;  mit xx = UBFINE und d = dont care
	andi	R19,$0F
	ori		R19,$20
	lds		R18,UBFINE			; Scaling Factor, Low
	swap	R18
	rcall	ANACON
	ldi		Xh,high(UBatt) 		; Pointer auf Anzeige-Fenster		
	ldi		XL,low(UBatt)
	rcall	B3BCD				; Ergebnis in BCD wandeln und ablegen
	rcall	CONVAB				; Abschwcher-Einstellung in Entfernung umrechnen
	rcall	CONVUM				; Messspannung in Bargraph-Anzeige wandeln

; ++++   Je nach Mode Display laden

	lds		R17,FLATIM			; Flash-Timer
	tst		R17					; >0?
	breq	M100ms75			; ja, dann Anzeige erneuern
	dec		R17					; Flash-Timer dekrementieren
	sts		FLATIM,R17
	rjmp	M100ms200			; -> Display nicht ndern!

; LCD-Anzeige auf neusten Stand bringen:

M100ms75:	
	lds		R16,MODE
	ldi		XL,low((SHOWTAB<<1)-2)	; Anfang des Anzeigestring-Arrays
	ldi		XH,high((SHOWTAB<<1)-2)
	ldi		ZL,low(MODETAB<<1)	; Anfang des Moden-Arrays
	ldi		ZH,high(MODETAB<<1)

M100ms80:
	adiw	XL,2				; X = X + 2
	lpm		r18,z+				; Byte aus Mode-Tabelle im Programmspeicher laden
	tst		r18					; Tabellenende erreicht?
	brmi	M100ms85			; ja...
	cp		r18,r16				; Bytes identisch?
	brne	M100ms80			; nein, dann weiter suchen...

; Mode in Tabelle gefunden:

M100ms85:	
	movw	ZH:ZL,XH:XL			; X -> Z
	lpm		XL,z+				; Anzeige-Stringadresse
	lpm		XH,z+				; ...laden
	movw	ZH:ZL,XH:XL			; X -> Z
	tst		ZH
	breq	M100ms90			; Nullvektor -> Sonderbehandlung ntig
	rcall	LCDSTR				; String aufbauen
	rcall	OUTLCD				; .. und Ergebnis anzeigen	
	rjmp	MSond_end

; Sonderbehandlungen bearbeiten:

M100ms90:
	lds		R16, mode			; get mode back	
	cpi		R16,MRxOff
	brne	M100ms95

	MSondRxOff_0:				; Mode Rx Off
		lds		R18,NFOX+1		; Mode RX Aus: Wenn nur 1 Fuchs.. 
		cpi		R18,1
		breq	MSondRxOff_5
		show	StrModeRxOffTimer ; Timer anzeigen
		rjmp	Msond_end

	MSondRxOff_5:	
		show	StrModeRxOffStopW	; Stopuhr anzeigen
		rjmp	Msond_End


M100ms95:						; Mode Hunt ARDF:
	MSondHuntArdf_0:
		cpi		R16,MHuntARDF
		brlt	M100ms100		; Test for ARDF Hunting mode
		cpi		R16, MHuntArdfStop+1
		brge	M100ms100

		lds		R18,NFOX+1		; Mode Peilen: Wenn nur 1 Fuchs.. 
		cpi		R18,1
		breq	MSondHuntArdf_5
		show	StrModeHuntArdfTimer	; Timer anzeigen
		rjmp	MSond_End

	MsondHuntArdf_5:
		show	StrModeHuntArdfStopw	; Stopuhr anzeigen
		rjmp	MSond_End


M100ms100:
	MSondStopw_0:
		cpi		R16,MAuxStopw	; Stopwatch mode ?
		brne	M100ms105		; No go check next special mode
		lds		R18,STOPWR
		tst		R18
		breq	MSondStopw_5
		show	StrModeStopwStop	; "Uhr Stop" anzeigen
		rjmp	Msond_End

	MSondStopw_5:	
		show	StrModeStopwStart	; "UhrStart" anzeigen
		rjmp	MSond_End


M100ms105:
	MSondHuntSelect_0:
		cpi		R16,MAuxHunt	; Hunting mode selection
		brne	M100ms110
		rcall	MHuntPrep		; Get index to R16
		lsl		R16				; Table is word oriented, multiply by 2
		ldi     Zh,high(2*HuntingModesStrings) 	; Base of table
	    ldi     Zl,low(2*HuntingModesStrings) 	; Add
		add		Zl,R16			; Offset
		adc		Zh,C0			; Word wide
		lpm		XL,z+			; Anzeige-Stringadresse
		lpm		XH,z+			; ...laden
		movw	ZH:ZL,XH:XL		; X -> Z
		lsl		Zl				; To byte mode
		rol		Zh

		rcall	LCDSTR			; .. ausfhren
		rcall	OUTLCD			; .. und Ergebnis anzeigen
		rjmp	MSond_End


M100ms110:
		cpi		R16, MAutoRxABSw ; Auto Toggle RXAB on/off mode?
		brne	M100ms115

	MSondAutoRxABSw_0:
		lds		R16, AutoRxABSw
		sbrc   	R16, AutoRxABBit	; Is it currently on ?
		rjmp	MSondAutoRxABSw_5	; yes
		show	StrModeAutoRxABSwOff	; No, it's off
		rjmp	MSond_End

	MSondAutoRxABSw_5:
		show	StrModeAutoRxABSwOn

M100ms115:
;	MsondLEDBright:
;		cpi		R16,MLEDBright	; Change LCD Brightness mode?
;		brne	MSond_25		; Branch if not
;
;		lds		R16,LEDBright
;		tst		R16
;		brne	MSondLEDBright5
;		show	StrModeToggleLEDBrightOn
;		rjmp	MSond_End

;	MSondLEDBright5:
;		show	StrModeToggleLEDBrightOFF
MsondAGCAudio:
		cpi		R16,MAGCAudio ; toggle audio tones during freq entry
		brne	MSond_25		; Branch if not

		lds 	R16, AGCAudio
		tst		R16
		brne	MSondFreqAgcSound5
		show	StrMToggleAGCAudioOFF
		rjmp	MSond_end

MSondFreqAgcSound5:
		show StrMToggleAGCAudioOn
	;	rjmp	MSond_End  Not needed as long as it is the last in the row


	MSond_25:
	Msond_30:
	MSond_End:
	


; ++++   Falls RX Power On und Frequenz gendert: Synthesizer laden

M100ms200:	
	sbis	PINC,4				; RX Power On Flag
	rjmp	M100ms205			; RX ist aus!
	lds		R16,FRQCHG			; Change Frequency Flag gesetzt?
	tst		R16
	breq	M100ms210

; Ja, Frequenz in PLL laden
	ldi		R16,I2CTSA6057		; Address of PLL Chip
	sts		I2CData,R16			; To top of Array
	sts		I2CData+1,C0
	lds		R16,FFINE			; Einstellung fr 143,9 MHz, DB0
	add		R16,R16				; FFINE * 2 + 1	
	inc		R16
	ldi		R17,$40
	ldi		R18,$27				; .. DB2
	lds		R24,FRQBIN			; Frequenzcode holen
	lds		R25,FRQBIN+1
	clc							; Ein Bit links schieben
	rol		R24
	rol		R25	
	add		R16,R24				; und zum Basiswert addieren
	adc		R17,R25
	sts		I2CData+2,R16
	sts		I2CData+3,R17
	ldi		R16,$27
	sts		I2CData+4,R16
	sts		I2CData+5,C0
	ldi		R16,6
	sts		I2cBytes,R16
	rcall	OutI2C			; Ergebnis an Synthesizer
	sts		FRQCHG, C0 			; .. und Flag lschen
	rjmp	M100ms210

M100ms205:	
	sts 	FRQCHG,C1 			; RX Off, Change Frq Flag setzen	

; Calculate the sum of Encoder events over the last 500ms:
; Every time a number >0 events is moved to the next 100ms slot
; Every time it is moved it is multiplied by 2 (LSL)
; Number of events is used to set the stepsize when adjusting the frequency.
; The effect is a kind of dynamic stepsize for freq. adjustment

M100ms210:	

;	lsl		EncoEvent300_400					; Multiply 300-400 ms slot by 2
	mov		EncoEvent400_500,EncoEvent300_400	; Put it in 400-500 ms slot
;	lsl		EncoEvent200_300					; Multiply 200-300 ms slot by 2
	mov		EncoEvent300_400,EncoEvent200_300	; Put it in 300-400 ms slot
	mov		EncoEvent200_300,EncoEvent100_200	; Put it in 200-300 ms slot
	cli											; No interrupts during calculations
	mov		EncoEvent100_200,EncoEvent0_100		; Put 0-100 ms slot in 100-200ms slot
	mov		EncoEvent0_100,C0					; Clear 0-100ms slot	
	sei			
	mov		EncoEventHistory,EncoEvent400_500	; Calculate weighted 
	add		EncoEventHistory,EncoEvent300_400	; sum of events over 
	add		EncoEventHistory,EncoEvent200_300	; the 100ms to
	add		EncoEventHistory,EncoEvent100_200	; 500ms time span


; *****************************************************************************
; Determine if there was a single or double click press on the switch
; The sum of all FiFo elements is calculated and the data is shifted one time
; slot If the sum of all events is >= 2 then we had a "Double Click"
; If the sum is 1 and equal to the last FiFo element then we had a single click.
; If the sum is 0 there was no short press event
; Side rffect is that single short presses on the switch are processed
; with a delay of SwitchPushFifoLength  * 100 ms
;
		cli							; No interrupts during history processing
		ldi		Zh, high(SwitchPushFiFo+SwitchPushFiFoLength-1)	; Z to end of FiFo
		ldi		Zl, low	(SwitchPushFiFo+SwitchPushFiFoLength-1)
		ldi		R18,SwitchPushFiFoLength-1	; Loop Counter
		ld		R17,Z			; Summing register
		push 	R17				; Need last slot later

M100ms215:		
		ld		R16,-Z			; Read previous slot
		add		R17,R16			; Add to running sum
		std		Z+1,r16			; Store it one slot up
		dec		R18				; Loop counter
		brne	M100ms215

		sts		SwitchPushFiFo, C0	; Clear the first time slot
		pop		R16				; Last time slot back
		tst		R17				; Any events in the whole FiFo ?
		breq	M100ms230		; No, done
		cpi		R17,2			; 2 or more events ?
		brlt	M100ms225
		ldi		R16,SwitchShortPress2		; Yes 2 or more so we had a double click
		sts		SwiFlg,R16		; Update the Switch Flag byte;

		ldi		R18,SwitchPushFiFolength ;Loop Counter
		adiw	Z,1				; Z back to start of SwitchPushFiFo array

M100ms220:
		st		Z+,C0			; Clear the SwitchPushFiFo array
		dec		R18
		brne	M100ms220		; Loop until done
		rjmp	M100ms230

M100ms225:						; There was one event in the Fifo
		cp		R17,R16			; Was the one event in the last time slot?
		brne	M100ms230		; No somewhere else, done
		ldi		R16, SwitchShortPress1	; Yes there was 1 event in the last slot
		sts		SwiFlg,R16		; Update the SwiFlg byte
		mov		SwitchFlgReg,R16	; And the register

M100ms230:
		sei



; Soundgenerator abarbeiten
	

	rcall	SOUGEN
	ret							;Done with 100ms processing


; *****************************************************************************
; This code is executed every second
;
M1s:							; Code for 1 sec processing
	sts		TACK,C0				; ...lschen
	lds		R16,soupnt+1 
	tst		R16					; Soundgenerator busy?
	brne	M1s5				; Do not save changes to EEProm this second
	rcall	savchg
M1s5:
	dec		RxXTimeReg 			;Current countdown time
	brne	M1s15				; No action if it's not at zero
		
; Countdown time has expired

	lds		R16,AutoRxABSw
	sbrs	R16,AutoRxABBit		; Is Toggle RxAB active ?
	rjmp	M1s10				; No don't swap, only reload the RxX value
	lds		R16, Mode
	cpi		R16, MFreqEntry		; Currently programming memories ?
	breq	M1s10				; Yes, then only reload the RxX value
	rcall	ToggleRxAB			; Swap RX A<>B

.If Processor == 8
	rcall	ToggleRxABNoise 	; Sound in Morse A or B
.EndIF
.If Processor == 168
	call	ToggleRxABNoise 	; Sound in Morse A or B
.EndIf

M1s10:
	rcall	ReloadRxX			; Reload the RxX variable

M1s15:
	ret							; Done with 1 s processing
	
; *****************************************************************************
; This code is executed every 10 seconds
;
M10s:							; Code for 10 sec processing
	sts		TOCK,C0				; ...lschen
	ret



; *****************************************************************************
; Subroutinen
; *****************************************************************************


; *****************************************************************************
; Akustisches S-Meter erzeugen 
;
; Eingangswert ist UMESSA 0..255, Bar-S-Meter ab 127 am Anschlag
; 
ACSM:	
	lds		R16,UMESSA			; Messpannung holen
	cpi		R16,150				; .. und auf 150 begrenzen 
	brcs	ACSM0
	ldi		R16,150
	;
ACSM0:
	clr		R17
	ldi		Zh,High(ACSMLookupRam);Base of table
	ldi		Zl,Low(ACsmLookupRam)
	lsl		R16					; Table is word oriented	
	adc		R17,c0				; Get carry to R17
	add		Zl,R16				; Add the offset word wide
	adc		Zh,R17
	ld		R17,Z+				; Low Byte
	ld		R18,Z				; High Byte
	;
ACSM7:
	lds		R19,SOUPNT+1		; Ist Sound Pointer <> 0 (Sound aktiv)?
	tst		R19
	brne	ACSM9				; .. dann nichts ausgeben
	;

.If Processor == 8
	in		R16,TCNT1L			; Wait until Counter High Byte is 0
	in		R16,TCNT1H			; .. to avoid noise by counter
.EndIf

.If Processor == 168
	lds		R16,TCNT1L			; Wait until Counter High Byte is 0
	lds		R16,TCNT1H			; .. to avoid noise by counter
.EndIf

	tst		R16					; .. counting to max (< 5ms @ 200 Hz)		
	brne	ACSM7
	
.If Processor == 8
	out		OCR1AH,R18			; Variablen Oscillator laden 
	out		OCR1AL,R17
.EndIf
.If Processor == 168
	sts		OCR1AH,R18			; Variablen Oscillator laden 
	sts		OCR1AL,R17
.EndIf
	
	lds		R16,ACSMON			; Piepser je nach Flag Ein-/Aus-Schalten
	lds		R17,ACSTIM 			; Wenn Nachlaufzeit > 0	..
	tst		R17
	brne	ACSM8
	clr		R16					; .. dann Piepser auf jeden Fall ausschalten
	
ACSM8:	
.If Processor == 8
	out		TCCR1A,R16
.EndIf
.If Processor == 168
	sts		TCCR1A,R16
.EndIf

	sbrc	R16,6				; NF Degate entsprechend schalten
	sbi		DDRB,2
	sbrs	R16,6
	cbi		DDRB,2
	;
ACSM9:
	ret

; *****************************************************************************
; Decrement the Acoustic S meter On threshold level
;
AcSmLvlDown:
	lds		R16, AcSmLv			; Get current Level
	dec		R16	
	rjmp	AcSmLvlUpDown

; *****************************************************************************
; Increment the Acoustic S meter On threshold level
;
AcSmLvlUp:	
	lds		R16,ACSMLV			; Get Current Level
	inc		R16
	
; *****************************************************************************
; Increment/Decrement the Acoustic S meter On threshold level
; Common code
;
AcSmLvlUpDown:
	andi	R16,$07				; Limit to 0..7
	sts		ACSMLV,R16			; Store new value
	ret

; *******************************************************************************************
; Messpannung A/D-wandeln und Maximum speichern
;
ADUM:
	ldi		R16,2				; Analog-Channel 2 = UMESS
	ldi		R19,$AB				; Scaling Factor fr S-Meter Empfindlichkeit
	clr		R18					; .. dto., Low Byte
	rcall	ANACON				; A/D-Wandeln, Ergebnis-Bereich 0..255
	sts		UMESSA,R16			; Momentanwert speichern fr akustische Ausgabe
	lds		R17,UMESSP			; Spitzenwert fr Balken-S-Meter bilden 
	cp		R16,R17
	brcs	ADUM8
	sts		UMESSP,R16
	;
ADUM8:
	lds		R17,UMESSP 			; Wenn Spitzenwert		
	lds		R18,ACSMLV			; .. grsser/gleich ACSM-Schwellwert
	swap	R18	
	cp		R17,R18
	brcs	ADUM9
	ldi		R16,ACSMDwellTime				; .. then reload ACSMHoldTime
	sts		ACSTIM,R16
	;
ADUM9:	
	rcall	ACSM				; S-Meter akustisch ausgeben
	ret

; ********************************************************************************************
; Einen Analogwert Kanal R16 einlesen und in R16 bergeben
;
; 	Scaling Factor in R19,R18 
; 	Benutzt R16 - R22
;

ANACON:	
.If Processor == 8
	out     ADMUX,R16			; ADMUX einstellen
.Endif
.If Processor == 168
	sts		ADMUX,R16
.Endif
	clr		R16					; Wait a little for I/O to settle
AC1:	
	dec		R16
	brne	AC1

.If Processor == 8
	sbi     ADCSRA,6        	; Start ADC
AC2:
	sbis	ADCSRA,4			; Wait for ADIF 
	rjmp	AC2
	sbi		ADCSRA,4			; Clear ADIF

 	in		R16,ADCL			; Get Conversion Results
 	in		R17,ADCH
.endif

.If Processor == 168
; Code still to write
	lds		R16,ADCSRA			; Start ADC
	sbr		R16,6
	sts		ADCSRA,R16

AC2:
	lds		R16,ADCSRA			; Wait for conversion ready
	sbrs	R16,4				; Test Interrupt flag
	rjmp	AC2				
	sts		ADCSRA,R16			; Clear Interrupt flag

	lds		R16,ADCL			; Get the result
	lds		R17,ADCH
.EndIf

	rcall	MPY16U
	mov		R16,R20				; 8 bit unsigned result
	tst		R21					; If Result Overflow..
	breq	AC3
	ldi		R16,$FF				; .. Set Result to max
AC3:	
	ret




; *****************************************************************************
; Select next Auxilary operating mode
;
AuxModesNext:
	ldi		Xl, MAuxStart	; Set limits
	ldi		Xh, MAuxStop
	rcall 	ModesNext		; Get next mode in R16 and stored in Mode
	cpi		R16, MAutoRxABSw ; Auto RxABSwitching mode?
	brne	AuxModesNext5
	sbrs	HuntMPropReg,HMPWatchPrioBit ; Allowed to enable AutoRxAB switching?
	rcall 	ModesNext		; No, advance to next mode

AuxModesNext5:
	ret





; *****************************************************************************
; Select previous Auxilary Operating mode
;
AuxModesPrev:
	ldi		Xl, MAuxStart	; set limits
	ldi		Xh, MAuxStop
	rcall	ModesPrev		; Get next mode in R16 and stored in Mode
	cpi		R16, MAutoRxABSw ; Auto RxABSwitching mode?
	brne	AuxModesPrev5
	sbrs	HuntMPropReg,HMPWatchPrioBit ; Allowed to enable AutoRxAB switching?
	rcall 	ModesPrev		; No, advance to next mode

AuxModesPrev5:
	ret


; *****************************************************************************
; Convert Bin in R16 to 2Byte BCD @ X, X+1
;
B2BCD:
	push	R16
	push	R17		
	push	R18	
	in      R17,SREG        ; Save Statusreg mit Carry Flag
	push	R17
	;
	clr		R17		; Einerstelle
	clr		R18		; Zehnerstelle

B2B1:
	tst		R16		; Bis Bin-Wert auf Null ist
	breq	B2B9
	dec		R16		; Binwert dekrementieren
	inc		R17		; und BCD-Wert inkrementieren
	cpi		R17,10
	brne	B2B1
	clr		R17
	inc		R18
	rjmp	B2B1

B2B9:
	st		X+,R18		; Ergebnis ablegen
	st		X,R17
	sbiw	X,1		; Make sure it also works across page boundaries.
	pop		R17
	out		SREG,R17	; Carry-Flag wiederherstellen	
	pop		R18
	pop		R17
	pop		R16
	ret	
	

; *****************************************************************************
; Convert Bin in R16 to 3Byte BCD @ X, X+1, X+2
;
B3BCD:	
	push	R16
	push	R17		
	push	R18	
	push	R19
	in      R17,SREG        ; Save Statusreg mit Carry Flag
	push	R17
	;
	clr		R17		; Einerstelle
	clr		R18		; Zehnerstelle
	clr		R19		; Hunderterstelle

B3B1:
	tst		R16		; Bis Bin-Wert auf Null ist
	breq	B3B9
	dec		R16		; Binwert dekrementieren
	inc		R17		; und BCD-Wert inkrementieren
	cpi		R17,10
	brne	B3B1
	clr		R17
	inc		R18
	cpi		R18,10
	brne	B3B1
	clr		R18
	inc		R19
	rjmp	B3B1

B3B9:
	st		X+,R19		; Ergebnis ablegen
	st		X+,R18
	st		X,R17
	sbiw	X,2		; Make sure it also works across page boundaries.
	pop		R17
	out		SREG,R17	; Carry-Flag wiederherstellen	
	pop		R19
	pop		R18
	pop		R17
	pop		R16
	ret	

; *****************************************************************************
; Adjust Low Battery voltage alarm level up/down in 100 mV steps
;
BatteryAlarmDown:
	lds		R16,UBWBIN	; get current value
	dec		R16
	rjmp	BatteryAlarmUpDown

BatteryAlarmUp:
	lds		R16,UBWBIN ; get current value
	inc		R16

BatteryAlarmUpDown:
	cpi		R16,UBatteryMax + 1	 ; Exceeded maximum?	
	brne	BatteryAlarmUpDown5
	ldi		R16,UBatteryMax		; Set maximum

BatteryAlarmUpDown5:
	cpi		R16,UBatteryMin-1	; Below minimum
	brne	BatteryAlarmUpDown10
	ldi		R16,UBatteryMin		; Set to minimum

BatteryAlarmUpDown10:
	sts		UBWBIN,R16			; Store new value
	rcall	CONVBW		; Convert Code to Textform
	ret


;******************************************************************************
; Convert 2 byte BCD in X and X+1 to binary in R16
;
BCD2B:
	push	R17
	ld		R16,X+		; Get Zehnerstelle
	lsl		R16		; Multiply by 10
	mov		R17,R16	
	lsl		R16	
	lsl		R16
	add		R16,R17
	ld		R17,X		; Get Einerstelle
	add		R16,R17		; And add to result
	sbiw	X,1		; Make sure it also works across page boundaries.
	pop		R17	
	ret

;******************************************************************************
;	Calibrate Attenuator (RFGain control)
;
CalAttPrep:					; Prepare for changes
	ldi		YL,low(ABSTAB)	; Tabel with PWM values for every 10 db 
	ldi		YH,high(ABSTAB)
	lds		R16, ABSBIN		; Current selected value.  0=0 db, 1= 5 dB, 2= 10 db 24 = 120 dB
	add		Yl, R16			; Calculate offset in ABSTABle
	adc		Yh, C0			; Word wide
	ld		R16, Y			; Get the selected value for further processing in R16
	ret

CalAttUp:					; Increase the current pwm value
	rcall	CalAttPrep		; Prepare for it
	cpi		R16,$FF			; Are we at max?
	breq	CalAttUp5
	inc		R16				; No, PWM value up.

CalAttUp5:
	rjmp	CalAttUpDown	; Common part of routine

CalAttDown:					; Decrease the current pwm value
	rcall	CalAttPrep		; Prepare for it
	cp		R16,C0			; Are we at min?
	breq	CalAttDown5
	dec		R16				; No, PWM value down
CalAttDown5:
	rjmp	CalAttUpDown	; Common part of routine

CalAttUpDown:				; Common part of
	st		Y,R16			; Save updated value back in ABSTABle
	rcall	CalAttUpdate	; Update displayed values
	ret
	
CalAttUpdate:
	rcall	CONVABtoDec		; Abschwcher einstellen
	ld		R16,Y			; Aktuellen Analogwert

CalAttUpdate5:
	rcall	CalAttPrep		; Get current Analog value to R16
	ldi		XL,low(ABSVAL)	
	ldi		XH,high(ABSVAL)
	rcall	B3BCD			; .. in 3 BCD in ABSVAL wandeln
	rcall	ADUM			; S-Meter-Wert D/A-Wandeln
	rcall	CONVUM			; S-Meter updaten
	ret

CalAttUpdateStop:
	ldi		R16, MCalAttn	; Back to CallAttn mode
	sts		Mode,R16
	ret						; And get out


CalAttNextPos:				; Increment to next table position
	lds		R16,ABSBIN		; ABSBIN is 5 dB steps
	inc		R16				
	cpi		R16,25			; bis maximal 24 (*5 = 120)
	brge	CalAttUpdateStop
	sts		ABSBIN,R16		; Save it
	rcall	CalAttUpdate	; And update the display
	ret

CalAttPrevPos:				; Decrement to prev table position
	lds		R16,ABSBIN		; ABSBIN is 5 dB steps
	dec		R16
;	dec		R16				
	cpi		R16,0			; bis minimal 0
	brlt	CalAttUpdateStop
	sts		ABSBIN,R16		; Save it
	rcall	CalAttUpdate		; And update the display
	ret

; *****************************************************************************
; Calibrate frequency. Allows to compensate for freq. inaccuracies of CPU
; Xtal in 1.25 kHz steps.
;
CalFreqUp:
	lds		R16, FFine
	inc		R16
	rjmp 	CalFreqUpDown

CalFreqDown:
	lds		R16, FFine
	dec		R16

CalFreqUpDown:
	sts		FRQCHG,C1	; Change Frq Flag setzen	
	cpi		R16,64+25+1		; Oberes Limit +25 ?
	brne	CalFreqUpDown5
	ldi		R16,64+25

CalFreqUpDown5:
	cpi		R16,64-25-1		; Unteres Limit -25 ?
	brne	CalFreqUpDown10
	ldi		R16,64-25

CalFreqUpDown10:
	sts		FFINE,R16	; Aktuellen Wert speichern
	rcall	CalFreqUpDownDisplay ; Update the display @ FFIVAL
  	ret

CalFreqUpDownDisplay:
	ldi		R17,64		; Anzeigewert ist FFINE - 64
	sub		R16,R17		; .. = -25 .. +25
	ldi		R17,'+'		; Vorzeichen bestimmen
	sbrc	R16,7
	ldi		R17,'-'
	sts		FFIVAL,R17	; .. und ablegen
	sbrc	R16,7		; Wenn Wert negativ
	neg		R16		; .. komplementieren 
	ldi		XL,low(FFIVAL+1)	; 
	ldi		XH,high(FFIVAL+1)
	rcall	B2BCD		; .. in 2 BCD in FFIVAL+1,2 wandeln
	ret

; *****************************************************************************
; Select next Calibration mode
;
CalModesNext:
	ldi		Xl, MCalStart
	ldi		Xh, MCalStop
	rjmp 	ModesNext

; *****************************************************************************
; Select previous Calibration mode
;
CalModesPrev:
	ldi		Xl, MCalStart
	ldi		Xh, MCalStop
	rjmp 	ModesPrev


; ********************************************************************
; GetLookup  # of synthesizer channels to change in table
;
ChannelsUpDown:
	mov		R16,EncoEvent0_100 ; last 100 ms
	add		R16,EncoEventHistory ; 100-400 ms back
	cpi		R16,17			; limit to 16
	brlo	ChannelsUpDown5
	ldi		R16,16

ChannelsUpDown5:
	ldi		Zh, high(Ram_SynthChannelStepsStart-1) 	; Z is start of table
	ldi		Zl, low(Ram_SynthChannelStepsStart-1)	; But min value in R16 = 1
	add		Zl,R16		; Add word wide offset in table
	adc		Zh,C0
	ld		R16,Z		; Get data out of table
	ret


; *****************************************************************************
; Clear the Encoder History array
;
ClearEncHistory:
	mov 	EncoEvent0_100, C0
	mov 	EncoEvent100_200, C0
	mov 	EncoEvent200_300, C0
	mov 	EncoEvent300_400, C0
	mov 	EncoEvent400_500, C0
	mov		PreviousUDReg,CurrentUDReg ; Save new direction
	ret


; *******************************************************************************************
; Fuchs Timer auf Start-Wert setzen
;
CLRTIM:	
	push	R16	
	cli					; Interrupt unterdrcken
	lds		R16,TFOXS	; Zeit von TFOX nach TIMER kopieren
	sts		TIMER,R16
	lds		R16,TFOXS+1
	sts		TIMER+1,R16
	lds		R16,TFOXS+2
	sts		TIMER+2,R16
	lds		R16,TFOXS+3
	sts		TIMER+3,R16
	sei					; Interrupt wieder an 	
	pop		R16
	ret


; *****************************************************************************
; Convert Attenuator setting to distance
;	
CONVAB:	
	rcall	ConvABtoDec

; ++ Entfernungsanzeige

	lds		R16,ABSBIN	; Abschwcher Code holen
	lds		R17,PFOXBN	; Korrekturfaktor Sendeleistung holen
	sub		R16,R17
	ldi		R17,7		; Basiskorrekturfaktor, +/-1 verringert/erhht
	add		R16,R17		; geschtzte Entfernung um Faktor 1,8 (5 dB)
	lds		R17,DistCf	; Add correction factor from Cal menu (-5..+5)
	add		R16,R17
	brpl	ConvAB5		; Wenn Ergebnis negativ
	ldi		R16,0		; .. auf 0 setzen

ConvAB5:
	cpi		R16,21		; Wenn Ergebnis > 20
	brcs	ConvAB10
	ldi		R16,20		; .. auf 20 setzen

ConvAB10:
	lds		R17,ABSCHW+1	; Default: Abschwcher-dB-Wert Anzeigen
	lds		R18,ABSCHW+2
	ldi		R19,$80		;   dB Symbol
	lds		R20,PFoxBn	; When PFoxBn = PFoxUnknown
	cpi		R20,PFoxUnknown		; Then always display dB's
	brne	ConvAB15
	lds		R16,Abschw		; When > 100 dB
	tst		R16
	breq	ConvAB15
	lds		R17,Abschw ; The 3 digit display without trailing dB sign
	lds		R18,Abschw+1
	lds		R19,Abschw+2
	clr		R16

ConvAB15:
	cpi		R16,4		; Leave dB display for first 4 entries
	brlo 	ConvAB20
	clc
	sbci	R16,4		; subsctract offset of 4
	mov		R17,R16
	lsl		R16			; Multiply by 2
	add		R16,R17		; R16 is now multiplied by 3
	ldi		Zh, high(DistanceTable*2)	; Start of Data table
	ldi		Zl, low(DistanceTable*2)
	add		Zl,R16		; Add offset in table
	adc		Zh,C0		; Word wide
	lpm		R17,Z+		; Get the data
	lpm		R18,Z+		; to 
	lpm		R19,Z+		; the 3 registers

ConvAB20:
	sts		DIST+0,R17	; Entfernungsanzeige speichern	
	sts		DIST+1,R18
	sts		DIST+2,R19	

; Abschwcher-Spannung aus Tabelle holen

	lds		R16,ABSBIN	; Abschwcher-Code
	ldi		XL,low(ABSTAB)	; ist Pointer in Tabelle
	ldi		XH,high(ABSTAB)
	add		XL,R16
	ld		R18,X+		; Analogwert holen

.If Processor == 8
	out		OCR2,R18	; Analogwert an PWM bergeben
.EndIf
.If Processor == 168
	sts 	OCR2A,R18
.EndIf
;
; Bei Abschwchung ab 65 dB Vorstufe ausschalten, sonst an
;
	lds		R16,ABSBIN	; Abschwcher-Code
	cpi		R16,13		;   >= 65 dB?
	brcc	ConvAB25
	sbi		DDRC,0		; Nein, Vorstufe Ein
	rjmp	ConvAB30

ConvAB25:
	cbi		DDRC,0		; Ja, Vorstufe Aus	
	;
ConvAB30:
	ret	

	
; *******************************************************************************************	
; Abschwchercode in Anzeige Dmpfung/Entfernung umrechnen
;
ConvABtoDec:
	lds		R16,ABSBIN	; Abschwcher Code holen
	mov 	R17,R16		; .. und mit 5 malnehmen
	lsl		R17
	lsl		R17
	add		R16,R17
	ldi		XH,high(ABSCHW)		; In BCD wandeln fr Abschwcheranzeige
	ldi		XL,low(ABSCHW)
	rcall	B3BCD
	ret


; *******************************************************************************************
; Batterie-Warnungs-Level Bin nach Dez wandeln
;
CONVBW:	
	lds		R16,UBWBIN	; Binrwert holen
	clr		R17		; Volt-Stelle
	clr		R18		; Zehntel-Volt-Stelle

CVBW1:
	tst		R16		; Loopen bis Binrwert auf 0 gezhlt
	breq	CVBW9
	dec		R16		; Binrwert dekrementieren
	inc		R18		; Dezimalwert inkrementieren
	cpi		R18,10		; bertrag handeln
	brne	CVBW1
	clr		R18
	inc		R17
	rjmp	CVBW1

CVBW9:
	sts		UBWARN,R17	; Dezimalwert ablegen
	sts		UBWARN+1,R18
	ret	


; *******************************************************************************************
; Dezimale Frequenzanzeige vorbereiten 2B FRQBIN -> 4B Freq ****
;
CONVF:
	lds		R24,FRQBIN	; Binres Frequenzcodewort holen
	lds		R25,FRQBIN+1		

; 3 LSB aus Frequenz separieren fr 1 kHz Anzeige

	clc
	clr		R17		; Separierte Bits nach R17
	ror		R25
	ror		R24
	ror		R17
	ror		R25
	ror		R24
	ror		R17
	ror		R25
	ror		R24
	ror		R17
	clc			; Separierte Bits rechtsbndig machen
	ror		R17
	swap	R17
	sbrc	R17,2		; Wenn Wert >= 4
	inc 	R17			; 1 dazu wg 1,25 kHz Abstimmschritt
	sts		Freq+3,R17	; Unterste Stelle fertig	
						; ...Rest ist im Bereich 0..220
	ldi		R16,3		; 1MHz Startwert
	ldi		R17,9		; 100 kHz
	mov		R18,C0	; 10 kHz

CONVF4:	
	tst		R24			; Frequenz ist Startwert + Rest * 10 kHz
	breq	CONVF6

CONVF5:	
	dec		R24
	inc		R18
	cpi		R18,10
	brne	CONVF4
	clr		R18
	inc		R17
	cpi		R17,10
	brne	CONVF4
	clr		R17
	inc		R16
	rjmp	CONVF4

CONVF6:
	tst		R25		; Fr grosse F ist bertrag 1 in R25
	breq	CONVF7		
	clr		R25		; .. dann nochmal 256 Increments
	rjmp	CONVF5 

CONVF7:
	sts		Freq,R16
	sts		Freq+1,R17
	sts		Freq+2,R18
	ret


; *******************************************************************************************
; Textstring fr Fuchsleistung in PFOX stellen 
;
Convpf:
	push	R16
	push	R17
	push	Xl
	push	Xh
	push	Zl
	push 	Zh
	lds		R16, PFoxBn			; Get Bin code for power
	lsl		R16	 				; PFOX string is 2 words long\
	lsl		R16
	ldi		Zl,low(PFoxStrings*2)	; Get address of table
	ldi		Zh,high(PFoxStrings*2)
	add		Zl,R16				; Add index*2
	adc		Zh,C0
	ldi		Xl,low(PFox)		; load X with Destination
	ldi		XH,high(PFox)
	ldi		R16,4				; Counter

ConvPf5:
	lpm		R17,Z+				; Get data
	st		X+,R17				; And store it
	dec		R16					; Next position
	brne	ConvPf5			; Done ?
	pop		Zh
	pop		Zl
	pop		Xh
	pop		Xl
	pop		R17
	pop		R16
	ret


; *******************************************************************************************
; Messspannung UMESSP in Bargraph UMESS(0..7) wandeln 
;
; Anzeige hat 8*4 = 32 Bars, Angezeigt wird UMESSP/4, also
; nur der Berreich 0..127 (Akustisches S-Meter geht bis 255)
;
CONVUM:
	lds		R17,UMESSP	; Spitzenwert Messspannung in R17 bilden
	lds		R16,UMESSP+1
	cp		R17,R16
	brcc	CONVUM01
	mov		R17,R16			

CONVUM01:
	lds		R16,UMESSP+2
	cp		R17,R16
	brcc	CONVUM02
	mov		R17,R16			

CONVUM02:
	lds		R16,UMESSP+3
	cp		R17,R16
	brcc	CONVUM09
	mov		R17,R16			

CONVUM09:
	lds		R16,UMESSP+2	; Spitzenwert-Buffer rechts schieben
	sts		UMESSP+3,R16
	lds		R16,UMESSP+1
	sts		UMESSP+2,R16
	lds		R16,UMESSP
	sts		UMESSP+1,R16
	sts		UMESSP, C0
	lsr		R17			; 0..127 fr Anzeige auf 0..31 skalieren
	lsr		R17
	ldi		R16,8		; Stellenzhler
	ldi		XH,high(Umess)		; Pointer auf Bargraph
	ldi		XL,low(UMESS)
	dec		R17			; Um 1 verringern auf -1 .. 30

CONVUM1:
	ldi		R19,$20		; Blank ist Bargraph-Wert 0
	tst		R17			; Wenn Anzeigewert < 0
	brmi	CONVUM9		;   .. diese Stelle Blank
	ldi		R19,$85		; Bargraphwert auf 4 voreinstellen
	cpi		R17,$04		; Wenn Anzeigewert < 4
	brcc	CONVUM9
	mov		R19,R17		; .. Bargraphwert ist 2 LSb von Anzeigewert
	andi	R19,$03
	ldi		R18,$82		; .. + $82
	add		R19,R18

CONVUM9:
	st		X+,R19		; Bargraphwert speichern
	ldi		R18,4		; Anzeigewert um 4 verringern
	sub		R17,R18
	dec		R16		; Loopcount, 8 Passes
	brne	CONVUM1
	ret

; *******************************************************************************************
; Memorysymbol erzeugen und speichern at position 7
;
CreateGainMemSym:
	rcall	CreateGainSym
	rcall	CreateMemSym7
	ret	


; *******************************************************************************************
; Gainsymbol creation at top of character position 7
;
CreateGainSym:	
	push	R17
	lds		R16,GainMode	; GainMode-Index laden
	dec		R16				; Gainmode is 1, 2 or 3, table starts at 0
	lsl 	R16				; Multiply by 4
	lsl 	R16
	ldi 	ZL, low(Ram_GainMode_Def)	;Start of Def
	ldi 	ZH, high(Ram_GainMode_Def)
	add		ZL,R16			; Add Offset
	adc		ZH,C0
	ldi 	R17, 4			; Character is 4 rows high
	clr		R26				; Start at the first row
	ldi 	R25, 7			; Program Character 7
	rcall	LCD_Char_Prog 	; Go and program the character
	pop 	R17
	ret


; *****************************************************************************
; Create the Memory Symbol at the bottum of the 8th programmable character
;
CreateMemSym7:
	ldi		R25,7		; 8 th character
	lds		R16,FrqX	; Index
	rcall	CreateSym
	ret


; *****************************************************************************
; Create the Rx symbol A or B
;
CreateRxXSym:
	ldi		R16,'A'		; Default is A
	sbrc	RxXReg,0	; When bit 0 is set 
	inc		R16			; to B
	sts		RxXSymbol,R16
	ret
	
		
; *******************************************************************************************
; Mem or Rx symbol creation at bottum of character position (R25)
;
CreateSym:	
	push	R17
	lsl 	R16				; Multiply index by 4
	lsl 	R16
	ldi 	ZL, low(Ram_MemNumDef)	;Start of Def
	ldi 	ZH, high(Ram_MemNumDef)
	add		ZL,R16			; Add Offset
	adc		ZH,C0
	ldi 	R17, 4			; Character is 4 rows high
	ldi		R26, 4			; Start at the 4th row
	rcall	LCD_Char_Prog 	; Go and program the character
	pop 	R17
	ret


; *****************************************************************************
; Debounce encoder Pushbutton every 8 ms.
;
DebEncPush:
	PUSH	R20				; Used as Temp Register
	lds		R20,EncEvent	; Did we had a encoder event with a pressed switch?
	cp		R20,C0
	breq	DebEncPush2		; No we did not, so go and debounce the switch of the encoder
	sbic	PinB,4			; Encoder Switch still pressed?
	sts		EncEvent,C0		; Switch was released, clear the encoder event up/down flag
	sts		DebDD,C0		; Allways set the debounce counter for the Encoder switch to 0
	rjmp	DebEncPush15	; so it will not trigger and get out

DebencPush2:
	LDS		R20,DEBDD		; Timer holen
	SBIC	PINB,4			; Wenn Bit  nicht auf 0
	rjmp	DebEncPush5		; ..then the encoder switch is not pressed (anymore)
	;						 so go and check if we had a short press
	; Lang drcken handeln
	CPI		R20,LongPress	; Counter schon am Limit (Longpress * 8ms)?
	BReq	DebEncPush15	; dann nix tun
	INC		R20				; Counter hochzaehlen
	STS		DEBDD,R20		; und retten
	CPI		R20,LongPress	; Jetzt am Limit?
	BRNE	DebEncPush15	; No,,,Then get out

	ldi		R20,6			; EncFlg is set to
	sts		EncFlg,R20		; Longpress value	
	rjmp	DebEncPush15	; and get out

; Kurz drcken handeln

DebEncPush5:
	cpi		R20,Shortpress	; War Taster Shortpress*8ms - Longpress*8ms pressed?
	brcs	DebEncPush10 	; less then Shortpress so done
	cpi		R20,Longpress	; Longpress*8 ms ?
	breq	DebEncPush10	; yes, done
	ldi		R20,EncShortPress1	; EncFlg is set to 
	sts		EncFlg,R20 		; EncShortPress1 value

DebEncPush10:	 	
	sts		DEBDD,C0		; Clear the Debounce counter

DebEncPush15:
	POP		R20
	RET


; *****************************************************************************
; Debounce Taster (alle 8 ms)
;
DEBNC2:	
	PUSH	R20		; Used as Temp Register

DEB4:
	LDS		R20,DEBST	; Timer holen
	SBIC	PIND,7		; Wenn Bit (SU) nicht auf 0
	rjmp	DEB41		; .. dann Taster offen

; Lang drcken handeln

	CPI		R20,LongPress	; Counter schon am Limit (1 sec)?
	BReq	DEB8		; dann nix tun
	INC		R20		; Counter hochzaehlen
	STS		DEBST,R20	; und retten
	CPI		R20,LongPress	; Jetzt am Limit?
	BRNE	DEB8
	LDi		R20,SwitchLongPress      ; Dann Flag setzen
	STS		SWIFLG,R20	; 	 	
	rjmp	DEB8

; Kurz drcken handeln

DEB41:
	cpi		R20,Shortpress		; War Taster 80 - 1000 ms gedrckt?
	brcs	DEB42 
	cpi		R20,Longpress
	breq	DEB42
 	lds		R20,SwitchPushFiFo	; Store the short press event
	inc		R20				; at the input in 100ms interval FiFo
	sts		SwitchPushFiFo,R20

DEB42:
	sts		DEBST,C0

DEB8:
	POP		R20
	RET


; ********************************************************************************************
; Drehgeber und Taster entprellen
;
; SWIFLG ist: Bit 0=Up, 1=Down, 2=Up + Pressed, 3=Down + Pressed,
;		  4=Drehdrcker Puls
;		  5=Taster Kurz, 6=Taster Lang, 7 = Drehdrucker Long
; Same as encflg
; Ergebnis in SWIFLG wird von Main auf $00 gesetzt.
;
; ++ Debounce Drehgeber Up/Down (alle 1 ms)

DEBNCE:	
	PUSH	R20		; Used as Temp Register
	push	R21		; Used as Temp Register
	LDS		R20,DEBP1 	; Debounce-Counter for rotary encoder
	SBIC	PINC,3		; P0 signal of Encoder
	RJMP	DEB1		; 
	CPI		R20,0		; Debounce counter at 0?
	BREQ	DEB3		; Do nothing
	DEC		R20			; Not yet so Counter runterzaehlen	
	STS		DEBP1,R20	; And save it for next time around
	CPI		R20,0		; Jetzt auf Null?
	BRNE	DEB3		; Do Nothing
	SBIC	PINB,0		; P1 signal of Encoder
	RJMP	DEBINC		; Increment
	RJMP	DEBDEC		; Decrement

DEB1:
	CPI		R20,2		; An oberem Limit (5.5 ms)?
	BREQ	DEB3		; Do Nothing
	INC		R20			; Increment debounce counter
	STS		DEBP1,R20	; And save it for next time around
	CPI		R20,2		; Jetzt am oberen Limit?
	BRNE	DEB3		; Do nothing
	SBIC	PINB,0		; P1 signal of decoder
	RJMP	DEBDEC		; decrement
	RJMP	DEBINC		; Increment

DEBINC: 
	mov		R21, C1				; Set Encoder Flag to 1
	mov		CurrentUDReg,R21	; Save current direction
	RJMP	DEB2

DEBDEC:
	ldi		R21,2
	mov		CurrentUDReg,R21	; Save Current directon

DEB2:	
	CPSE	CurrentUDReg,PreviousUDReg ; Has up/down direction changed?
	rcall   ClearEncHistory	; Yes, clear encoder pulses history
	inc		EncoEvent0_100	; No, increment events in the current 100ms timeslot

	SBIc	PINB,4			; When Encoder switch is pressed
	rjmp	Deb2_2
	inc		R21				; Increment EncFlg (encoder flags) with 2
	inc		R21
	sts		EncEvent,C1		; Indicate  switch pressed and up/down event occurred

Deb2_2:
	sts		EncFlg,R21		; 

DEB3:
	POP		R21
	Pop		R20
	Ret


; *****************************************************************************
; Decrement R16 within the limits in Xl and Xh. Does wrapping. Carry is clear
; when Xl was not reached, Carry is set when wrapping back to Xh has occurred
;
DECLIM:	
	cp		R16,XL	; Am unteren limit?
	breq	DECL1
	dec		R16		; Nein: dekrementieren
	clc				; Und Carry Flag lschen
	rjmp	DECL9

DECL1:	
	mov		R16,XH	; Ja: auf oberes Limit setzen
	sec				; Und Carry Flag setzen

DECL9:	
	ret	


; *****************************************************************************
; Decrement the active fox number
;
DecrFox:
	d2bcd	FOX,1,10
	ret

; ********************************************************************************************
; Delay x ms, x in R25 (x * 5 * 256 * 3 Cycle / 4 MHz)
;
DELXMS:	
	PUSH	R16
	PUSH	R17
	PUSH	R25

DXMS:
	CLR		R16
	LDI		R17,5		

D1MS:
	INC		R16		; 3 cycle inner loop
	BRNE	D1MS		; " "     "     " 	
	DEC		R17
	BRNE	D1MS
	DEC		R25
	BRNE	DXMS
	POP		R25
	POP		R17
	POP		R16
	RET


; *****************************************************************************
; Distance correction factor down in steps of 5 dB
;
DistCorDown:
	lds 	R16, DistCf	; get current value
	dec		R16
	rjmp	DistCorUpDown

; *****************************************************************************
; Distance correction factor up in steps of 5 dB
;
DistCorUp:
	lds 	R16, DistCf	; get current value
	inc		R16
	rjmp	DistCorUpDown

; *****************************************************************************
; Distance correction factor up/down
;
DistCorUpDown:
	cpi		R16, -6		; Limit to -5
	brne	DistCorUpDown5
	ldi		R16, -5

DistCorUpDown5:
	cpi		R16, 6		; Limit to +5
	brne	DistCorUpDown10
	ldi		R16, 5

DistCorUpDown10:
	sts		DistCf,R16	; Save new value
	tst		R16			; Determine sign
	brmi	DistCorUpDown20

DistCorUpDown15:		; Entry point during init
	ldi		R17, '+'	; >0  >> +
	rjmp	DistCorUpDown25

DistCorUpDown20:
	ldi		R17, '-'	; must be < 0 so -

DistCorUpDown25:
	sts		DCfVal,R17	; Save sign
	sbrc	R16,7		; If value is negative
	neg		R16			; Invert it
	sts		DCfVal+1,R16	; And store it
	ret


; *****************************************************************************
; EEPromRead, Read bytes from EEProm to Ram
; X = Ram Pointer
; Y	= EEprom Pointer
; Z = # of Bytes to transfer
;
EEPromRead:
		clr		R24				; Bytecounter
		Clr		R25

EEPromRead5:		
		OUT		EEARH,Yh	; High Adresse ist 0
		OUT		EEARL,Yl
		adiw	Y,1
		OUT		EECR,c1	; Set EEPROM Read Cmd

EEPromRead10:
		IN		R17,EECR	; Wait for EEPROM Read complete
		ANDI	R17,$01
		BRNE	EEPromRead10
		IN		R16,EEDR	; Daten lesen
		ST		X+,R16		; Byte bzw low nibble speichern
		adiw	R25:R24,1	; Loopcounter
		CP		R24,Zl		; Alle Bytes gelesen?
		BRNE	EEPromRead5
		cp		R25,Zh
		brne	EEPromRead5
		ret

; *****************************************************************************
; End Of Transmission Alarm Time Down
;
EOTAlarmDown:
	d2bcd TAlarm,0,30
	ret


; *****************************************************************************
; End Of Transmission Alarm Time Up (0-30 sec)
;
EOTAlarmUp:
        ldi		XL,low(TAlarm)		; Pointer auf BCD-String
        ldi		XH,high(TAlarm)
        rcall	BCD2B				; Wandelt 2B BCD @ X,X+1 nach Bin in R16
        mov		XL,C0				; Lower Limit
        ldi		XH,30				; Upper Limit
        rcall	INCLIM				; Increment R16 within limits in XH,XL
        ldi		XL,low(TAlarm)		; Pointer auf BCD-String
        ldi		XH,high(TAlarm)
        rcall	B2BCD				; Wandelt Bin in R16 in 2B BCD @ X, X+1
		ret


;******************************************************************************
; Freq Change by R18 (High Byte) and R16 (Low Byte) # of channels
;
FreqChange:
		lds		R24,FRQBIN		; Low Byte of Frequency Code
		lds		R25,FRQBIN+1	; High Byte
		add		R24,R16			; Add weighted Encoder history
		adc		R25,R18
		tst		R25				; Wenn Ergebnis negativ
		brpl	FreqChange5		
		clr		R24				; .. auf 0 setzen
		clr		R25

FreqChange5:

.Ifdef FBER68			; Conditional assembly, see FBER68 def at top
						;
	adiw	R24,$20		; Fr 146: Wenn Ergebnis > 1760 = 06E0	
	cpi		R25,$07
	brcs	FreqChange10
	clr		R24		; .. auf 1760 setzen
	ldi		R25,$07

FreqChange10:
	sbiw	R24,$20


.Else

	sbiw	R24,$20	;	; Fr 148: Wenn Ergebnis > 3360 = 0D20	
	brcs	FreqChange10 ; .. Wenn Ergebnis < 20 nix tun
	cpi		R25,$0D	
	brcs	FreqChange10
	clr		R24		; .. auf 3360 setzen
	ldi		R25,$0D

FreqChange10:
	adiw	R24,$20

.EndIf
	sts		FRQBIN,R24	; Ergebnis abspeichern als Binrwert	
	sts		FRQBIN+1,R25
	rcall 	CONVF
	sts		FRQCHG,C1
	rcall	vfoismem
	ret



; *****************************************************************************
; Freq down by (Encoder History) channels of 1.25 kHz.
;
FreqDown:
;	mov		R16,EncoEvent0_100		; Get # of events in current 100ms slot
;	add		R16,EncoEventHistory	; Add to previous 100-400ms slots
	rcall	ChannelsUpDown		; Get weighted # of channels
;	andi	R16,127			; Strip of MSB so we can do proper
	neg		R16				; negative math
	ldi		R18,$FF			; Make MSB negative
	rjmp	FreqChange



; *****************************************************************************
; FreqDownARDF12_3
;
FreqDownArdf12_3:
	lds		R16,FrqX	; Get current freq
	cpi		R16,3		; 
	brlt 	FreqDownArdf12_3_5 ; Br if < 3
	lds		R16,Frq512	; Get saved frequency back
	rjmp 	FreqDownARDF12_3_10

FreqDownARDF12_3_5:
	ldi		R18, 0b00000011		; Toggle frequency
	eor		R16,R18		; 1 <> 2
	sts		Frq512,R16	; Save the new frequency		

FreqDownARDF12_3_10:
	sts		FrqX,R16	; Make it active
	rcall	NextPrevMem	; Update other parameters
	ret


; *****************************************************************************
; Mem down in ARDF2_5 mode
;
FreqDownArdf2_5:
	rcall	GetCurrentFrq1x2 ; Get required frequency for currently active fox in R16
	lds		R18,FrqX		; Current frequency
	cpi		R18,3
	breq	FreqDownArdf2_5_10	; Restore stored value
	ldi		R18,0b00000011	; Toggle 1<>2
	eor		R16,R18
	st		Z,R16			; Store new value back in FRQ1x2 table

FreqDownArdf2_5_10:	
	rcall	FreqDownARDF12_3_10
	ret


; *****************************************************************************
; Freq up by (Encoder History) channels of 1.25 kHz.
;
FreqUp:
;	mov		R16,EncoEvent0_100
;	add		R16,EncoEventHistory
	rcall	ChannelsUpDown		; Get weighted number of channels
;	andi	R16,127			;Strip of MSB

FreqUp5:
	mov		R18,c0			; Clear MSB
	rjmp	FreqChange



; *****************************************************************************
; Change RF gain, check for limits 
;
GainChange:
	cpi		R16,-1		; Unterlauf?
	brne	GainChange5	; nein...
	ldi		R16,0		; -> auf minimal 0 begrenzen

GainChange5:
	cpi		R16,25		; berlauf?
	brne	GainChange10; nein...
	ldi		R16,24		; -> auf maximal 24 begrenzen

GainChange10:
	sts		ABSBIN,R16
	ret


; *****************************************************************************
; Decrease RF gain 
;
GainDown:
	lds 	R16,ABSBIN
	inc		R16			; Increment ABSBIN with 1
	rcall	GainChange
	ret

; *****************************************************************************
; Increase RF gain
;
GainUp:
	lds		R16, ABSBIN		
	dec		R16
	rcall	GainChange
	ret


; *******************************************************************************************
; Get the frequency for the currently active fox to R16
;
GetCurrentFrq1x2:
	ldi		ZH,High(Frq1x2)	; Pointer auf FRQ1X2-Entry bauen
	ldi		ZL,Low (Frq1x2)
	lds		R16,Fox+1			; Add fox number
	add		ZL,R16			;  as offset
	adc		Zh,C0			; Word wide
	ld		R16,Z			; .. und Entry lesen
	ret


; *******************************************************************************************
; Frequenz aus Array holen FRQ1..6 -> FRQBIN
;	
GETFRQ:	
	lds		R16,FRQX	; Pointer auf aktive Frequenz bauen
	dec		R16		; 0..5
	ldi		YL,LOW(FRQ1); Anfang des Frequenz-Arryas	
	clr		YH
	add		YL,R16		; Wort-Pointer
	add		YL,R16		; ...erzeugen
	ld		R16,Y+		; Frequenz (H) laden
	sts		FRQBIN,R16	; ...und aktuell speichern
	ld		R16,Y		; Frequenz (L)
	sts		FRQBIN+1,R16	; ...speichern
	ret


; *******************************************************************************************
; Fuchsleistung aus Array laden
;
GETPF:
	lds		r16,FRQX	; Index auf aktive Frequenz laden (1-6)
	dec		r16			;0..5
	ldi		YL,LOW(PFOX1)	; Anfang des Leistungs-Arrays
	ldi		YH,High(Pfox1)
	add		YL,r16		; Byte-Pointer erzeugen
	adc		Yh,c0
	ld		r16,y+		; Leistungs-Array auslesen
	sts		PFOXBN,r16	; ...und als aktuellen Wert speichern
	ret

; *****************************************************************************
; Send Byte in R16 on the I2C buss and wait for Ack

I2SEND:	
	PUSH	R16
	PUSH 	R20
	LDI		R20,8	; Loopcounter bis 8

I2SE1:
	TST		R16		; Get MSb to N-Flag
	BRPL	I2SE2	; Put MSb on SDA
	SDA_HI
	RJMP	I2SE3

I2SE2:	
	SDA_LO

I2SE3:
	SCL_HI		; toggle Clock 
	SCL_LO
	LSL		R16	; Shift next bit to MSb
	DEC		R20	; Loopcount
	BRNE	I2SE1
	SDA_HI		; SDA freigeben fuer Ack
	SCL_HI		; Ack anfordern
	CLR		R0	; Error Flag loeschen

I2SE4:	
	SBIC	PINC,5	; Falls SDA nicht Low ..
	INC		R0	;   .. Error Flag setzen
	SCL_LO		; Ack-Sequenz abschliessen
	POP		R20
	POP		R16
	RET


; *****************************************************************************
; ++++ Send Stop Sequence on I2C
;
I2STOP:
	SDA_LO
	SCL_HI
	SDA_HI
	RET


; *****************************************************************************
; ++++ Send Start Sequence on I2C
;
I2STRT:	
	SDA_LO
	SCL_LO
	RET


; *****************************************************************************
; ---- Wait > 5 usec after any transition - siehe Bus Spec
;
I2WAIT:
	PUSH 	R16
	LDI		R16,8	; 14 fuer 8 Mhz; 7 fuer 4 MHz Clock

I2W1:
	DEC		R16	; 3 Cycle inner Loop	
	BRNE	I2W1	
	POP		R16
	RET


; ********************************************************************************************
; Increment R16 within limits in XL,XH
;
INCLIM:
	cp		R16,XH		; Am oberen limit? (Signed! so keep XH < $80
	breq	INCLIM1
	inc		R16			; Nein: inkrementieren
	clc					; und Carry Flag loeschen
	rjmp	INCLIM9

INCLIM1:
	mov		R16,XL		; Ja: auf unteres Limit setzen
	sec					; Carry Flag setzen

INCLIM9:
	ret


; *****************************************************************************
; Increment the active fox number
;
IncrFox:
        ldi		XL,low(Fox)		; Pointer auf BCD-String
        ldi		XH,high(Fox)
        rcall	BCD2B		; Wandelt 2B BCD @ X,X+1 nach Bin in R16
        mov		XL,C1		; Lower Limit
        ldi		XH,10		; Upper Limit
        rcall	INCLIM		; Increment R16 within limits in XH,XL
        ldi		XL,low(Fox)		; Pointer auf BCD-String
        ldi		XH,high(Fox)
        rcall	B2BCD		; Wandelt Bin in R16 in 2B BCD @ X, X+1
		ret


; *****************************************************************************
; Increment the stopwatch
;
IncrStopW:
	i2bcd	STOPW+3,0,59	; Uhr hochzhlen Sekunden
	brcc	IncrStopW5		; 
	i2bcd	STOPW+1,0,59	; Minuten
	brcc	IncrStopW5
	lds		R24,STOPW		; Stunde
	inc		R24
	andi	R24,$07			; Bei 8 h wrappen
	sts		STOPW,R24	

IncrStopW5:
	ret


; ********************************************************************************************
; Write Data in R25 (High Nibble) to LCD ( 1 Data Shot)
;
LC1DAT:	
	push	R25		; Entry for Data Mode
	swap	R25		
	andi	R25,$0F
	ori		R25,$10
	rjmp 	LC1CM0

LC1CMD:	
	push	R25		; Entry for Command Mode
	swap	R25		
	andi	R25,$0F
	
LC1CM0:
	ori		R25,$C0		; Pull-Ups fuer Taster
	out		PORTD,R25	; Post Data
	nop
	nop
	nop
	nop
	nop
	sbi		PORTD,5		; Toggle Enable Bit
	nop
	nop
	nop
	nop
	nop
	cbi		PORTD,5
	ldi		R25,70		; Wait 40 usec = 200 cycles

LC1CM1:	
	dec		R25
	brne	LC1CM1
	pop		R25
	ret


; *******************************************************************************************
; Multiple Schreiboperation mit gleichen Daten
;
LCREP8:
	rcall	LC2DAT
	rcall	LC2DAT

LCREP6:
	rcall	LC2DAT
	rcall	LC2DAT
	rcall	LC2DAT

LCREP3:
	rcall	LC2DAT
	rcall	LC2DAT

; Write Data in R25 as Data to LCD (2 Nibble Shots)

LC2DAT:
	push	R25
	rcall	LC1DAT		; Send high nibble
	swap	R25
	rcall	LC1DAT		; and low nibble
	pop		R25
	ret


; *******************************************************************************************
; Write Data in R25 as Command to LCD (2 Nibble Shots)
;
LC2CMD:	
	push	R25
	rcall	LC1CMD		; Send high nibble
	swap	R25
	rcall	LC1CMD		; and low nibble
	pop		R25
	ret


;****************************************************************************************
; Program the LCD characters which are free to be defined by the user
; Z   = Points to Character definition data to be programmed
; R17 = Number of Character definition bytes to program 
; R25 = Character # to program (0...7)
; R26 = Starting row of the character to be programmed. 0 = Top Row, 7 = Cursor Row
;
; Registers changed R25, R26, ZL, ZH 
;	
LCD_Char_Prog:
	push 	R16
	clr		R16		; Byte counter
	lsl		R25		; Calculate address to programm
	lsl		R25		; by multiplying character number
	lsl		R25		; by 8
	add		R25,R26	; and adding the offset inside the character
	ldi		R26,$40	; and the position offset 
	add		R25,R26	; in the LCD chip and then:
	rcall 	LC2CMD	; Send R25 as the start of the character's address to the display

LCD_Char_prog_1:
	ld		R25,Z+	; Get a byte from RAM memory
	rcall 	LC2DAT	; And send it to the display
	inc 	R16		; Byte counter
	cpse 	R16,R17 ; Are we done ?
	rjmp	LCD_Char_prog_1 ; No
	pop		R16
	ret


; *******************************************************************************************
; Subroutine to Copy String to LCD Data Field
;
; String pointed to by Z 
; Variables are inserted according to second 16 Bytes in Data String
; Also handles leading zero suppression

LCDSTR:
	PUSH	R16

;++++ Make Y pointer to LCD Data Field
	rcall	LdiYLcdDat

;++++ Copy Data String

LCDST1:
	LPM		R0,Z+	; Get one Byte to R0
	ST		Y+,R0		; Store Byte
	CPI		YL,LOW(LCDDAT)+LCD_size_char ; Repeat 16 times
	BRNE	LCDST1	
	;
	rcall	LdiYLcdDat

;++++ Insert Variable String

LCDST2:
	lpm		R0,Z+
	MOV		R16,R0		; Use High Register for Immediate OpCodes
	CPI		YL,LOW(LCDDAT)	; Wenn erstes Byte
	BRNE	LCDS21
	TST		R16				; .. und Inhalt = 0
	BRNE	LCDS21
	RJMP	LCDSTA		; dann keine Variablen = Fertig

LCDS21:
	CPI		R16,0		; 0 is next Digit
	BREQ	LCDST5

LCDST3:
	MOV		R16,R0		; If not 0: <$10 is NOP
	ANDI	R16,$F0
	BREQ	LCDST9
	MOV		XL,R0
	LDI		XH,0	

LCDST5:
	LD		R16,X+		; Get Variable
	CPI		R16,10		; Wenn Im Bereich 0..9
	BRCC	LCDS51		; 
	ORI		R16,'0'		; Convert 0..9 to ASCII

LCDS51:
	CPI		R16,'0'		; Handle leading zero
	BRNE	LCDS52
	LD		R17,Y		; 
	CPI		R17,':'
	BRNE	LCDS52
	LDI		R16,' '		; Replace leading zero by Space

LCDS52:
	CPI		R16,$80		; Codes im Bereich $80-$87 (Sonderzeichen)
	BRCS	LCDST6
	CPI		R16,$88
	BRCC	LCDST6
	ANDI	R16,$07				; .. nach 0..7 umsetzen

LCDST6:
	ST		Y,R16				; Insert character

LCDST9:
	INC		YL					; Increment Write Pointer
	CPI		YL,LOW(LCDDAT)+LCD_size_char ; Repeat 16 times
	BRNE	LCDST2	

LCDSTA:	
	POP		R16
	RET


; *****************************************************************************
; Load Y with the address of the LCDDAT array
;
LdiYLcdDat:
	ldi		YH,High(LCDDat)
	ldi		YL,Low(LCDDat)
	ret


; *****************************************************************************
; Mem down in FPMem mode
;
MemDownFpMem:
	rcall	GetCurrentFrq1x2 ; Get required frequency for currently active fox in R16
	lds		R18,FrqX		; Current frequency
	cpi		R18,6
	breq	MemDownFpMode10	; Restore stored value
	mov		Xl,C1
	ldi		Xh,5
	rcall declim
	st		Z,R16

MemDownFpMode10:
	rcall	FreqDownArdf12_3_10
	ret


; *****************************************************************************
; Select frequency entry mode
;
MFreqEntrySelect:
	ldi		R16, MFreqEntry

MFreqEntrySelect10:
	sts		Mode, R16			; Save new mode
	ldi		R16, AGC			; AGC
	sts		GainMode, R16		; ....On in freq entry mode
	rcall	CreateGainSym		; Update gain symbol display
	sts		AcsMon, C0			; Acoustic Smeter Off
	ret


; *****************************************************************************
; Select next Hunting mode
;
MhuntNext:
		rcall	MHuntPrep			; Setup parameters for inclim

MHuntNext5:
		rcall	inclim				; Next mode
		rcall 	MHuntNextPrev		; Process the result
		sbrs	HuntMPropReg,HMPMenuDispBit ; Can we display this mode?
		rjmp	MHuntNext			; No, try the next one
		ret


; *****************************************************************************
; Process the result of the MHuntNext MHuntPrev changes
;
MHuntNextPrev:
		push 	R16					; Save index for a while
		sbrc	RxXReg, 0			; Make sure RxA is active before changing modes
		rcall	ToggleRxABAll		; RxB is active so toggle it back first
		pop		R16
		ldi		Zl,low(HuntingModes*2)	; Load Z with start of table
		ldi		Zh,high(HuntingModes*2)
		add		Zl,R16				; Add Index
		adc		Zh,C0				; Word Wide
		lpm		R16,Z				; Get new Hunting mode
		sts		MHuntX,R16			; Save the new Hunting mode
		adiw	Z,(HuntingModesProp-HuntingModes)*2 ; Displacement to HuntingModesProp table
		lpm		HuntMPropReg,Z		; Get the properties for this Hunting mode
		sts		AutoRxABSw, C0		; Clear Auto RxAB Switching when changing modes
		ldi		Zh,high(RxAParam)	; Start of stored Data
		ldi		Zl,low(RxAParam)
		rcall	StoreRxAB			; Store RxA
		rcall	StoreRxAB			; And make RxB equal to RxA
		ret		


; *****************************************************************************
; Prepare for hunting mode changes
;
MHuntPrep:
		lds		R17, MHuntX		; Get current Hunting mode
		ldi		Zh,High(HuntingModes*2)	; Load Z with start of table
		ldi		Zl,low(HuntingModes*2)
		ldi		R16,-1			; Index
MHuntPrep5:
		inc		R16
		lpm		R18,Z+			; Hunting mode from table
		cp		R18,R17			; Equal to current hunting mode?
		brne	MHuntprep5		; No, keep on looking
		ldi		Xh,(HuntingmodesEnd - huntingmodes)*2-2 ; Warning last byte is filler byte!!! otherwise substract 1
		mov		Xl,C0
		ret						; Exit with R16 as index in various hunting mode tables


; *****************************************************************************
; Select previous Hunting mode
;
MHuntPrev:
		rcall 	MHuntPrep			; Setup parameters for inclim

MHuntPrev5:
		rcall	declim				; Previous mode
		rcall	MHuntNextPrev		; Process the result
		sbrs	HuntMPropReg,HMPMenuDispBit ; Can we display this mode?
		rjmp	MHuntPrev			; No try the previous one
		ret


; *****************************************************************************
; Select next mode.
; Xl has lower limit
; Xh has upper limit
;
ModesNext:
	lds		R16, Mode
	rcall	inclim
	sts		Mode, R16
	ret

; *****************************************************************************
; Select previous mode.
; Xl has lower limit
; Xh has upper limit
;
ModesPrev:
	lds		R16, Mode
	rcall	declim
	sts		Mode, R16
	ret


;++++  Subroutine for 16x16 Bit Unsigned Multiplication
;
; This subroutine multiplies the two 16-bit register variables 
; Mp16uH:mp16uL and mc16uH:mc16uL.
; The result is placed in m16u3:m16u2:m16u1:m16u0.
;  
; Number of words	:14 + return
; Number of cycles	:153 + return
; Low registers used	:None
; High registers used   :7 R16..R22
;	
;++ Subroutine Register Variables

.def	mc16uL	=r16		;multiplicand low byte
.def	mc16uH	=r17		;multiplicand high byte
.def	mp16uL	=r18		;multiplier low byte
.def	mp16uH	=r19		;multiplier high byte
.def	m16u0	=r18		;result byte 0 (LSB)
.def	m16u1	=r19		;result byte 1
.def	m16u2	=r20		;result byte 2
.def	m16u3	=r21		;result byte 3 (MSB)
.def	mcnt16u	=r22		;loop counter

;++ Code

MPY16u:
	clr		m16u3		;clear 2 highest bytes of result
	clr		m16u2
	ldi		mcnt16u,16	;init loop counter
	lsr		mp16uH
	ror		mp16uL
	
m16u_1:
	brcc	noad8		;if bit 0 of multiplier set
	add		m16u2,mc16uL	;add multiplicand Low to byte 2 of res
	adc		m16u3,mc16uH	;add multiplicand high to byte 3 of res

noad8:
	ror		m16u3		;shift right result byte 3
	ror		m16u2		;rotate right result byte 2
	ror		m16u1		;rotate result byte 1 and multiplier High
	ror		m16u0		;rotate result byte 0 and multiplier Low
	dec		mcnt16u		;decrement loop counter
	brne	m16u_1		;if not done, loop more
	ret


; *****************************************************************************
; Enter Aux modes
;
MSetAuxSelect:
	ldi		R16, MAuxStart	; First Setting menu
	sts		Mode, R16		; is the next mode of operation
	ret


; *****************************************************************************
; Enter Attenuator Calibration Mode
;
MSetCalAtt:
	ldi		R16, MCalAttnActive	; Attenuator Calibration Active
	sts		Mode, R16			; is the next mode of operation
	sts		ABSBIN,C0 			; Select 0db Attenuation
	ldi		R16,MGC				; Turn on MGC for RF
	sts		Gainmode,R16
	mov		R16,C0
	rcall 	CalAttUpdate5
	ret

; *****************************************************************************
; Enter setting modes
;
MSetStartSelect:
	ldi		R16, MSetStart	; First Setting menu
	sts		Mode, R16		; is the next mode of operation
	ret

; *****************************************************************************
; Zu neuem Kanal gehrende Tonfolge ausgeben

NEXTF_TONCHA:
	lds 	R16,FRQX	; Zu neuem Kanal gehrende Tonfolge ausgeben

NextF_TonchB:			; Sound in Morse code the value in R16 (1..6 only)
	push	r17
	cpi		R16,1
	brne	NEXTF_TONCH1
	sound	SOUF1		; Neue Frequenz 1 Tonfolge

NEXTF_TONCH1:
	cpi		R16,2
	brne	NEXTF_TONCH2
	sound	SOUF2		; Neue Frequenz 2 Tonfolge

NEXTF_TONCH2:
	cpi		R16,3
	brne	NEXTF_TONCH3
	sound	SOUF3		; Neue Frequenz 3 Tonfolge

NEXTF_TONCH3:
	cpi		R16,4
	brne	NEXTF_TONCH4
	sound	SOUF4		; Neue Frequenz 4 Tonfolge

NEXTF_TONCH4:
	cpi		R16,5
	brne	NEXTF_TONCH5
	sound	SOUF5		; Neue Frequenz 5 Tonfolge

NEXTF_TONCH5:
	cpi		R16,6
	brne	NEXTF_TONCH6
	sound	SOUF6		; Neue Frequenz 6 Tonfolge

NEXTF_TONCH6:
	pop		r17

NEXTF_Exit:
	ret


; ********************************************************************************************
; Frequenz-Index erhhen:
;
NextFreqIndex:
	lds		R16,FRQX	; Frequenz-Index
	lds		R17,NFREQ
	cp		R16,R17
	brne	NextFreqIndex1
	ldi		R16,1		; ...bei > Maximum nach 1 wrappen
	rjmp	NextFreqIndex2

NextFreqIndex1:
	inc		R16		; ...erhhen

NextFreqIndex2:
	sts		FRQX,R16
	ret


; ********************************************************************************************
; Nchsten gespeicherten Kanal einstellen:
;
NextMem:
	rcall	NextFreqIndex		; Frequenz-Index um 1 erhhen
	rcall	NextPrevMem
	ret

NextPrevMem:
	rcall	GETFRQ		; Get new Frequency FRQ 1..6 -> FRQBIN
	rcall	CONVF		; Dezimale Frequenzanzeige bereitstellen
	rcall	vfoismem
	rcall	GETPF		; aktuelle Fuchsleistung aus Array holen
	rcall	CONVPF		; Convert Code to Textform
	sts		FRQCHG,C1	; Frequency Changed Flag setzen (=PLL laden)
	flash	STRFRQ,20	; Neue Frequenz String flashen
	rcall	NextF_Toncha
	ret


; *****************************************************************************
; Decrement the number of Foxes
;
NFoxesDown:
        ldi		XL,low(NFox)	; Pointer auf BCD-String
        ldi		XH,high(NFox)
        rcall	BCD2B			; Wandelt 2B BCD @ X,X+1 nach Bin in R16
		cp		R16,C1			; At lower limit ?
		breq	NFoxesUpDown	; Then get out
        ldi		XL,1			; Lower Limit
        ldi		XH,10			; Upper Limit
        rcall	DECLIM			; Decrement R16 within limits in XH,XL
        ldi		XL,low(Nfox)	; Pointer auf BCD-String
 		ldi		XH,high(NFox)
        rcall	B2BCD			; Wandelt Bin in R16 in 2B BCD @ X, X+1
		rjmp	NFoxesUpDown	; Do the last part

; *****************************************************************************
; Increment the number of Foxes
;
NFoxesUp:
        ldi		XL,low(NFox)	; Pointer auf BCD-String
        ldi		XH,high(NFox)
        rcall	BCD2B			; Wandelt 2B BCD @ X,X+1 nach Bin in R16
		cpi		R16,10			; At higher limit ?
		breq	NFoxesUpDown	; Then get out
        mov		XL,C1			; Lower Limit
        ldi		XH,10			; Upper Limit
        rcall	INCLIM			; Increment R16 within limits in XH,XL
        ldi		XL,low(NFox)	; Pointer auf BCD-String
        ldi		XH,high(NFox)
        rcall	B2BCD			; Wandelt Bin in R16 in 2B BCD @ X, X+1
		rjmp	NFoxesUpDown	; Do the last part

; *****************************************************************************
; NFOxes Up/Down common code part
;
NFoxesUpDown:
	rcall	ClrTim		; Restart the Fox timer
	ret


;******************************************************************************
; Increment the number of frequencies (QRG's) Range is 1-6
;
NQrgUp:
	lds		R16,NFreq
	cpi		R16,6			; Maximum erreicht?
	breq	NQrgUpDown5	; ja, keine Erhhung mehr mglich
	inc		R16
	rjmp	NQrgUpDown

; *****************************************************************************
; Decrement the number of frequencies (QRG's) Range is 1-6
;
NQrgDown:
	lds		R16,NFreq
	cpi		r16,1			; Mimumum erreicht?
	breq	NQrgUpDown5	; ja, keine Verminderung mehr mglich...
	dec		R16


; *****************************************************************************
; Increment/Decrement the number of frequencies (QRG's) Range is 1-6
; Common code
;
NQrgUpDown:
	sts		NFREQ,R16			; Store new number
	sts		FRQX,C1 	; Nach nderung: Frequenz 1 aktivieren
	rcall 	vfoismem
	rcall	GETFRQ			; Neue Frequenz holen
	rcall	CONVF			; ... und in Anzeigewert umrechnen
	rcall	GETPF			; aktuelle Fuchsleistung aus Array holen
	rcall	CONVPF			; ... und in Anzeigewert umrechnen
	sts		FRQCHG,C1 	; Frequency Changed Flag setzen (=PLL laden)

NQrgUpDown5:
	ret


; *******************************************************************************************
; I2C-Treiber
;
; HW-Interface: SDA auf Pin PC5, SCL auf Pin PC4
; Leitungen werden fuer Low aktiv runtergezogen, fuer High 
; freigegeben und gehen ueber externen Pull-Up hoch
; Return-Code fuer alle Ops in R0: 0 = OK, 1 = Missing Ack
; I2C Bytes has # of bytes to transmit, incl chip address
;
OutI2C:
		lds		R17,I2CBytes ; # of Bytes to transmit
		ldi		Zl,low(I2CData)
		ldi		Zh,high(I2CData)
		rcall	I2Strt	; Send start sequence

OutI2C5:
		ld		R16,Z+	; Get data from array
		rcall	I2Send	; And send it
		dec		R17		; Loop counter
		brne	OutI2C5
		rcall	I2Stop	; Stop sequence
		Ret

		




; *******************************************************************************************
; Copy 16 Bytes from RAM (LCDDAT) to Display if what is in LCDDAT is different from what is
; on the display already. What is on the display is in LCDDAT_Current.
;
OUTLCD:
	push	R16				;Get some breathing space
	push	R17
	push	R25
	push	YL
	push	YH
	push	ZL
	push	ZH
	;
	rcall	LdiYLcdDat
;	ldi 	YL,Low(LCDDat)		; Get the address pointers to the new to be displayed array
;	ldi		Yh,High(LCDDat)
	ldi 	Zl,Low(LCDDat_Current)	; Pointer tot the currently displayed array
	ldi		Zh,High(LCDDat_Current)
	ldi 	R16,LCD_size_char	; Loop counter for all the characters on the display

OUTLCD1:
	ld		R17, Y+				; Get a Byte from the LCDDAT array
	ld		R25, Z+				; And a Byte from the currently displayed array
	cp		R17,R25				; Are they equal?
	brne	OUTLCD2		 		; Go and display the new LCDDAT array if not equal to LCDDAT_Current
	dec 	R16
	brne	OUTLCD1				; Continue comparing untill all characters are done
	rjmp 	OUTLCD5				; No need to display new data, get out of here!

; LCDDAT and LCDDAT_Current arrays are different so go and display the LCDDAT array
; While doing that, update LCDDAT_Current array

OUTLCD2:
	rcall 	UpdateLCD 

OUTLCD5:						; We are done. breath out
	pop		ZH
	pop		ZL
	pop		YH
	pop 	YL
	pop 	R25
	pop 	R17
	pop 	R16
	ret


; *****************************************************************************
; Set PortB pin 5 High so LCD background LED is bright
;
;PortLEDBright:
;	sbi		DDRB,5		; Pin 5 of Port B is output
;	sbi		PortB,5		; And make it high
;	ret


; *****************************************************************************
; Set PortB pin 5 as input with high impedance so LCD Background LED is Dim
;
;PortLEDDim:
;	cbi		DDRB,5		; Pin 5 of Port B is input
;	cbi		PortB,5		; And make it tristate
;	ret


; *****************************************************************************
; Setup for QRG power change
;
PQrgSetUp:
	rcall	GetPf	; Get current Power out of Array
	mov		Xl,C0	; Lower limit
	ldi		Xh,PFoxUnknown	; Upper Limit
	lds		R16,PFoxbn	; Get current power
	ret


; *****************************************************************************
; Increase Power of current QRG
;
PQrgUp:
	rcall	PQrgSetUp	; Prep registers
	rcall 	IncLim		; Increment power index
	rjmp	PQrgUpdown	; Go to common up/down part of routine


; *****************************************************************************
; Decrease Power of current QRG
;
PQrgDown:
	rcall	PQrgSetUp	; Prep registers
	rcall 	DecLim		; Decrement power index


; *****************************************************************************
; Increase/Decrease Power of Current QRG, Common Code.
;
PQrgUpDown:
	sts		PFoxBn,R16 	; Save new index
	rcall	ConvPF		; Convert index to text
	rcall	PutPf		; Write new value to Power array
	ret


; ********************************************************************************************
; Frequenz-Index vermindern:
;
PrevFreqIndex:	
	lds		R16,FRQX	; Frequenz-Index
	cp		R16,C1
	brne	PrevFreqIndex1
	lds		R16,NFREQ	; ...bei 1 nach Maximum wrappen
	rjmp	PrevFreqIndex2

PrevFreqIndex1:
	dec		R16

PrevFreqindex2:		
	sts		FRQX,R16
	ret


; *******************************************************************************************
; Vorherigen gespeicherten Kanal einstellen:
;
PrevMem:
	rcall	PrevFreqIndex		; Frequenz-Index um 1 vermindern
	rcall	NextPrevMem
	ret


; *******************************************************************************************
; Fuchsleistung in Array speichern
;
PUTPF:	
	lds		r16,FRQX	; Index auf aktive Frequenz laden (1-6)
	dec		R16			; 1..5
	ldi		YL,LOW(PFOX1)	; Anfang des Leistungs-Arrays
	ldi		YH,High(PFox1)
	add		YL,r16		; Wort-Pointer erzeugen
	adc		Yh,C0
	lds		r16,PFOXBN	; aktuelle Leistung laden
	st		Y+,r16		; ...und in Leistungs-Array zurck speichern
	ret



; *****************************************************************************
; Reload the RxX time variable with either RxATimeBin or RxBTimeBin
;
ReloadRxX:
; Reset the RxXTimeRegister 
	lds		R16, RxATimeBin	; A is default
	sbrc	RxXReg,0
	lds		R16,RxBTimeBin
	mov		RxXTimeReg,R16
	ret


; *****************************************************************************
; Restore receiver settings to the RxA or RxB array
; Upon entry Z must have first address of RxA or RxB array
;
RestoreRxAB:
	; Restore the parameters
	ld		R16,Z+		; FrqX
	sts		FrqX,R16
	ld		R16,Z+		; AbsBin
	sts		AbsBin,R16
	ld		R16,Z+		; FrqBin
	sts		FrqBin,R16
	ld		R16,Z+		; FrqBin+1
	sts		FrqBin+1,R16
	ld		R16,Z+		; AcSMon
	sts		AcsMon,R16
	ld		R16,Z+
	sts		GainMode,R16
	ld		R16,Z+
	sts		MHuntx,R16
	ld		HuntMPropReg,Z
	ret


; *****************************************************************************
; Return, A Subroutine which does nothing just return. 
; Used for state machine subroutine calls not needing actions
;
Return:
	Ret


; *****************************************************************************
; Save changes in config etc. to eeprom
;
SAVCHG:	
	LDI		YL,LOW(EEPBEG)	; Pointer auf Variablen
	CLR		YH
	LDI		R20,0		; Bytecounter und EEPROM Low Adresse
	OUT		EEARH,YH	; High Adresse ist 0

SAVCH1:	
	LD		R16,Y+		; 1 Variable holen

; 	Derzeitigen Wert aus EEPROM holen und vergleichen

	OUT		EEARL,R20	; EEPROM Entry Adresse laden
	LDI		R17,$01
	OUT		EECR,R17	; Set EEPROM Read Cmd

SAVCH2:
	IN		R17,EECR	; Wait for EEPROM Read complete
	ANDI	R17,$01
	BRNE	SAVCH2
	IN		R17,EEDR	; Daten lesen
	CP		R16,R17		; .. und vergleichen
	BREQ	SAVCH4

;	Wenn ungleich: ins EEPROM schreiben


SAVCH3:	
	IN		R17,EECR	; Wait for EEPROM Ready to write	
	ANDI	R17,$02
	BRNE	SAVCH3
	OUT		EEARL,R20
	OUT		EEDR,R16
	CLI					; Interrupt ausschalten
	LDI		R17,$04
	LDI		R18,$02
	OUT		EECR,R17	; Set EEPROM Master Write Enable
	OUT		EECR,R18	; Set EEPROM Write	
	SEI			; Interrupt wieder an

SAVCH4:	
	INC		R20
	CPI		R20,EEPEnd-EEPBEG ; Gesamter Bereich geschrieben?
	BRNE	SAVCH1

SAVCH9:
	ret	


; *****************************************************************************
; Set Memory # 6
;
SetMem6:
	ldi		R16,6
	rjmp	SetMem

; *****************************************************************************
; Set Memory #3 (Used in MHuntArdf12_3 and MHuntArdf2_5 modes)
;
SetMem3:
	ldi		R16,3			; Activate frequency 3

SetMem:
	lds		R17,Frqx		; Get current memory #
	cpse	R17,R16			; Was it allready 3/6?
	sts		Frq512,R17		; No Save it
	sts		FrqX,R16		; Frequency 
	rcall	NextPrevMem		; Update other parameters
	ret


; *****************************************************************************
; Select next settings mode
;
SettingModesNext:
	ldi		Xl, MSetStart
	ldi		Xh, MSetStop
	rjmp 	ModesNext


; *****************************************************************************
; Select previous settings mode
;
SettingModesPrev:
	ldi		Xl, MSetStart
	ldi		Xh, MSetStop
	rjmp 	ModesPrev


; ********************************************************************************************
; Soundgenerator (Alle 100 ms aufrufen)
;
; Wenn Soundpointer aktiv: Nchstes Element aus Sound-String holen
; und mit Frequenz 10 kHz/Wert piepsen
;
SOUGEN:	
	lds		ZH,SOUPNT+1
	lds		ZL,SOUPNT	; Sound-Pointer holen;
	tst		ZH			; Wenn MSB = 0
	breq	SOUG9		; .. dann nichts ausgeben
	lpm					; Get one Byte to R0
	adiw	ZL,1		; Increment Z

.If Processor == 8
	out		OCR1AH,R0	; Variablen Oscillator laden 
 	clr		R18
	out		OCR1AL,R18	; Do not use C0, R18 is used later as well
.Endif

.If Processor == 168;
	sts		OCR1AH,R0
	clr		R18
	sts		OCR1AL,R18
.Endif

	dec		R0			; Sonderfall Wert = 1
	breq	SOUG1		; .. Piepser bleibt aus 
	ldi		R18,$40		; .. sonst Piepser aktivieren

SOUG1:
.If Processor == 8	
	out		TCCR1A,R18
.Endif

.If Processor == 168;
	sts		TCCR1A,R18
.Endif
;
	sbi		DDRB,2		; NF Degate aktivieren	
	lpm					; Vorschau auf nchstes Byte
	tst		R0			; Wenn Wert = $0
	brne	SOUG2
	clr		ZH			; .. Fertig, SOUPNT MSB auf 0 
SOUG2:	
	sts		SOUPNT,ZL	; Sound-Pointer retten		
	sts		SOUPNT+1,ZH
SOUG9:
	ret


; *****************************************************************************
; Start or Stop the stopwatch
;
StopwStartStop:
	lds		R18,STOPWR	; Get current status
	tst		R18
	brne	StopwStartStop_5; Go and stop it

; Stopwatch Stop, Clear and start		

	sts		STOPWR,C0	; Run-Flag lschen
	sts		STOPW,C0		; Zeit auf 0 setzen
	sts		STOPW+1,C0
	sts		STOPW+2,C0
	sts		STOPW+3,C0
	sts		STOPW+4,C0
	sts		STOPWR,C1		; und Run-Flag setzen
	rjmp	StopwStartStop_10

StopwStartStop_5: 		; 
	sts		STOPWR,C0	; .... sonst Stopuhr anhalten

StopwStartStop_10:
	ret


; *******************************************************************************************
; Aktuelle Frequenz in FRQBIN -> FRQ1..6 speichern
;
StoreMem:
	lds		R16,FRQX	; Index auf aktive Frequenz laden 
	dec		R16			; 0..5
	lsl		R16			; Times 2
	ldi		YL,LOW(FRQ1); Anfang des Frequenz-Arrays
	ldi		YH,high(Frq1)
	add		YL,R16		; Wort-Pointer
	adc		Yh,C0		; ...erzeugen
	lds		R16,FRQBIN	; aktuelle Frequenz (H) laden
	st		Y+,R16		; ...und in Frequenz-Array speichern
	lds		R16,FRQBIN+1	; Frequenz (L)
	st		Y,R16		; ...im Array speichern
	rcall 	vfoismem	; Update display witth mem symbol
	sound	soup
	ret


; *****************************************************************************
; Store receiver settings to the RxA or RxB array
; Upon entry Z must have first address of RxA or RxB array
;
StoreRxAB:								; Store parameters at RxA or RxB
	lds		R16,FrqX
	st		Z+,R16
	lds		R16,AbsBin
	st		Z+,R16
	lds		R16,FrqBin
	st		Z+,R16
	lds		R16,FrqBin+1
	st		Z+,R16
	lds		R16,ACSMon
	st		Z+,R16
	lds		R16,GainMode
	st		Z+,R16
	lds		R16,MHuntx
	st		Z+,R16
	mov		R16,HuntMPropreg
	st		Z+,R16
	ret


; *****************************************************************************
; Increment Time of Fox (20 ms steps)
;
TFoxMsUp:
		push	R16
        push	XL
	    push	XH
        ldi		XL,low(TFoxMs)	; Pointer auf BCD-String
        ldi		Xh,high(TfoxMs)
        rcall	BCD2B		; Wandelt 2B BCD @ X,X+1 nach Bin in R16
        mov		XL,	C0			; Lower Limit
        ldi		XH,	99			; Upper Limit
        rcall	INCLIM		; Increment R16 within limits in XH,XL
		rcall	IncLim		; twice so we do 20 ms steps
		ldi		XL,low(TFoxMs)	; Pointer auf BCD-String
    	ldi		XH,high(TFoxMs)
    	rcall	B2BCD		; Wandelt Bin in R16 in 2B BCD @ X, X+1
        rjmp	TFoxMsUpDownPost

; *****************************************************************************
; Decrement Time of Fox (20 ms steps)
;
TFoxMsDown:
		push	R16
        push	XL
        push	XH
        ldi		XL,low(TFoxMs)	; Pointer auf BCD-String
        ldi		XH,high(TfoxMs)
        rcall	BCD2B		; Wandelt 2B BCD @ X,X+1 nach Bin in R16
        mov		XL,	C0			; Lower Limit
        ldi		XH,	99			; Upper Limit	Rcall	TFoxMsUpDownPre
 	    rcall	DECLIM		; Decrement R16 within limits in XH,XL
		rcall	DecLim		; Twice so we do 20 ms
    	ldi		XL,low(TFoxMs)	; Pointer auf BCD-String
    	ldi		Xh,high(TFoxMs)
	    rcall	B2BCD		; Wandelt Bin in R16 in 2B BCD @ X, X+1


; *****************************************************************************
; Increment/Decrement Transmission time of Fox (Milli Seconds * 10) 1..99 range
; Common code
;
TFoxMsUpDownPost:
	
	rcall	ClrTim
	pop		XH
	pop		XL
	pop		R16
	ret

; *****************************************************************************
; Increment Transmission time of Fox (Seconds) 1..99 range
;
TFoxSUp:

        ldi	XL,low(TFoxS)		; Pointer auf BCD-String
        ldi	XH,high(TFoxS)
        rcall	BCD2B		; Wandelt 2B BCD @ X,X+1 nach Bin in R16
        mov	XL,C1		; Lower Limit
        ldi	XH,99		; Upper Limit
        rcall	INCLIM		; Increment R16 within limits in XH,XL
        ldi	XL,low(TFoxS)		; Pointer auf BCD-String
        ldi	XH,high(TFoxS)
        rcall	B2BCD		; Wandelt Bin in R16 in 2B BCD @ X, X+1
		rjmp	TFoxSUpDown


; *****************************************************************************
; Decrement Transmission time of Fox (Seconds) 1..99 range
;
TFoxSDown:
	d2bcd	TFOXS,1,99

; *****************************************************************************
; Increment/Decrement Transmission time of Fox (Seconds) 1..99 range
; Common code
;
TFoxSUpDown:
	sts		Fox,C0
	sts		Fox+1,C1
	rcall	CLRTIM		; Nach nderung: Fuchstimer neu starten
	ret


; ********************************************************************************************
; 320 usec Timer Interrupt Handler
; ********************************************************************************************
;
TIMINT:
	PUSH	R16
	PUSH	R22
	PUSH	R23		; Get some work registers
	PUSH	R24
	PUSH	XL
	PUSH	XH
	IN      R24,SREG        ; Save Statusreg
	PUSH    R24

; Ab hier alle 320us:

TIM500:	
	ldi		R22,-25		; Set Counter to Startvalue (Divide by 64*25) 
	out		TCNT0,R22
	sei					; Enable next interrupt
	inctim	TIK3U1,3,TIM501	; Increment to 1 ms 
	rcall	DEBNCE		; Debounce Drehgeber alle 1ms

TIM501:	
	inctim	TIK3U2,25,TIM599 ; Increment to 8 msec
	rcall	DebEncPush	; Debounce Encoder Pushbutton every 8 ms
	rcall	DEBNC2		; Debounce Taster every 8 ms

; Ab hier alle 8ms:

TIM8MS:
	inctim	TIK8M,5,TIM502	; Increment to 40 msec	
	rjmp	TIM504		; weiter nach 0ms, 40ms, 80ms ...		

TIM502:
	cpi		R24,2
	breq	TIM504		; weiter nach 16ms

TIM599:	
	rjmp	TIM99		; --> Ende

; Ab hier alle 20ms im Abstand: 16ms-24ms-16ms...

TIM504:
	d2bcd	TIMER+2,0,99	; Decrement Timer ms	
	in		R22,SREG	; Save the Carry Flag
	d2bcd	TIMER+2,0,99
	sbrs	R22,0		; Skip if Carry set (after first Decrement)
	rjmp	TIM101

TIM10:
	d2bcd	TIMER,0,99	; Decrement Timer s

TIM101:
	lds		R24,TIMER+3	; Wenn Timer 00:00 erreicht hat
	tst		R24
	brne	TIM102		
	lds		R24,TIMER+2
	tst		R24
	brne	TIM102
	lds		R24,TIMER+1
	tst		R24
	brne	TIM103
	lds		R24,TIMER
	tst		R24
	brne	TIM103
	rjmp	TIM104

TIM102:
	rjmp	TIM20

TIM103:	
	rjmp	TIM18

TIM104:
	lds		R24,TFOXS	; .. Timer auf Startwert setzen
	sts		TIMER,R24
	lds		R24,TFOXS+1
	sts		TIMER+1,R24
	lds		R24,TFOXS+2
	sts		TIMER+2,R24
	lds		R24,TFOXS+3
	sts		TIMER+3,R24
	sts		FrqTim,C1
	lds		R24,FOX		; Wenn derzeitiger Fuchs <> Maxwert
	lds		R23,NFOX
	cp		R23,R24
	brne	TIM11
	lds		R24,FOX+1
	lds		R23,NFOX+1
	cp		R23,R24
	breq	TIM12

TIM11:	
	rcall IncrFox
	rjmp	TIM20

TIM12:
	sts		Fox, C0 ; Sonst Fuchs auf 1 setzen
	sts		FOX+1,C1

TIM18:
	lds		R22,TALARM	; Get Alarm-Time
	lds		R23,TALARM+1
	add		R22,R23		; Wenn Zeit = 0
	breq	TIM20		; .. nix tun
	lds		R22,TIMER+1	; Sonst Timer mit Alarm vergleichen
	cp		R22,R23
	brne	TIM20
	lds		R23,TALARM
	lds		R22,TIMER
	cp		R22,R23
	brne	TIM20
	sts		ALATIM, C1 ; Bei Match: Flag fr Main setzen

TIM20:
	inctim	TIK100,5,TIM21	; Increment 20 msec Timer up to 5 (100 ms)
	sts		TICK, C1 ; *** @ 100ms set Timing Flag for Main

TIM21:
	inctim	TIK20M,50,TIM99	; Increment 20 msec Timer up to 50 (1 s)

; Ab hier alle 1s:

	sts		Tack,C1	; 1s timing flag for main
	lds		R24,STOPWR	; Falls Stopwatch an
	tst		R24
	breq	TIM30
	rcall	IncrStopw	; Increment the stopwatch

TIM30:	
	inctim	TIK1S,10,TIM99	; Increment 1 sec timer to 10

;	++ 10 s Service

	sts		tock,c1			; Set 10 s marker
	inctim	TIK10S,6,TIM99	; Increment 10 sec Timer to 6

; 	++ 1 min Service
    
	lds		R16,BatTest	; Get current battery test timer
	tst		R16
	brne	Tim32		;
	sts		BATTIM,C1 ; Check Battery Time Flag setzen	
	ldi		R16,BatTestInterval ;Reload the count down counter

Tim32:
	dec		R16
	sts		Battest,R16

;	++ Conclusion of Interrupt Service

TIM99:  
	CLI			; No interrupts during conclusion
	POP     R24             ; Restore Status Reg
    OUT     SREG,R24
	POP		XH
	POP		XL
	POP		R24
	POP		R23
	POP		R22
	POP		R16
    RETI                    ; Interrupt-Handling beendet


; *****************************************************************************
; Toggle the Aucoustic S-meter on and off
;
ToggleAcSm:
	lds		R16,ACSMON		; .. Akustisches S-Meter Ein/Aus toggeln
	ldi		R18,$40
	eor		R16,R18
	sts		ACSMON,R16
	tst		R16				; Falls neu eingeschaltet ..
	breq	ToggleAcSm5	
	ldi		R16,30			; .. dann Nachlaufzeit auf 3 Sekunden
	sts		ACSTIM,R16

ToggleAcSm5:
	ret


; *****************************************************************************
; Toggle auto RX A/B switching on/off
;
ToggleAutoRxABSw:
		lds		R16,AutoRxABSw
		eor		R16, C1		; Toggle bit 0
		sts		AutoRxABSw, R16
		ret


; *****************************************************************************
; Toggle the FP hunting mode
;
ToggleMHuntFPMEM:
ToggleMHuntFP:
		rcall	MHuntPrep	; Get index to R16 (R17 and R18 clobbered)
		ldi		Zh,high(HuntingModesToggle*2)
		ldi		Zl,low(HuntingModesToggle*2)
		add		Zl,R16		; Add index
		adc		Zh,C0		; Word wide
		lpm		R16,Z		; Get new hunting mode
		sts		MhuntX,R16
		ret


; *****************************************************************************
; Toggle PB5 to change the current through the LCD Background LCD
;
;ToggleLEDBright:
;		rcall 	PortLedDim		; Default to DIM LED
;		lds		R16,LEDBright	; Get current status
;		eor		R16,C1			; Invert it
;		sts		LEDBright,R16	; And put it back
;		breq	ToggleLEDBright5 ; When new status is 0 get out, DIM was default
;		rcall	PortLEDBright	; Turn LED on bright

;ToggleLEDBright5:
;		ret

; *****************************************************************************
; Toggle AGC tones during freq entry on/off
;
ToggleAGCAudio:
		lds 	R16,AGCAudio ; Get current status
		eor		R16,C1			; Invert it
		sts		AGCAudio,R16
		ret

; *****************************************************************************
; Toggle RxA<>RxB
;
ToggleRxAB:
	ldi		Zh,high(RxAParam)	; Start of parameters to save to Z
	ldi		Zl,low(RxAParam)
	sbrc	RxXReg,0			; Skip if RxB is active
	adiw	Z,RxBParam-RxAParam 	; Rx2 is active
	rcall	StoreRxAB			; Store the selected reiver settings

; Restore the complementary Rx parameters.

	ldi		Zh,high(RxAParam)	; Start of saved parameters to Z
	ldi		Zl,low(RxAParam)
	sbrs	RxXreg,0			; Skip if Rx2 WAS active
	adiw	Z,RxBParam-RxAParam	; Add offset for Rx2
	eor		RxXreg,C1			; Toggle Rx1/2
	rcall	RestoreRxAB	

;Update some more parameters
ToggleRxABEnd:
	sts 	FRQCHG,C1		; Force update of PLL
	rcall	VfoIsMem		; Update the Mem symbol
	rcall	CreateGainSym 	; Update the Gain symbol
	rcall	CreateRxXSym 	; UPdate the RX symbol (A/B)
	rcall	convf			; Update the frequency display
	ret


; ****************************************************************************
; Execute all the little routines to toggle RxA <> RxB
;
ToggleRxABAll:
	rcall	ToggleRxAB
	rcall	ToggleRXABNoise ; Sound Morse Code A or B 
	rcall	ReloadRxX		; Reload the RxX variable
	ret


; *****************************************************************************
; Make the appropriate noise when Rx A or B is activated.
;
ToggleRXABNoise:
	ldi		Zl,low(SouA*2)
	sbrc	RxXreg,0		; A is default
	ldi		Zl,low(SouB*2)	
	sts		SOUPNT,ZL
	ldi		Zh,high(SouA*2)
	sbrc	RxXReg,0
	ldi		Zh,high(SouB*2)
    sts		SOUPNT+1,ZH
	mov		SwitchFlgReg,C0 ; Done , clear the flag
	ret


; *****************************************************************************
; TRxA on time down
;
TrxAdown:
	Mov		Xl,C1
	ldi		Xh,120
	lds		R16,RxATimeBin
	rcall	declim
	sts		RxATimeBin,R16
	ldi		Xl,Low(RxATime)
	ldi		XH,high(RxATime)
	rcall	B3BCD
	ret


; *****************************************************************************
; TRxA on time up
;
TrxAUp:
	Mov		Xl,C1
	ldi		Xh,120
	lds		R16,RxATimeBin
	rcall	inclim
	sts		RxATimeBin,R16

TrXaUp5:
	ldi		Xl,Low(RxATime)
	ldi		XH,high(RxATime)
	rcall	B3BCD
	ret


; *****************************************************************************
; TRxB on time down
;
TrxBdown:
	Mov		Xl,C1
	ldi		Xh,120
	lds		R16,RxBTimeBin
	rcall	declim
	sts		RxBTimeBin,R16
	ldi		Xl,Low(RxBTime)
	ldi		XH,high(RxBTime)
	rcall	B3BCD
	ret


; *****************************************************************************
; TRxB on time up
;
TrXBUp:
	Mov		Xl,C1
	ldi		Xh,120
	lds		R16,RxBTimeBin
	rcall	inclim
	sts		RxBTimeBin,R16

TRxBUp5:
	ldi		Xl,Low(RxBTime)
	ldi		XH,high(RxBTime)
	rcall	B3BCD
	ret


; *****************************************************************************
; Adjust Battery readout calibration value.
;
UBFineUp:
	lds		R16, UBFine		; Current value
	inc 	R16
	rjmp	UBFineUpDown

UBFineDown:
	lds 	R16,UBFine		; Current value
	dec 	R16

UBFineUpDown:
	sts		UBFine,R16		; Store new value

UBFineUpDown5:				; Gets called from init part of code
	ldi		XL,low(UBFVAL)	; and convert it to BCD 
	ldi		XH,high(UBFVAL)	
	rcall	B3BCD
	ret	


;=============================================================================================
; Update the 2 lines on the LCD  
;
UpdateLCD: 
	rcall	LdiYLcdDat
;	ldi 	YL,LOW(LCDDat)	; Get the address pointers to the new to be displayed array
;	ldi		YH,High(LCDDat)
	ldi 	Zl,LOW(LCDDat_Current)	; Pointer to the currently displayed array
	ldi		Zh,High(LCDDat_Current)
	ldi		R25,$80			;DDRAM-Adress auf 0
	rcall 	LC2CMD
	rcall 	UpdateLCDLine
	ldi		R25,$A8		; DDRAM-Adress to 40 = Start of 2nd line
	rcall 	LC2CMD
	rcall 	UpdateLCDLine
	ret


;==============================================================================
; Udate one line on the LCD
;
UpdateLCDLine:
	ldi		R16,LCD_size_char/2		; counter

UpdateLCdLine5:
	ld		R25,Y+		; Get one Data Byte
	rcall	LC2DAT		; Write it to the Display
	st		Z+,R25 		; And to the array keeping track of what is going on
	dec		R16			; Decrement Loopcounter
	brne	UPdateLCDLine5
	ret


;==============================================================================
; VfoIsMem
; Determine if VFO frequency in FRQBIN, FRQBIN+1 is equal to one of the stored 
; Memory frequencies in FRQ1....FRQ6. Will display the mem channel # in Roman
; Numerals if frqBin equals one of the frequencies stored in frq1..6.
; No exact match is needed. Roman numerals are displayed as long as the delta
; is smaller then the DeltaValue constant. For larger differences a sliding
; arrow(s) is/are displayed to indicate the absolute size and the direction of
; the delta
;
VfoIsMem:
		ldi		ZL, low(FRQ1)	; Z is the pointer to the stored frequencies
		ldi		ZH, high(FRQ1)
		ldi		Yl, low(FrqBinDelta) ; Y is the pointer to the FrqBinDelta Array
		ldi		Yh, high(FrqBinDelta)
		CLR		R16				; R16 is the loop counter

VfoIsMem5:
		lds		R24, FRQBIN		; Current VFO frequency to R24
		lds		R25, FRQBIN+1	; ....and to R25
		ld		Xl,Z+			; Get Low byte for memory channel, point to next location
		ld		Xh,Z+			; and the high byte
		sub		R24,Xl			; Subtract VFO from mem freq
		st		Y+,R24			; and store the result in the FrqBinDelta Array
		sbc		R25,Xh			; High Byte
		st		Y+,R25
		brpl	VfoIsMem7		; Delta is positive
		com		R24				; Calculate absolute value
		com		R25				; for a 16 bit one's complement
		add		R24,C1			; negative number
		adc		R25,C0

VfoIsMem7:						; Delta in R24,R25 is now absolute value
		tst		R25
		brne	VfoIsMem10		; Delta is larger then 255 so don't bother checking further
		cpi		R24,VfoMemDelta	; Is delta >= 3?
		brsh	VfoIsMem10		; Yep, try next memory content
		rjmp	VfoIsMem60		; No, VFO = one of the memories +/- DeltaValue

VfoIsMem10:
		inc		R16				; Next mem frequency location
		cpi		R16,6			; Done?
		brne	VfoIsMem5

; No match found between VFO (FRQBIN) and one of the memories
; Now sort the FrqBin Delta Array 
; (variation on bubble sort, less efficient but less code)
; R16 is outer loop counter and is loaded with 6 when we get here
;
VfoIsMem15:
		ldi		R17,5			; inner loop counter
		ldi		Yh,high(FrqBinDelta+12) ; End of data array to sort +1
		ldi		Yl,low(FrqBinDelta+12)		

VfoIsMem20:
		ld		R18,-Y		;msb	(N)
		ld		R19,-Y		;lsb
		ld		R20,-Y		;msb	(N-1)
		ld		R21,-y		; lsb
		cp		R19,R21		; 16 bit signed compare
		cpc		R18,R20
		brlt	VfoIsMem25			; Do not swap

; Store words back but swapped
				
		st		Y,R19		; lsb
		std		Y+1,R18		; msb
		std		Y+2,R21		; lsb
		std		Y+3,R20		; msb

VfoIsMem25:
		adiw	Y,2			; point to new N = N-1
		dec		R17			; inner loop
		brne	VfoIsMem20
		dec		r16			; outer loop
		brne	VfoIsMem15

; FrqBindelta array is now sorted for 16 bit signed binary words
; FrqBinDelta is lsb of highest value
; FrqBinDelta+11 is msb of lowest value

		lds		R16,FrqBinDelta+1 ; MSB  of highest value
		lds		R17,FrqBinDelta+11 ; MSB of lowest value
		sts		Deltasign,C0	; Set delta sign to a positive default
		mov		R18,R16			; Do not clobber R16 with EOR instruction
		eor		R18,R17			; Does array have pos. and neg. numbers?
		brmi	VfoIsMem30		; yes it does
		tst		R17				; MSB  of lowest value
		lds		R16,FrqBinDelta+10
		brpl	VfoIsMem50		; all numbers are positive
		
; All numbers are negative

		lds		R16,FrqBinDelta	
		lds		R17,FrqBinDelta+1
		rjmp	VfoIsMem45				; br always

VfoIsMem30:		; There are positive AND negative numbers in the FrqBinDelta array
		ldi		Yh,high(FrqBinDelta) ; End of data array 
		ldi		Yl,low(FrqBinDelta)	

VfoIsMem35:							; Find the transition from pos to neg number
		ld		R16,Y+		; lsb	(N)
		ld		R17,Y+		; msb
		ld		R18,Y+		; lsb	(N+1)
		ld		R19,y		; msb
		mov		R20,R17
		eor		R20,R19		; Sign different?
		brmi	VfoIsMem40	; Yes
		sbiw	Y,1			; Reset pointer
		rjmp	VfoIsMem35	; Go compare the next two words

VfoIsMem40:
		mov		R20,R18		; Copy N+1 word to some reg's
		mov		R21,R19
		add		R20,R16		; Find the largest delta from 0 by adding the two numbers
		adc		R21,R17
		tst		R21			; Is result negative?
		brmi	VfoIsMem50	; yes
		mov		R16,R18		; Swap N+1 to N
		mov		R17,R19

VfoIsMem45:
		sts		Deltasign,C1	; indicate neagative
		com		R16			; Calculate absolute value
		com		R17			; for a 16 bit one's complement
		add		R16,C1		; negative number
		adc		R17,C0

VfoIsMem50:					;divide the absolute number

.Ifdef FBER68 
		ldi		R18,3		; by 8 for 144-146 code
.Else
		ldi		R18,4		; by 16 for 144-148 code, so it still fits in one byte
.Endif

VfoIsMem55:	
		lsr		R17			; MSB
		ror		R16			; LSB
		dec		r18			; loop counter
		brne	VfoIsMem55
		cpi		R16,3 		; Limit delta to 3
		brlo	VfoIsMem56  
		Ldi		R16,3

VfoIsMem56:					; Add the sign bit times 4
		lds		R17,Deltasign
		lsl		R17
		lsl		R17
		add		R16,R17
		ldi		R17,(Ram_MemDelta-Ram_MemNumDef)/4		; add the offset in the table
		add		R16,R17

VfoIsMem60:						; Point to Character definition for mem symbol
		lsl 	R16				; Character def is 4 rows high, Multiply by 4
		lsl 	R16
		ldi 	ZL, low(Ram_MemNumDef)
		ldi 	ZH, high(Ram_MemNumDef)
		add		ZL,R16			; Calculate the offset in the definition table
		adc		ZH,C0
		ldi 	R25, 7			; Point to char pos 7
		ldi		R26, 4			; Start at row 4
		ldi 	R17, 4			; Character is 4 rows high
		rcall	LCD_Char_Prog 	; Go and program the character
		ret


; *****************************************************************************
; Konstanten Bereich
; *****************************************************************************
;
; *****************************************************************************
; Distance table
;
DistanceTable:
	.db 1,'k',5,1,'k','m'		; 1,5 km, 1km
	.db	7,0,0,5,0,0				; 700m , 500m
	.db	3,0,0,2,0,0				; 300m , 200m
	.db	1,5,0,1,0,0				; 150m , 100m
	.db 5,0,'m',3,0,'m'			; 50m  , 30m
	.db 2,0,'m',1,0,'m'			; 20m  , 10m
	.db 0,5,'m',0,3,'m'			; 5m   , 3m
	.db 0,2,'m',0,1,'m'			; 2m   , 1m
	.db '<',1,'m',$FF			; <1m and padding byte


; *****************************************************************************
; HuntingModes. 
; Table with Valid hunting modes of the receiver.
; Used in Display string selection, previous and next hunting mode selection
; and hunting mode capabilities selection
;
HuntingModes:
	.db		MHuntArdf, 	MHuntArdf12_3
	.db		MhuntArdf2_5, MHuntFpGain
	.db		MHuntFpGainMem, MHuntFpFreq
	.db		MHuntFpFreqMem, $FF
HuntingModesEnd:


; *****************************************************************************
; HuntingModesToggle. Table with the Hunting mode to select after a short press
; on the encoder knob. (Toggles adjust gain or frequency in FP hunting modes)
;
HuntingModesToggle:
	.db		0,0				; Not possible in ARDF mode
	.db		0,MHuntFpFreq
	.db		MHuntFpFreqMem,MHuntFpGain
	.db		MHuntFpGainMem,$ff


; *****************************************************************************
; HuntingModesProp. Table with Hunting mode properties
;
; Bit 0,1	Default Gain Mode, 1= DGC (Auto Gain Reduction), 2 = MGC, 3 = AGC
; Bit 2		HMPFP, Huntingmode is a FP mode
; Bit 3		HMPTimedMem, Huntingmode is a Timed Memory Hunting mode
; Bit 4		HMPWatchPrio,Huntingmode is a Watch Priority (capable)Hunting mode
; Bit 5		HMPAGC, AGC allowed in this hunting mode
; Bit 6		HMPMenuDisp, Display hunting mode in hunting mode selection menu
; Bit 7		Currently not used
;
HuntingModesProp:
	.db 	HMPMenuDisp + DGC , HMPMenuDisp +DGC											; MHuntArdf, 	MHuntArdf12_3
	.db 	HMPMenuDisp + DGC+HMPTimedMem, HMPMenuDisp +HMPAGC + HMPFp + MGC+ HMPWatchPrio	; MhuntArdf2_5,	MHuntFpGain
	.db		HMPMenuDisp + HMPAGC+HMPTimedMem+HMPFp+MGC, HMPAGC+HMPFP+AGC+ HMPWatchPrio	; MHuntFpGainMem, MHuntFpFreq
	.db		HMPAGC+HMPTimedMem+HMPFp+AGC,$FF											; MHuntFpFreqMem, padding byte


; *****************************************************************************
; Hunting Modes Strings
; Addresses of Strings to display when selecting the various hunting modes
; Note: 4 FP modes are using only 2 different strings. 
;
HuntingModesStrings:
	.dW		StrModeHuntArdf		; MHuntArdf
	.dW		StrModeHuntArdf12_3	; MHuntArdf12_3
	.dW		StrModeHuntArdf2_5	; MhuntArdf2_5
	.dw		StrModeHuntFp		; MHuntFpGain
	.dW		StrModeHuntFpMem	; MHuntFpGainMem
	.dw		StrModeHuntFp		; MHuntFpFreq
	.dW		StrModeHuntFpMem	; MHuntFpFreqMem


; *****************************************************************************
; ModeTab
; Table with all the different operating modes of the receiver.
; It is searched for a match with the MODE variable untill the end (SFF) is
; reached.

ModeTab:
		.db MRxOff, MHuntArdf				; Rx Off, ARDF Hunting mode
		.db MHuntArdf12_3, MHuntArdf2_5		; ARDF Hunting modes
		.db MHuntFpGain, MHuntFpGainMem		; FP Hunting modes
		.db	MHuntFpFreq, MHuntFpFreqMem
		.db MFreqEntry, MAuxFreqCh
		.db MAutoRxABSw, MAuxStopw
		.db MAuxTimer, MPQRG
		.db	MAuxSettings,MNFoxes
		.db MTRxA,MTRxB
		.db MTFoxS,MTFoxMs
		.db MAuxHunt,MNQrg
		.db MEOTAlarm,MAcSmLv
		.db MCalStarting,MCalBat
		.db MCalDist,MCalFreq
		.db MCalAttn,MBatAl
		.db MAGCAudio,MCalAttnActive
		.db $FF	,$FF		; End of table, Should never reach this.


; *****************************************************************************
; Table with the adresses of the tables with the different routines to 
; perform for each individual action of the encoder/switch. 
; First step in state machine processing.
;
EncoderStateTable:
		.dw EncoderUp			; Turned Clockwise
		.dw EncoderDown			; Turned Counter Clockwise
		.dw EncoderUpPressed	; Turned Clockwise while switch pressed
		.dw	EncoderDownPressed	; Turned Counter Clockwise while switch pressed
		.dw	EncoderShortPress1	; Short single press on shaft
		.dw EncoderLongPress	; Long press on shaft

		
; *****************************************************************************
;
; Mode action table for Encoder up direction.
;
EncoderUp:

;Hunting modes	Decimal	Hex
;MHuntStart	0	
	.dw Return		; MRxOff	
	.dw GainUp		; MHuntARDF	
	.dw GainUp		; MHuntArdf12_3
	.dw GainUp		; MHuntArdf2_5
	.dw GainUp		; MHuntFpGain	 	
	.dw GainUp		; MHuntFpGainMem
	.dw FreqUp		; MHuntFpFreq
	.dw	FreqUp		; MHuntFpFreqMem
;MHuntStop 
		
;Freq Entry Modes		
;MFreqEntryStart
	.dw FreqUp		; MFreqEntry
;MFreqEntryStop	
		
;Aux Modes		
;MAuxStart
	.dw AuxModesNext ; MAuxFreqCh
	.dw	AuxModesNext ; MAutoRxABSw	
	.dw AuxModesNext ; MAuxStopw	
	.dw AuxModesNext ; MAuxTimer	
	.dw AuxModesNext ; MPQRG
;	.dw	AuxModesNext ; MLEDBright
	.dw AuxModesNext ; MAuxSettings	
;MAuxStop
		
;Setting Modes		
;MSetStart							
	.dw SettingModesNext	; ModeNFoxes
	.dw SettingModesNext	; MTRxA
	.dw SettingModesNext	; MTRxB		
	.dw SettingModesNext	; ModeTFoxS		
	.dw SettingModesNext	; ModeTFoxMs	
	.dw SettingModesNext	; MAuxHunt		
	.dw SettingModesNext	; ModeNQrg		
	.dw SettingModesNext	; ModeEOTAlarm 	
	.dw SettingModesNext	; ModeAcSmLv	
;MSetStop
		
;Calibration Modes		
;MCalStart							
	.dw	CalModesNext		; ModeCalStarting
	.dw CalModesNext		; ModeCalBat
	.dw CalModesNext		; MCalDist	
	.dw CalModesNext		; ModeCalFreq	
	.dw CalModesNext		; ModeCalAttn	
	.dw CalModesNext		; ModeBatAl		
	.dw CalModesNext		; MAGCAudio
;MCalStop

;Attenuator Calibration active
;MCalAttnStart	
	.dw	CalAttUp			; MCalAttnActive
;MCalAttnStop


; *****************************************************************************
; Mode action table for Encoder down direction.
;
EncoderDown:
;Hunting modes	Decimal	Hex
;MHuntStart						
	.dw Return		; MRxOff		
	.dw GainDown	; MHuntARDF		
	.dw GainDown	; MHuntArdf12_3
	.dw GainDown	; MHuntArdf2_5
	.dw GainDown	; MHuntFpGain
	.dw GainDown	; MHuntFpGainMem
	.dw FreqDown	; MHuntFpFreq
	.dw	FreqDown	; MHuntFpFreqMem
;MHuntStop
		
;Freq Entry Modes		
;MFreqEntryStart				S10
	.dw FreqDown	; MFreqEntry
;MFreqEntryStop
		
;Aux Modes		
;MAuxStart						
	.dw AuxModesPrev ; MAuxFreqCh
	.dw AuxModesPrev ; MAutoRxABSw	
	.dw AuxModesPrev ; MAuxStopw	
	.dw AuxModesPrev ; MAuxTimer	
	.dw AuxModesPrev ; MPQRG
;	.dw	AuxModesPrev ; MLEDBright	
	.dw AuxModesPrev ; MAuxSettings
;MAuxStop
		
;Setting Modes		
;MSetStart						
	.dw SettingModesPrev	; ModeNFoxes
	.dw SettingModesPrev	; MTRxA
	.dw SettingModesPrev	; MTRxB	
	.dw SettingModesPrev	; ModeTFoxS
	.dw SettingModesPrev	; ModeTFoxMs
	.dw SettingModesPrev	; MAuxHunt
	.dw SettingModesPrev	; ModeNQrg
	.dw SettingModesPrev	; ModeEOTAlarm
	.dw SettingModesPrev	; ModeAcSmLv
;MSetStop
		
;Calibration Modes		
;MCalStart						
	.dw	CalModesPrev		; ModeCalStarting
	.dw CalModesPrev		; ModeCalBat
	.dw CalModesPrev		; MCalDist
	.dw CalModesPrev		; ModeCalFreq
	.dw CalModesPrev		; ModeCalAttn
	.dw CalModesPrev		; ModeBatAl
	.dw CalModesPrev		; MAGCAudio	
;MCalStop	

;Attenuator Calibration active
;MCalAttnStart					
	.dw	CalAttDown			; MCalAttnActive
;MCalAttnStop


; *****************************************************************************
;
; Mode action table for Encoder up+pressed.
;
EncoderUpPressed:
;Hunting modes	Decimal	Hex
;MHuntStart				
	.dw Return		; MRxOff
	.dw NextMem		; MHuntARDF
	.dw SetMem3		; MHuntArdf12_3
	.dw SetMem3		; MHuntArdf2_5
	.dw NextMem		; MHuntFpGain 	
	.dw SetMem6		; MHuntFpGainMem
	.dw NextMem		; MHuntFpFreq
	.dw	SetMem6		; MHuntFpFreqMem
;MHuntStop
		
;Freq Entry Modes		
;MFreqEntryStart		
	.dw NextFreqIndex		; MFreqEntry
;MFreqEntryStop
		
;Aux Modes		
;MAuxStart				
	.dw Return		; MAuxFreqCh
	.dw	Return		; MAutoRxABSw
	.dw Return		; MAuxStopw	
	.dw IncrFox		; MAuxTimer	
	.dw	PQrgUp		; MPQrg
;	.dw	Return		; MLEDBright
	.dw Return		; MAuxSettings	
;MAuxStop
		
;Setting Modes		
;MSetStart				
	.dw NFoxesUp	; ModeNFoxes
	.dw TrxAUp		; MTRxA
	.dw TrxBUp		; MTRxB	
	.dw TFoxSUp		; ModeTFoxS	
	.dw TFoxMsUp	; ModeTFoxMs
	.dw MHuntNext	; MAuxHunt
	.dw NQrgUp		; ModeNQrg
	.dw EOTAlarmUp	; ModeEOTAlarm
	.dw AcSmLvlUp	; ModeAcSmLv
;MSetStop
		
;Calibration Modes		
;MCalStart				
	.dw	Return		; ModeCalStarting
	.dw UBFineUp	; ModeCalBat
	.dw DistCorUp	;MCalDist
	.dw CalFreqUp	; ModeCalFreq
	.dw Return		; ModeCalAttn
	.dw BatteryAlarmUp	; ModeBatAl
	.dw Return		; MAGCAudio	
;MCalStop


;Attenuator Calibration active
;MCalAttnStart			
	.dw	CalAttNextPos	; MCalAttnActive
;MCalAttnStop


; *****************************************************************************
; Mode action table for Encoder Down+pressed.
;
EncoderDownPressed:
;Hunting modes	Decimal	Hex
;MHuntStart					
	.dw Return				; MRxOff
	.dw PrevMem				; MHuntARDF
	.dw FreqDownArdf12_3	; MHuntArdf12_3
	.dw FreqDownArdf2_5		; MHuntArdf2_5
	.dw PrevMem				; MHuntFpGain 
	.dw MemDownFpMem		; MHuntFpGainMem
	.dw PrevMem				; MHuntFpFreq
	.dw	MemDownFpMem		; MHuntFpFreqMem
;MHuntStop
		
;Freq Entry Modes		
;MFreqEntryStart
	.dw PrevFreqIndex		; MFreqEntry
;MFreqEntryStop
		
;Aux Modes		
;MAuxStart
	.dw Return		; MAuxFreqCh
	.dw Return		; MAutoRxABSw
	.dw Return		; MAuxStopw	
	.dw DecrFox		; MAuxTimer	
	.dw	PQrgDown	; MPQRG
;	.dw	Return		; MLEDBright
	.dw Return		; MAuxSettings	
;MAuxStop
		
;Setting Modes		
;MSetStart
	.dw NFoxesDown	; ModeNFoxes
	.dw	TrxADown	; MTRxA
	.dw TrxBDown	; MTRxB		
	.dw TFoxSDown	; ModeTFoxS		
	.dw TFoxMsDown	; ModeTFoxMs	
	.dw MHuntPrev	; MHunt		
	.dw NQrgDown	; ModeNQrg		
	.dw EOTAlarmDown ; ModeEOTAlarm 
	.dw AcSmLvlDown	; ModeAcSmLv	
;MSetStop
		
;Calibration Modes		
;MCalStart					
	.dw	Return		; ModeCalStarting
	.dw UBFineDown	; ModeCalBat
	.dw DistCorDown	; MCalDist
	.dw CalFreqDown	; ModeCalFreq
	.dw Return		; ModeCalAttn
	.dw BatteryAlarmDown 	; ModeBatAl
	.dw Return		; MAGCAudio
;MCalStop


;Attenuator Calibration active
;MCalAttnStart				
	.dw	CalAttPrevPos	; MCalAttnActive
;MCalAttnStop


; *****************************************************************************
; Mode action table for Encoder short press.
;
EncoderShortPress1:
;Hunting modes	Decimal	Hex
;MHuntStart					
	.dw Return			; MRxOff		0
	.dw ToggleAcsM		; MHuntARDF		1
	.dw ToggleAcsM		; MHuntArdf12_3
	.dw ToggleAcsM		; MHuntArdf2_5
	.dw ToggleMHuntFP		; MHuntFpGain 	
	.dw ToggleMHuntFPMem	; MHuntFpGainMem 	
	.dw ToggleMHuntFp		; MHuntFpFreq
	.dw	ToggleMHuntFPMem	; MHuntFpFreqMem
;MHuntStop
		
;Freq Entry Modes		
;MFreqEntryStart			
	.dw NextMem		; MFreqEntry 
;MFreqEntryStop	
		
;Aux Modes		
;MAuxStart					
	.dw MFreqEntrySelect	; MAuxFreqCh
	.dw ToggleAutoRxABSw	; MAutoRxABSw
	.dw StopwStartStop		; MAuxStopw
	.dw ClrTim				; MAuxTimer	
	.dw NextMem				; MPQrg
;	.dw ToggleLEDBright		; MLEDBright
	.dw MSetStartSelect		; MAuxSettings	
;MAuxStop
		
;Setting Modes		
;MSetStart					
	.dw Return		; ModeNFoxes
	.dw Return		; MTRxA
	.dw Return		; MTRxB	
	.dw Return		; ModeTFoxS
	.dw Return		; ModeTFoxMs
	.dw MHuntNext	; MHunt
	.dw Return		; ModeNQrg
	.dw Return		; ModeEOTAlarm
	.dw Return		; ModeAcSmLv
;MSetStop	
		
;Calibration Modes		
;MCalStart					
	.dw	Return		; ModeCalStarting	
	.dw Return		; ModeCalBat
	.dw	Return		; MCalDist	
	.dw Return		; ModeCalFreq	
	.dw MSetCalAtt	; ModeCalAttn	
	.dw Return		; ModeBatAl	
	.dw ToggleAGCAudio	;MAGCAudio
;MCalStop


;Attenuator Calibration active
;MCalAttnStart				
	.dw	CalAttNextPos	; MCalAttnActive
;MCalAttnStop


; *****************************************************************************
; Mode action table for Encoder Long pressed
;
EncoderLongPress:
;Hunting modes	Decimal	Hex
;MHuntStart	0	
	.dw Return		; MRxOff
	.dw ToggleAcsM	; MHuntARDF
	.dw ToggleAcsM	; MHuntArdf12_3
	.dw ToggleAcsM	; MHuntArdf2_5
	.dw ToggleAcsM	; MHuntFpGain 
	.dw ToggleAcsM	; MHuntFpGainMem
	.dw ToggleAcsM	; MHuntFpFreq
	.dw	ToggleAcsM	; MHuntFpFreqMem
;MHuntStop

;Freq Entry Modes		
;MFreqEntryStart
	.dw StoreMem	; MFreqEntry
;MFreqEntryStop
		
;Aux Modes		
;MAuxStart
	.dw Return		; MAuxFreqCh
	.dw	Return		; MAutoRxABSw
	.dw Return		; MAuxStopw	
	.dw Return		; MAuxTimer	
	.dw	Return		; MAuxHunt
;	.dw	Return		; MLEDBright
	.dw Return		; MAuxSettings	
;MAuxStop	
		
;Setting Modes		
;MSetStart					
	.dw Return	; ModeNFoxes
	.dw Return		; MTRxA
	.dw Return		; MTRxB	
	.dw Return	; ModeTFoxS
	.dw Return	; ModeTFoxMs
	.dw Return	; ModePQrg
	.dw Return	; ModeNQrg
	.dw Return	; ModeEOTAlarm
	.dw Return	; ModeAcSmLv
;MSetStop	
		
;Calibration Modes		
;MCalStart	
	.dw	Return		; ModeCalStarting
	.dw Return		; ModeCalBat
	.dw Return		; MCalDist
	.dw Return		; ModeCalFreq
	.dw Return		; ModeCalAttn
	.dw Return		; ModeBatAl
	.dw	Return		; MAGCAudio
;MCalStop	


;Attenuator Calibration active
;MCalAttnStart	
	.dw	Return		; MCalAttnActive
;MCalAttnStop



; *****************************************************************************
;ShowTab
; Table with the address of the LCD string to be displayed for each entry in
; ModeTab. 0 indicates an exception and is processed seperate
; (Toggling parameters requiring different strings)
;
ShowTab:
; ARDF Hunting modes
	.dw	0	; Rx off
	.dw	0
	.dw	0
	.dw	0

; FP Hunting modes
	.dw	StrModeHuntFpGain<<1
	.dw	StrModeHuntFpGainMem1<<1
	.dw	StrModeHuntFpFreq<<1
	.dw	StrModeHuntFpfreqMem1<<1

; Frequency Entry modes
	.dw	StrModeFreqInp<<1

; Aux modes
	.dw	StrAuxFreqCh<<1
	.dw 0
	.dw	0
	.dw	StrModeFoxTimer<<1
	.dw	StrModePQrg<<1
;	.dw	0
	.dw	StrModeSettings<<1

; Setting Modes
	.dw	StrModeNFoxes<<1
	.dw StrModeTimeRxA<<1
	.dw	StrModeTimeRxB<<1
	.dw	StrModeTFoxS<<1
	.dw	StrModeTFoxMs<<1
	.dw	0
	.dw	StrModeNQrg<<1
	.dw	StrModeEOTAlarm<<1
	.dw	StrModeAcSmLvl<<1

; Calibration Modes
	.dw	StrAdjustmentModesStart<<1
	.dw	StrModeCalBat<<1
	.dw StrModeCalDist<<1
	.dw	StrModeCalFreq<<1
	.dw	StrModeCalAttn<<1
	.dw	StrModeBatAl<<1
	.dw 0

;Attenuator Calibration
	.dw	StrCalAttn<<1


; *****************************************************************************
; Display Strings
;
; Die ersten 16 Bytes sind der Festtext-Teil der Anzeige
;  ":" in Variablenposition = 0 als Space zeigen (Leading Zero Suppress)
;   0  in Variablenposition = Komprimiertes dB Zeichen
;   1         "             = Pfeile Up + Down
;   2         "             = Block 1 Linie breit (S-Meter)
;   3         "             = Block 2 Linien breit
;   4         "             = Block 3 Linien breit
;	5		  "             = Block 4 Linien breit
;   6         "             = Currently not used
;   7         "             = Dynamic; Gain Mode and Memory are combined
; $E1 = , $EF = , $F5 = , $E2 = , kein , , , $7E = ->, $7F = <-
;
; Die zweiten 16 Bytes sind jeweils Pointer auf die Variablen
;  1..8 = Leer
; Variablenname = Variable (Adresse <= $FF!) 
;  0 = Nchste Stelle der aktuellen Variable
; Variablenwerte 0..9 werden in ASCII '0'..'9' umgesetzt
;  " $80..87 werden in Sonderzeichen 0..7 (siehe oben) umgesetzt
;
; Fr Texte ohne Variablen gengen 18 Byte: Erste 16 Byte wie oben, danach 0,0
; 
StrAdjustmentModesStart:
	;   	>12345678<
	.DB  	"Adjustm."
	.DB		"Out=Done"
	.DW	0
	
StrAla:	
;   		>12345678<	Sendezeit-Ende Alarm 
	.DB  	"FoxOffIn"
	.DB		" :. sec "
	.DB     1,2,3,4,5,6,7,8  
	.DB     1,TALARM,0,4,5,6,7,8
;
;
StrBat:
;		   	>12345678<	Batteriespannung niedrig
	.DB  	"Battery "
	.DB		" < .,. V"
	.DB     1,2,3,4,5,6,7,8  
	.DB     1,2,3,UBWARN,5,0,7,8
;
;
STRAuxFreqCh:
;   		>12345678<	Frequenz ndern anbieten
	.DB  	"14.:... "
	.DB		$7E," Change"
	.DB     1,2,Freq,GainMemSymbol,Freq+1,0,0,RxXSymbol  
	.DB     1,2,3,4,5,6,7,8;
;
;
StrCalAttn:	
	;   	>12345678<	Abschwcher kalibrieren
	.DB  	":.. :..",0
	.DB		"        "
	.DB     ABSVAL,0,0,4,ABSCHW,0,0,8  
	.DB     UMESS,0,0,0,0,0,0,0
;
;
StrFrq:
	;   	>12345678<	Kanal gendert
	.DB  	"14..... "
	.DB		"   - . -"
	.DB     1,2,Freq,GainMemSymbol,Freq+1,0,0,8  
	.DB     1,2,3,4,5,FRQX,7,8
;
;
StrModeAcSmLvl:
;		   	>12345678<	Akustisches S-Meter Schwelle einstellen
	.DB  	"Audio.SM"
	.DB		"over ./8"
	.DB     1,2,3,4,5,6,7,8  
	.DB     1,2,3,4,5,ACSMLV,7,8
;
;
StrModeAutoRxABSwOff:
	;   	>12345678<
	.DB		"<> RxA/B"
	.DB		"Auto OFF"
	.DW	0
;
;
StrModeAutoRxABSwOn:
	;   	>12345678<
	.DB		"<> RxA/B"
	.DB		"Auto ON "
	.DW	0
;
;
StrModeBatAl:
;   		>12345678<	Batterie-Warnschwelle einstellen
	.DB  	"BatWarng"
	.DB		"< .,. V",1
	.DB     1,2,3,4,5,6,7,8  
	.DB     1,2,UBWARN,4,0,6,7,8
;
;
StrModeCalAttn:
;		   	>12345678<	Abschwcher kalibrieren
	.DB  	"Cal.Attn"
	.DB		$7E," Start "
	.DW	0
;
;
StrModeCalBat:
;		   	>12345678<	UBat kalibrieren

	.DB  	"Cal.UBat"
	.DB     "...:.,.V"  
	.DB     1,2,3,4,5,6,7,8  
	.DB     UBFVAL,0,0,UBATT,0,6,0,8
;
;
StrModeCalDist:
;   		>12345678<	Entfernungs-Korrektur
	.DB  	"Cal.Dist"
	.DB		":..  ..",1
	.DB     1,2,3,4,5,6,7,8  
	.DB     DIST,0,0,4,5,DCFVAL,0,8
;
;
StrModeCalFreq:
;	 	  	>12345678<	Frequenz kalibrieren
	.DB  	"Cal.F..."
	.DB		"        "
	.DB     1,2,3,4,5,FFIVAL,0,0  
	.DB     UMESS,0,0,0,0,0,0,0
;
;
StrModeEOTAlarm:
;	   	>12345678<	EOT Alarmzeit einstellen
	.DB  	"Tx Alarm"
	.DB		"- :.s ",1," "
	.DB     1,2,3,4,5,6,7,8  
	.DB     1,2,TALARM,0,5,6,7,8
;
;
StrModeFoxTimer:
;		   	>12345678<	Timer neu starten
	.DB  	"TmrStart"
	.DB		"F:.  :.s"
	.DB     1,2,3,4,5,6,7,8  
	.DB     1,FOX,0,4,5,TIMER,0,8
;
;
StrModeFreqInp:				;Changing Frequency mode
;   		>12345678<
	.DB  	"14.....",1
	.DB		"        "
	.DB     1,2,Freq,Gainmemsymbol,Freq+1,0,0,8
	.DB     UMESS,0,0,0,0,0,0,FRQX
;
;
StrModeHuntArdf:
	;		>12345678<
	.dB		"DF1FO  ",1
	.dB		"ARDF Std"
	.dW 0
;
;
StrModeHuntArdf12_3:
;			>12345678<
	.dB		"DF1FO  ",1
	.dB		"ARDF12>3"
	.dW 0
;
;
StrModeHuntArdf2_5:
;			>12345678<
	.dB		"DF1FO  ",1
	.dB		"ARDF 2*5"
	.dW 0
;
;
StrModeHuntArdfStopw:
;			>12345678<
	.DB  	". .. :.."	
	.DB	"        "
	.DB     STOPW,GainMemSymbol,STOPW+1,0,AUTOON,DIST,0,0   
	.DB     UMESS,0,0,0,0,0,0,0
;
;
StrModeHuntArdfTimer:
	.DB  	". :. :.."	
	.DB	"        "
	.DB     FOX+1,GainMemSymbol,TIMER,0,AUTOON,DIST,0,0    
	.DB     UMESS,0,0,0,0,0,0,0
;
;
StrModeHuntFp:
;			>12345678<
	.dB		"PA2PIM ",1
	.dB		"FP Mode "
	.dW 0
;
;
StrModeHuntFpFreq:			;FP Hunting mode frequency adjust
	;   	>12345678<
	.DB		"        "
	.DB		"        "
	.DB		Dist,0,0,GainMemSymbol,Freq,0,0,0
	.DB		UMESS,0,0,0,0,0,0,0
;
;
StrModeHuntFpFreqMem1:		;FP Hunting Mem mode frequency adjust
	;   	>12345678<
	.DB		"        "
	.DB		"     F: "
	.DB		Dist,0,0,GainMemSymbol,Freq,0,0,0
	.DB		UMESS,0,0,0,0,6,Fox,0
;
;
StrModeHuntFpGain:			;FP Hunting mode Gain adjust
	;   	>12345678<
	.DB		"        "
	.DB		"        "
	.DB		Freq,0,0,0,GainMemSymbol,Dist,0,0
	.DB		UMESS,0,0,0,0,0,0,0
;
;
StrModeHuntFpGainMem1:		;FP Hunting Mem mode Gain adjust
	;   	>12345678<		; Low RSSI
	.DB		"        "
	.DB		"     F: "
	.DB		Freq,0,0,0,GainMemSymbol,Dist,0,0
	.DB		UMESS,0,0,0,0,6,Fox,0
;
;
StrModeHuntFpMem:
;			>12345678<
	.dB		"PA2PIM ",1
	.dB		"FP Mem M"
	.dW 0
;
;
StrModeMenuStart:
	;   	>12345678<	Men neu aufgemacht
	.DB  	"14..... "
	.DB		".:..:.,."
	.DB     1,2,Freq,GainMemSymbol,Freq+1,0,0,RxXSymbol
	.DB     STOPW,1,0,0,UBATT,0,7,0
;
;
StrModeNFoxes:
;		   	>12345678<	Zahl Timer einstellen
	.DB  	"# Timed",1
	.DB		"Foxes : "
	.DB     1,2,3,4,5,6,7,8  
	.DB     1,2,3,4,5,6,NFOX,0
;
;
StrModeNQrg:
;		   	>12345678<	Zahl Frequenzen einstellen 
	.DB  	"# QRG's "
	.DB	"   : ",1,"  "
	.DB     1,2,3,4,5,6,7,8  
	.DB     1,2,3,NFREQ,5,6,7,8
;
;
StrModePQrg:
;		   	>12345678<	Sendeleistung Fchse einstellen
	.DB  	"P QRG   "
	.DB		":.,. .W",1
	.DB     1,2,3,4,5,6,GainMemSymbol,8
	.DB     PFOX,0,3,0,5,0,7,8
;
;
StrModeRxOffStopW:
;   		>12345678<	RX Off
	.DB  	" RX OFF "
	.DB		".:..:.. "
	.DB     1,2,3,4,5,6,7,8  
	.DB     STOPW,2,0,0,5,0,0,8
;
;
StrModeRxOffTimer:
;		   	>12345678<	RX Off
	.DB  	" RX OFF "
	.DB		"F:.  :.s"
	.DB     1,2,3,4,5,6,7,8
	.DB     1,FOX,0,4,5,TIMER,0,8
;
;
StrModeSettings:
;	   		>12345678<	Set-Up-Menu aufrufen
	.DB  	"Settings"
	.DB	$7E, "  -menu"
	.DW 0
;
;
StrModeStopwStart:
; 		  	>12345678<	Stopuhr neu starten
	.DB  	"ClkStart"
	.DB		".:..:.. "
	.DB     1,2,3,4,5,6,7,8  
	.DB     STOPW,2,0,0,5,0,0,8
;
;
StrModeStopwStop:
;		   	>12345678<	Stopuhr stoppen
	.DB  	"Clk Stop"
	.DB		".:..:.. "
	.DB     1,2,3,4,5,6,7,8  
	.DB     STOPW,2,0,0,5,0,0,8
;
;
StrModeTFoxMs:	
;		   	>12345678<	Zeit Fchse ms einstellen
	.DB  	" T Fox ",1
	.DB		":.,>..<s"
	.DB     1,2,3,4,5,6,7,8  
	.DB     TFOXS,0,3,4,0,0,7,8
;
;
StrModeTFoxS:
;		   	>12345678<	Zeit Fchse Sekunden einstellen 
	.DB  	" T Fox ",1
	.DB		">:.<,..s"
	.DB     1,2,3,4,5,6,7,8  
	.DB     1,TFOXS,0,4,5,0,0,8
;
;
StrModeTimeRxA:
;   		>12345678<	RxA on time
	.DB  	"Time RxA"
	.DB		"ON ",1,"::.s"
	.DB     1,2,3,4,5,6,7,8  
	.DB     1,2,3,4,RxATime,0,0,8
;
;
StrModeTimeRxB:
;   		>12345678<	RxA on time
	.DB  	"Time RxB"
	.DB		"ON ",1,"::.s"
	.DB     1,2,3,4,5,6,7,8  
	.DB     1,2,3,4,RxBTime,0,0,8
;
;
;StrModeToggleLEDBrightOn:
;   		>12345678<	
;	.DB  	"LCD LED "
;	.DB		"Bright<>"
;	.Dw     0
;
;
;StrModeToggleLEDBrightOFF:
;   		>12345678<	
;	.DB  	"LCD LED "
;	.DB		"Dim   <>"
;	.Dw     0
;
StrMToggleAGCAudioOn:
;			>12345678<	
	.DB  	"AGCAudio"
	.DB		"Tone ON "
	.Dw     0
;
StrMToggleAGCAudioOFF:
;			>12345678<	
	.DB  	"AGCAudio"
	.DB		"Tone OFF"
	.Dw     0

StrPowerOn:
;   		>12345678<	Power On
	.DB 	"FJRX-2FP"	;FP added 
	.DB		"Ver. ",VERSHI,".",VERSLO	
	.DW     0
	;


; *****************************************************************************
; QRG Power strings
; pfox pfox+1, pfox+2<space>pfox+3
;
PFoxStrings:
PFoxStringsStart:
.db	0,1,0,$E4					;1 micro
.db 0,3,0,$E4					;3 micro
.db	1,0,0,$E4					;10 micro
.db	3,0,0,$E4					;30 micro
.db	0,0,1,'m'					;0,1 milli
.db 0,0,3,'m'					;0,3 milli
.db	0,1,0,'m'					;1,0 milli
.db 0,3,0,'m'					;3,0 milli
.db 1,0,0,'m'					;10 milli
.db 3,0,0,'m'					;30 milli
.db 0,0,1,' '					;0,1W
.db 0,0,3,' '					;0,3W
.db 0,1,0,' '					;1,0W
.db 0,3,0,' '					;3,0W
.db	1,0,0,' '					;10,0W
.db	3,0,0,' '					;30,0W
.db '?','?','?',' ' 			; Unknown power
PFoxStringsEnd:


; *****************************************************************************
; Sound Strings
;
; Diese Strings enthalten Tonwertfolgen, die im 100ms-Raster 
;   abgespielt werden. Frequenz ist 10 kHz / Wert:
; 3 = 3,3 kHz, 10 = 1 kHz, 100 = 100 Hz
; Sonderflle: 1 = kein Ton, 0 = End of String  
;
SOUPO:
		.DB	1,9,1,9,1,9,1,13	; Power-On-Fanfare
		.DB	13,13,13,0	

;SouGainUp:			.db 1,1,10,10,15,1,1,0
;SouGainUpSerie: 	.db 10,10,15,1,1,0
;SougainDown: 		.db 1,1,20,20,30,1,1,0
;SouGainDownSerie: 	.db 20,20,30,1,1,0


 SOUABS:	.DB	1,1,20,10,1,1,0,0	; Automatik-Abschwcher

 SOUABSs:	.DB	10,10,0,0	; Automatik-Abschwcher in freq entry mode
 SOUAB2:	.DB	20,10,20,10,1,1,0,0	; Automatik-Abschwcher, Sound fr Serie

SOUALA:	.DB	3,4,6,9,12,18,24,0	; 'Ende - x sec'-Alarm	

SOUBAT:							; Low Battery
		.DB	15,10,5,10
		.Db 16,11,6,11
		.Db 17,12,7,12
		.Db 18,13,8,13
		.Db 19,14,9,14
		.Db 20,15,10,15
		.Db 21,16,11,16
		.DB	22,17,12,17
		.DB 22,22,22,22
		.db 22,0

;SOUAA0:	.DB	1,25,1,25,1,25,1,0	; Abschwcher-Automatik aus


;SOUAA1:	.DB	1,25,25,1,1,25,25,1,0,0	; Abschwcher-Automatik ein		

SOUF1:	.DB	1,15,1,1,1,10,1,10,10,10,1,10,10,10,1,10,10,10,1,10,10,10,1,0	; Frequenz 1
SOUF2:	.DB	1,15,1,1,1,10,1,10,1,10,10,10,1,10,10,10,1,10,10,10,1,0		; F 2
SOUF3:	.DB	1,15,1,1,1,10,1,10,1,10,1,10,10,10,1,10,10,10,1,0			; F 3
SOUF4:	.DB	1,15,1,1,1,10,1,10,1,10,1,10,1,10,10,10,1,0			; F 4
SOUF5:	.DB	1,15,1,1,1,10,1,10,1,10,1,10,1,10,1,0			; F 5
SOUF6:	.DB	1,15,1,1,1,10,10,10,1,10,1,10,1,10,1,10,1,0			; F 6

SouA:	.dB 1,15,1,1,1,10,1,10,10,10,1,0			; A  (Receiver A selected, or AGC)
SouB:	.dB 1,15,1,1,1,10,10,10,1,10,1,10,1,10,1,0	; B  (Receiver B selected)
SouD:	.db 1,15,1,1,1,10,10,10,1,10,1,10,1,0		; D DF1FO gain mode
SouM:	.db 1,15,1,1,1,10,10,10,1,10,10,10,1,0		; M (Manual Gain Control)
SouP:	.DB 1,15,1,1,1,10,1,10,10,10,1,10,10,10,1,10,1,0 ; P(rogrammed memory)

; Store the compilation date and time at the end of the flash mem
;.org FLASHEND-4
.db	__DATE__,",", __TIME__



;******************************************************************************
;
; E E P R O M  D A T A 
;
;******************************************************************************
.eseg
.org 0

; *****************************************************************************
; EEprom init
;
EePromInitValues:
;																					EEP1					
;	Tbd1,  F1 144.1, F2 144.2, F3 144.3, F4 144.4, F5 144.5, F6,144.6, Active Freq,	# Foxes,
	.Db	0, $97,$02,  $F0,0,    $40,$01,  $90,$01,  $E0,$01,  $30,$02,  $01,			0,5
;
; 		Tx time 60.00s,	Pfox1-6,   			PFoxBN,	,NFREQ,	Alarmtime,	ACSMLV threshold,Fr Modus 1x2<>3
	.Db 6,0,0,0,      	16,16,16,16,16,16,  16,   	 1,		1,0,		0,				1
;																						,EEP2:
; Fr Modus 1x2<>3: Fuchs 10,1-9, jeweils Frequenz 1, MHuntX, RxATimeBin,RxBTimeBin,	Abschwchertabelle 0,5
	.DB	 			1,1,1,1,1,1,1,1,1,					  MHuntFpGain,30,	5,			18,35
;
; Abschwchertabelle 10..45 dB
	.DB	50,61,70,76,82,87,95,107

; Abschwchertabelle 50 85 dB
	.DB 120,135,145,73,80,85,91,102

; Abschwchertabelle 90..120 dB, 	Freq-Korrektur (+01)
	.DB 114,127,142,147,150,154,159	,65

;	Ub-Mess-Korrektur,	Bat Alarm 8,0V,	DISTCF, AGCAudio
	.DB 	126,				80,		0, 		1
EePromInitValuesEnd:	


; *****************************************************************************
; Acoustic S meter lookup table.
; Table index is RSSI value out of TCA440 chip and A/D converted by  CPU
;
; Contains divider values to be put in Timer 1
;
; Code is generated by spreadsheet: Audio S meter curve generator.xls
;
; Spreadsheet allows different min max frequencies and weighting factors for
; an exponential, logarithmic and linear curve.
; This particular table has the following parameters:
; Min Freq.	100 Hz
; Max Freq.	3000 Hz
; Max_RSSI	150 A/D counts
; FCrystal	5000000 Hz
; Weighting:
; 	Exponential		50%
; 	lin				50%
; 	Log				0%
;	
ACSMlookup:
.Dw	25000		;	100	Hz
.Dw	22560		;	111	Hz
.Dw	20550		;	122	Hz
.Dw	18865		;	133	Hz
.Dw	17432		;	143	Hz
.Dw	16198		;	154	Hz
.Dw	15125		;	165	Hz
.Dw	14183		;	176	Hz
.Dw	13349		;	187	Hz
.Dw	12605		;	198	Hz
.Dw	11939		;	209	Hz
.Dw	11337		;	220	Hz
.Dw	10792		;	232	Hz
.Dw	10296		;	243	Hz
.Dw	9841		;	254	Hz
.Dw	9424		;	265	Hz
.Dw	9040		;	277	Hz
.Dw	8685		;	288	Hz
.Dw	8355		;	299	Hz
.Dw	8049		;	311	Hz
.Dw	7763		;	322	Hz
.Dw	7496		;	333	Hz
.Dw	7246		;	345	Hz
.Dw	7011		;	357	Hz
.Dw	6790		;	368	Hz
.Dw	6582		;	380	Hz
.Dw	6385		;	391	Hz
.Dw	6200		;	403	Hz
.Dw	6023		;	415	Hz
.Dw	5857		;	427	Hz
.Dw	5698		;	439	Hz
.Dw	5547		;	451	Hz
.Dw	5403		;	463	Hz
.Dw	5266		;	475	Hz
.Dw	5136		;	487	Hz
.Dw	5011		;	499	Hz
.Dw	4891		;	511	Hz
.Dw	4776		;	523	Hz
.Dw	4666		;	536	Hz
.Dw	4561		;	548	Hz
.Dw	4460		;	561	Hz
.Dw	4362		;	573	Hz
.Dw	4269		;	586	Hz
.Dw	4179		;	598	Hz
.Dw	4092		;	611	Hz
.Dw	4008		;	624	Hz
.Dw	3927		;	637	Hz
.Dw	3849		;	649	Hz
.Dw	3773		;	662	Hz
.Dw	3700		;	676	Hz
.Dw	3630		;	689	Hz
.Dw	3561		;	702	Hz
.Dw	3495		;	715	Hz
.Dw	3431		;	729	Hz
.Dw	3368		;	742	Hz
.Dw	3308		;	756	Hz
.Dw	3249		;	769	Hz
.Dw	3192		;	783	Hz
.Dw	3137		;	797	Hz
.Dw	3083		;	811	Hz
.Dw	3030		;	825	Hz
.Dw	2979		;	839	Hz
.Dw	2929		;	853	Hz
.Dw	2881		;	868	Hz
.Dw	2834		;	882	Hz
.Dw	2788		;	897	Hz
.Dw	2743		;	911	Hz
.Dw	2699		;	926	Hz
.Dw	2656		;	941	Hz
.Dw	2614		;	956	Hz
.Dw	2574		;	971	Hz
.Dw	2534		;	986	Hz
.Dw	2495		;	1002	Hz
.Dw	2457		;	1017	Hz
.Dw	2420		;	1033	Hz
.Dw	2383		;	1049	Hz
.Dw	2347		;	1065	Hz
.Dw	2312		;	1081	Hz
.Dw	2278		;	1097	Hz
.Dw	2245		;	1114	Hz
.Dw	2212		;	1130	Hz
.Dw	2180		;	1147	Hz
.Dw	2148		;	1164	Hz
.Dw	2117		;	1181	Hz
.Dw	2087		;	1198	Hz
.Dw	2057		;	1215	Hz
.Dw	2027		;	1233	Hz
.Dw	1999		;	1251	Hz
.Dw	1970		;	1268	Hz
.Dw	1943		;	1287	Hz
.Dw	1915		;	1305	Hz
.Dw	1889		;	1323	Hz
.Dw	1862		;	1342	Hz
.Dw	1837		;	1361	Hz
.Dw	1811		;	1380	Hz
.Dw	1786		;	1399	Hz
.Dw	1761		;	1419	Hz
.Dw	1737		;	1439	Hz
.Dw	1713		;	1459	Hz
.Dw	1690		;	1479	Hz
.Dw	1667		;	1499	Hz
.Dw	1644		;	1520	Hz
.Dw	1622		;	1541	Hz
.Dw	1600		;	1562	Hz
.Dw	1578		;	1584	Hz
.Dw	1556		;	1606	Hz
.Dw	1535		;	1628	Hz
.Dw	1515		;	1650	Hz
.Dw	1494		;	1673	Hz
.Dw	1474		;	1696	Hz
.Dw	1454		;	1719	Hz
.Dw	1434		;	1742	Hz
.Dw	1415		;	1766	Hz
.Dw	1396		;	1791	Hz
.Dw	1377		;	1815	Hz
.Dw	1358		;	1840	Hz
.Dw	1340		;	1865	Hz
.Dw	1322		;	1891	Hz
.Dw	1304		;	1917	Hz
.Dw	1286		;	1943	Hz
.Dw	1269		;	1970	Hz
.Dw	1251		;	1997	Hz
.Dw	1234		;	2024	Hz
.Dw	1218		;	2052	Hz
.Dw	1201		;	2081	Hz
.Dw	1185		;	2109	Hz
.Dw	1169		;	2138	Hz
.Dw	1153		;	2168	Hz
.Dw	1137		;	2198	Hz
.Dw	1121		;	2229	Hz
.Dw	1106		;	2260	Hz
.Dw	1091		;	2291	Hz
.Dw	1076		;	2323	Hz
.Dw	1061		;	2356	Hz
.Dw	1046		;	2389	Hz
.Dw	1031		;	2423	Hz
.Dw	1017		;	2457	Hz
.Dw	1003		;	2491	Hz
.Dw	989			;	2527	Hz
.Dw	975			;	2563	Hz
.Dw	961			;	2599	Hz
.Dw	948			;	2636	Hz
.Dw	934			;	2674	Hz
.Dw	921			;	2712	Hz
.Dw	908			;	2751	Hz
.Dw	895			;	2791	Hz
.Dw	882			;	2831	Hz
.Dw	870			;	2872	Hz
.Dw	857			;	2914	Hz
.Dw	845			;	2957	Hz
.Dw	833			;	3000	Hz

ACSMLookupEnd:


; *****************************************************************************
; Character definitions for the Character Generator RAM of the LCD
;
EEProm_Char_def_start:	

; Character generator position 0
; Compressed dB symbol
;
.db 6, 5, 5, 14, 21, 21, 14, 0

; Character generator position 1
; UP Down arrow
;
.db 0b00000010, 0b00000101, 0b00000000, 0b00000000, 0b00000000, 0b00000101, 0b00000010, 0b00000000

; Character generator position 2
; S meter character #1 (1 line)
;
.db 0b00010000, 0b00010000, 0b00010000, 0b00010000, 0b00010000, 0b00010000, 0b00010000, 0b00010000;
;
; Character generator position 3
; S meter character #2 (2 lines)
;
.db 0b00011000, 0b00011000, 0b00011000, 0b00011000, 0b00011000, 0b00011000, 0b00011000, 0b00011000
;
; Character generator position 4
; S meter character #3 (2 long line and one shorter line for scale effect)
;
.db 0b00011000, 0b00011010, 0b00011010, 0b00011010, 0b00011010, 0b00011010, 0b00011010, 0b00011000

; Character generator position 5
; S meter character #3 (2 long line and two shorter line for scale effect)
;
.db 0b00011000, 0b00011011, 0b00011011, 0b00011011, 0b00011011, 0b00011011, 0b00011011, 0b00011000

; Character generator position 6
; Currently not used, no need to program it or use flash space
;
EEProm_Char_def_1:
; End of fixed additional characters loaded in the Character generator RAM
;
; Following character definitions are dynamic depending on operational parameters
; - Memory /Receiver selected
; - Gain Mode selected
; - ..... (Future)
;
;
EEProm_MemNumDef:
;
; Character generator position 7, lower 4 rows
; Roman number 1

EEProm_MemNum1_Def:
.db  0b00000100, 0b00000100, 0b00000100, 0b00000100
;
; Character generator position 7
; Roman number 2
;
EEProm_MemNum2_Def:
.db 0b00001010, 0b00001010, 0b00001010, 0b00001010
;
; Character generator position 7
; Roman number 3
;
EEProm_MemNum3_Def:
.db 0b00010101, 0b00010101, 0b00010101, 0b00010101
;
; Character generator position 7
; Roman number 4
;
EEProm_MemNum4_Def:
.db  0b00010101, 0b00010101, 0b00010010, 0b00010010
;
; Character generator position 7
;Roman number 5
;
EEProm_MemNum5_Def:
.db 0b00001010, 0b00001010, 0b00000100, 0b00000100
;
; Character generator position 7
;Roman number 6
;
EEProm_MemNum6_Def:
.db  0b00010101, 0b00010101, 0b00001001, 0b00001001
;
;
EEProm_MemDelta1:
.db 0,8,16,8
;
;
EEProm_MeMDelta2:
.db 0,4,8,4
;
;
EEProm_MemDelta3:
.db 0,2,4,2
;
;
EEProm_MemDelta4:
.db 0,9,18,9
;
;
EEProm_MemDelta5:
.db 0,2,1,2
;
;
EEProm_MemDelta6:
.db 0,4,2,4
;
;
EEProm_MemDelta7:
.db 0,8,4,8
;
;
EEProm_MemDelta8:
.db 0,18,9,18
;
;
; Character generator position 7, top 4 rows
EEProm_GainModeDef:

; Blank Gain mode display character, top four rows
;
;EEProm_GainMode0_Def:	
;.db 0, 0, 0,0
;
; Character generator position 7
; DF1FO mode (ARDF) only REDUCING gain automatic, mode 1, symbol <<
;
EEProm_GainMode1_Def:
.db  0b00001001, 0b00010010, 0b00001001, 0
;
; Character generator position 7
; Manual gain mode, adjust gain with encoder, symbol ||
;
EEProm_GainMode2_Def:
.db 0b00001010, 0b00001010, 0b00001010, 0
;
; Character generator position 7
; Auto gain mode, adjust gain up and down, symbol <>
;
EEProm_GainMode3_Def:
.db 0b00001010, 0b00010001, 0b00001010, 0
;
EEProm_Char_def_end:

; *****************************************************************************
; Synthesizer step size lookup table.
; Index in table is # of encoder clicks over the last 500ms
; Value is # of synthesizer channels to change
; Entry 5-16 are a linear curve
;
;	Max # of clicks	40
;	Min # of clicks	2
;	Max Frequency step (kHz)	50.00
;	Size of lookup table (bytes)	16
;	# of 100ms bins	5
;	Delta	38
;	Delta/step	3.166666667




SynthChannelStepsStart:
.db	1
.db	1
.db	1
.db	2
.db	5
.db	8
.db	12
.db	15
.db	18
.db	21
.db	24
.db	27
.db	31
.db	34
.db	37
.db	40

SynthChannelStepsEnd:


